@@ -3,21 +3,18 @@
  * This file is part of the YATE Project http://YATE.null.ro
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include "yatengine.h"
@@ -169,6 +166,7 @@ bool Configuration::load(bool warn)
     FILE *f = ::fopen(c_str(),"r");
     if (f) {
 	String sect;
+	bool start = true;
 	for (;;) {
 	    char buf[1024];
 	    if (!::fgets(buf,sizeof(buf),f))
@@ -181,6 +179,11 @@ bool Configuration::load(bool warn)
 	    if (pc)
 		*pc = 0;
 	    pc = buf;
+	    // skip over an initial UTF-8 BOM
+	    if (start) {
+		String::stripBOM(pc);
+		start = false;
+	    }
 	    while (*pc == ' ' || *pc == '\t')
 		pc++;
 	    switch (*pc) {
