@@ -3,21 +3,18 @@
  * This file is part of the YATE Project http://YATE.null.ro
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include "yateclass.h"
@@ -110,7 +107,7 @@ DataBlock::~DataBlock()
 
 void* DataBlock::getObject(const String& name) const
 {
-    if (name == YSTRING("DataBlock"))
+    if (name == YATOM("DataBlock"))
 	return const_cast<DataBlock*>(this);
     return GenObject::getObject(name);
 }
@@ -382,6 +379,25 @@ bool DataBlock::unHexify(const char* data, unsigned int len, char sep)
     return (iBuf >= n);
 }
 
+// This variant of unHexify automatically detects presence of separators
+bool DataBlock::unHexify(const char* data, unsigned int len)
+{
+    char sep = 0;
+    if (len > 2) {
+	const char* s = " :;.,-/|";
+	while (char c = *s++) {
+	    unsigned int offs = 2;
+	    if (data[0] == c)
+		offs++;
+	    if (len == offs || data[offs] == c) {
+		sep = c;
+		break;
+	    }
+	}
+    }
+    return unHexify(data,len,sep);
+}
+
 String DataBlock::sqlEscape(char extraEsc) const
 {
     unsigned int len = m_length;
