@@ -3,21 +3,18 @@
  * This file is part of the YATE Project http://YATE.null.ro
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include "yatephone.h"
@@ -467,7 +464,7 @@ void DataConsumer::destroyed()
 
 void* DataConsumer::getObject(const String& name) const
 {
-    if (name == YSTRING("DataConsumer"))
+    if (name == YATOM("DataConsumer"))
 	return const_cast<DataConsumer*>(this);
     return DataNode::getObject(name);
 }
@@ -692,7 +689,7 @@ void DataSource::synchronize(unsigned long tStamp)
 
 void* DataSource::getObject(const String& name) const
 {
-    if (name == YSTRING("DataSource"))
+    if (name == YATOM("DataSource"))
 	return const_cast<DataSource*>(this);
     return DataNode::getObject(name);
 }
@@ -725,7 +722,7 @@ void DataEndpoint::destroyed()
 
 void* DataEndpoint::getObject(const String& name) const
 {
-    if (name == YSTRING("DataEndpoint"))
+    if (name == YATOM("DataEndpoint"))
 	return const_cast<DataEndpoint*>(this);
     return RefObject::getObject(name);
 }
@@ -1055,6 +1052,7 @@ bool DataEndpoint::clearData(DataNode* node)
 // Change source(s) or consumer(s)
 bool DataEndpoint::control(NamedList& params)
 {
+    // TODO how do we handle this case????? operation-status
     return (m_source && m_source->control(params)) ||
 	(m_consumer && m_consumer->control(params)) ||
 	(m_peerRecord && m_peerRecord->control(params)) ||
@@ -1152,7 +1150,7 @@ DataTranslator::~DataTranslator()
 
 void* DataTranslator::getObject(const String& name) const
 {
-    if (name == YSTRING("DataTranslator"))
+    if (name == YATOM("DataTranslator"))
 	return const_cast<DataTranslator*>(this);
     return DataConsumer::getObject(name);
 }
