@@ -3,23 +3,20 @@
  * This file is part of the YATE Project http://YATE.null.ro
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
  * Idea and initial implementation (as HashTable) by Maciek Kaminski
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include "yateclass.h"
@@ -48,7 +45,7 @@ HashList::~HashList()
 
 void* HashList::getObject(const String& name) const
 {
-    if (name == YSTRING("HashList"))
+    if (name == YATOM("HashList"))
 	return const_cast<HashList*>(this);
     return GenObject::getObject(name);
 }
@@ -73,7 +70,19 @@ ObjList* HashList::find(const GenObject* obj) const
     XDebug(DebugAll,"HashList::find(%p) [%p]",obj,this);
     if (!obj)
 	return 0;
-    unsigned int i = obj->toString().hash() % m_size;
+    ObjList* found = 0;
+    for (unsigned int i = 0; !found && i < m_size; i++)
+	if (m_lists[i])
+	    found = m_lists[i]->find(obj);
+    return found;
+}
+
+ObjList* HashList::find(const GenObject* obj, unsigned int hash) const
+{
+    XDebug(DebugAll,"HashList::find(%p,%u) [%p]",obj,hash,this);
+    if (!obj)
+	return 0;
+    unsigned int i = hash % m_size;
     return m_lists[i] ? m_lists[i]->find(obj) : 0;
 }
 
