@@ -3,21 +3,18 @@
  * This file is part of the YATE Project http://YATE.null.ro
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include "yateclass.h"
@@ -25,13 +22,13 @@
 using namespace TelEngine;
 
 ListIterator::ListIterator(ObjList& list, int offset)
-    : m_objects(0)
+    : m_objects(0), m_hashes(0)
 {
     assign(list,offset);
 }
 
 ListIterator::ListIterator(HashList& list, int offset)
-    : m_objects(0)
+    : m_objects(0), m_hashes(0)
 {
     assign(list,offset);
 }
@@ -40,6 +37,8 @@ ListIterator::~ListIterator()
 {
     m_length = 0;
     delete[] m_objects;
+    if (m_hashes)
+	delete[] m_hashes;
 }
 
 void ListIterator::clear()
@@ -51,6 +50,11 @@ void ListIterator::clear()
     GenObject** tmp = m_objects;
     m_objects = 0;
     delete[] tmp;
+    if (m_hashes) {
+	unsigned int* tmph = m_hashes;
+	m_hashes = 0;
+	delete[] tmph;
+    }
 }
 
 void ListIterator::assign(ObjList& list, int offset)
@@ -80,6 +84,7 @@ void ListIterator::assign(HashList& list, int offset)
     if (!m_length)
 	return;
     m_objects = new GenObject* [m_length];
+    m_hashes = new unsigned int[m_length];
     offset = (m_length - offset) % m_length;
     unsigned int i = 0;
     for (unsigned int n = 0; n < list.length(); n++) {
@@ -89,7 +94,9 @@ void ListIterator::assign(HashList& list, int offset)
 	for (l = l->skipNull(); i < m_length; l = l->skipNext()) {
 	    if (!l)
 		break;
-	    m_objects[((i++) + offset) % m_length] = l->get();
+	    unsigned int idx = ((i++) + offset) % m_length;
+	    m_objects[idx] = l->get();
+	    m_hashes[idx] = l->get()->toString().hash();
 	}
     }
     while (i < m_length)
@@ -108,7 +115,7 @@ GenObject* ListIterator::get(unsigned int index) const
 	    return obj;
     }
     else if (m_hashList) {
-	if (m_hashList->find(obj) && obj->alive())
+	if (m_hashList->find(obj,m_hashes[index]) && obj->alive())
 	    return obj;
     }
     return 0;
