@@ -3,21 +3,18 @@
  * This file is part of the YATE Project http://YATE.null.ro
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include "yatengine.h"
@@ -25,6 +22,19 @@
 
 using namespace TelEngine;
 
+class QueueWorker : public GenObject, public Thread
+{
+public:
+    inline QueueWorker(MessageQueue* queue)
+	: Thread("MessageQueueWorker"),m_queue(queue)
+	{}
+    virtual ~QueueWorker();
+protected:
+    virtual void run();
+private:
+    RefPointer<MessageQueue> m_queue;
+};
+
 Message::Message(const char* name, const char* retval, bool broadcast)
     : NamedList(name),
       m_return(retval), m_data(0), m_notify(false), m_broadcast(broadcast)
@@ -58,7 +68,7 @@ Message::~Message()
 
 void* Message::getObject(const String& name) const
 {
-    if (name == YSTRING("Message"))
+    if (name == YATOM("Message"))
 	return const_cast<Message*>(this);
     return NamedList::getObject(name);
 }
@@ -286,7 +296,10 @@ bool MessageRelay::receivedInternal(Message& msg)
 
 MessageDispatcher::MessageDispatcher(const char* trackParam)
     : Mutex(false,"MessageDispatcher"),
-      m_trackParam(trackParam), m_changes(0), m_warnTime(0)
+      m_hookMutex(false,"PostHooks"),
+      m_msgAppend(&m_messages), m_hookAppend(&m_hooks),
+      m_trackParam(trackParam), m_changes(0), m_warnTime(0),
+      m_hookCount(0), m_hookHole(false)
 {
     XDebug(DebugInfo,"MessageDispatcher::MessageDispatcher('%s') [%p]",trackParam,this);
 }
@@ -367,9 +380,9 @@ bool MessageDispatcher::dispatch(Message& msg)
 #ifdef XDEBUG
     Debugger debug("MessageDispatcher::dispatch","(%p) (\"%s\")",&msg,msg.c_str());
 #endif
-#ifndef NDEBUG
-    u_int64_t t = Time::now();
-#endif
+
+    u_int64_t t = m_warnTime ? Time::now() : 0;
+
     bool retv = false;
     ObjList *l = &m_handlers;
     Lock mylock(this);
@@ -390,16 +403,22 @@ bool MessageDispatcher::dispatch(Message& msg)
 	    // mark handler as unsafe to destroy / uninstall
 	    h->m_unsafe++;
 	    mylock.drop();
-#ifdef DEBUG
-	    u_int64_t tm = Time::now();
-#endif
+
+	    u_int64_t tm = m_warnTime ? Time::now() : 0;
+
 	    retv = h->receivedInternal(msg) || retv;
-#ifdef DEBUG
-	    tm = Time::now() - tm;
-	    if (m_warnTime && (tm > m_warnTime))
-		Debug(DebugInfo,"Message '%s' [%p] passed through %p in " FMT64U " usec",
-		    msg.c_str(),&msg,h,tm);
-#endif
+
+	    if (tm) {
+		tm = Time::now() - tm;
+		if (tm > m_warnTime) {
+		    mylock.acquire(this);
+		    const char* name = (c == m_changes) ? h->trackName().c_str() : 0;
+		    Debug(DebugInfo,"Message '%s' [%p] passed through %p%s%s%s in " FMT64U " usec",
+			msg.c_str(),&msg,h,
+			(name ? " '" : ""),(name ? name : ""),(name ? "'" : ""),tm);
+		}
+	    }
+
 	    if (retv && !msg.broadcast())
 		break;
 	    mylock.acquire(this);
@@ -433,26 +452,50 @@ bool MessageDispatcher::dispatch(Message& msg)
     }
     mylock.drop();
     msg.dispatched(retv);
-#ifndef NDEBUG
-    t = Time::now() - t;
-    if (m_warnTime && (t > m_warnTime)) {
-	unsigned n = msg.length();
-	String p;
-	for (unsigned i = 0; i < n; i++) {
-	    NamedString *s = msg.getParam(i);
-	    if (s)
-		p << "\n  ['" << s->name() << "']='" << *s << "'";
+
+    if (t) {
+	t = Time::now() - t;
+	if (t > m_warnTime) {
+	    unsigned n = msg.length();
+	    String p;
+	    p << "\r\n  retval='" << msg.retValue().safe("(null)") << "'";
+	    for (unsigned i = 0; i < n; i++) {
+		NamedString *s = msg.getParam(i);
+		if (s)
+		    p << "\r\n  param['" << s->name() << "'] = '" << *s << "'";
+	    }
+	    Debug("Performance",DebugMild,"Message %p '%s' returned %s in " FMT64U " usec%s",
+		&msg,msg.c_str(),retv ? "true" : "false",t,p.safe());
 	}
-	Debug("Performance",DebugMild,"Message %p '%s' retval '%s' returned %s in " FMT64U " usec%s",
-	    &msg,msg.c_str(),msg.retValue().c_str(),retv ? "true" : "false",t,p.safe());
     }
-#endif
-    l = &m_hooks;
-    for (; l; l=l->next()) {
-	MessagePostHook *h = static_cast<MessagePostHook*>(l->get());
-	if (h)
-	    h->dispatched(msg,retv);
+
+    m_hookMutex.lock();
+    if (m_hookHole && !m_hookCount) {
+	// compact the list, remove the holes
+	for (l = &m_hooks; l; l = l->next()) {
+	    while (!l->get()) {
+		if (!l->next())
+		    break;
+		if (l->next() == m_hookAppend)
+		    m_hookAppend = &m_hooks;
+		l->remove();
+	    }
+	}
+	m_hookHole = false;
     }
+    m_hookCount++;
+    for (l = m_hooks.skipNull(); l; l = l->skipNext()) {
+	RefPointer<MessagePostHook> ph = static_cast<MessagePostHook*>(l->get());
+	if (ph) {
+	    m_hookMutex.unlock();
+	    ph->dispatched(msg,retv);
+	    ph = 0;
+	    m_hookMutex.lock();
+	}
+    }
+    m_hookCount--;
+    m_hookMutex.unlock();
+
     return retv;
 }
 
@@ -461,13 +504,15 @@ bool MessageDispatcher::enqueue(Message* msg)
     Lock lock(this);
     if (!msg || m_messages.find(msg))
 	return false;
-    m_messages.append(msg);
+    m_msgAppend = m_msgAppend->append(msg);
     return true;
 }
 
 bool MessageDispatcher::dequeueOne()
 {
     lock();
+    if (m_messages.next() == m_msgAppend)
+	m_msgAppend = &m_messages;
     Message* msg = static_cast<Message *>(m_messages.remove(false));
     unlock();
     if (!msg)
@@ -495,14 +540,26 @@ unsigned int MessageDispatcher::handlerCount()
     return m_handlers.count();
 }
 
+unsigned int MessageDispatcher::postHookCount()
+{
+    Lock lock(m_hookMutex);
+    return m_hooks.count();
+}
+
 void MessageDispatcher::setHook(MessagePostHook* hook, bool remove)
 {
-    lock();
-    if (remove)
-	m_hooks.remove(hook,false);
+    m_hookMutex.lock();
+    if (remove) {
+	// zero the hook, we'll compact it later when safe
+	ObjList* l = m_hooks.find(hook);
+	if (l) {
+	    l->set(0,false);
+	    m_hookHole = true;
+	}
+    }
     else
-	m_hooks.append(hook);
-    unlock();
+	m_hookAppend = m_hookAppend->append(hook);
+    m_hookMutex.unlock();
 }
 
 
@@ -510,4 +567,134 @@ MessageNotifier::~MessageNotifier()
 {
 }
 
+/**
+ * class MessageQueue
+ */
+
+static const char* s_queueMutexName = "MessageQueue";
+
+MessageQueue::MessageQueue(const char* queueName, int numWorkers)
+    : Mutex(true,s_queueMutexName), m_filters(queueName), m_count(0)
+{
+    XDebug(DebugAll,"Creating MessageQueue for %s",queueName);
+    for (int i = 0;i < numWorkers;i ++) {
+	QueueWorker* worker = new QueueWorker(this);
+	worker->startup();
+	m_workers.append(worker);
+    }
+    m_append = &m_messages;
+}
+
+void MessageQueue::received(Message& msg)
+{
+    Engine::dispatch(msg);
+}
+
+MessageQueue::~MessageQueue()
+{
+    XDebug(DebugAll,"Destroying MessageQueue for %s",m_filters.c_str());
+}
+
+void MessageQueue::clear()
+{
+    Lock myLock(this);
+    for (ObjList* o = m_workers.skipNull();o;o = o->skipNext()) {
+	QueueWorker* worker = static_cast<QueueWorker*>(o->get());
+	worker->cancel();
+	o->setDelete(false);
+    }
+    m_workers.clear();
+    m_messages.clear();
+}
+
+bool MessageQueue::enqueue(Message* msg)
+{
+    if (!msg)
+	return false;
+    Lock myLock(this);
+    m_append = m_append->append(msg);
+    m_count++;
+    return true;
+}
+
+bool MessageQueue::dequeue()
+{
+    Lock myLock(this);
+    ObjList* o = m_messages.skipNull();
+    if (!o)
+	return false;
+    if (m_messages.next() == m_append)
+	m_append = &m_messages;
+    Message* msg = static_cast<Message*>(m_messages.remove(false));
+    if (!msg)
+	return false;
+    m_count--;
+    myLock.drop();
+    received(*msg);
+    TelEngine::destruct(msg);
+    return true;
+}
+
+void MessageQueue::addFilter(const char* name, const char* value)
+{
+    Lock myLock(this);
+    m_filters.setParam(name,value);
+}
+
+void MessageQueue::removeFilter(const String& name)
+{
+    Lock myLock(this);
+    m_filters.clearParam(name);
+}
+
+bool MessageQueue::matchesFilter(const Message& msg)
+{
+    Lock myLock(this);
+    if (msg != m_filters)
+	return false;
+    for (unsigned int i = 0;i < m_filters.length();i++) {
+	NamedString* param = m_filters.getParam(i);
+	if (!param)
+	    continue;
+	NamedString* match = msg.getParam(param->name());
+	if (!match || *match != *param)
+	    return false;
+    }
+    return true;
+}
+
+void MessageQueue::removeThread(Thread* thread)
+{
+    if (!thread)
+	return;
+    Lock myLock(this);
+    m_workers.remove((GenObject*)thread,false);
+}
+
+/**
+ * class QueueWorker
+ */
+
+QueueWorker::~QueueWorker()
+{
+    if (m_queue)
+	m_queue->removeThread(this);
+    m_queue = 0;
+}
+
+void QueueWorker::run()
+{
+    if (!m_queue)
+	return;
+    while (true) {
+	if (!m_queue->count()) {
+	    Thread::idle(true);
+	    continue;
+	}
+	m_queue->dequeue();
+	Thread::check(true);
+    }
+}
+
+
 /* vi: set ts=8 sw=4 sts=4 noet: */
