@@ -3,21 +3,18 @@
  * This file is part of the YATE Project http://YATE.null.ro 
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include <stdlib.h>
@@ -94,7 +91,7 @@ MimeHeaderLine::~MimeHeaderLine()
 
 void* MimeHeaderLine::getObject(const String& name) const
 {
-    if (name == YSTRING("MimeHeaderLine"))
+    if (name == YATOM("MimeHeaderLine"))
 	return const_cast<MimeHeaderLine*>(this);
     return NamedString::getObject(name);
 }
@@ -104,9 +101,11 @@ MimeHeaderLine* MimeHeaderLine::clone(const char* newName) const
     return new MimeHeaderLine(*this,newName);
 }
 
-void MimeHeaderLine::buildLine(String& line) const
+void MimeHeaderLine::buildLine(String& line, bool header) const
 {
-    line << name() << ": " << *this;
+    if (header)
+	line << name() << ": ";
+    line << *this;
     const ObjList* p = &m_params;
     for (; p; p = p->next()) {
 	NamedString* s = static_cast<NamedString*>(p->get());
@@ -316,7 +315,7 @@ MimeAuthLine::MimeAuthLine(const MimeAuthLine& original, const char* newName)
 
 void* MimeAuthLine::getObject(const String& name) const
 {
-    if (name == YSTRING("MimeAuthLine"))
+    if (name == YATOM("MimeAuthLine"))
 	return const_cast<MimeAuthLine*>(this);
     return MimeHeaderLine::getObject(name);
 }
@@ -326,9 +325,11 @@ MimeHeaderLine* MimeAuthLine::clone(const char* newName) const
     return new MimeAuthLine(*this,newName);
 }
 
-void MimeAuthLine::buildLine(String& line) const
+void MimeAuthLine::buildLine(String& line, bool header) const
 {
-    line << name() << ": " << *this;
+    if (header)
+	line << name() << ": ";
+    line << *this;
     const ObjList* p = &m_params;
     for (bool first = true; p; p = p->next()) {
 	NamedString* s = static_cast<NamedString*>(p->get());
@@ -592,7 +593,7 @@ MimeMultipartBody::MimeMultipartBody(const MimeMultipartBody& original)
 // Find object by class name, descend into parts
 void* MimeMultipartBody::getObject(const String& name) const
 {
-    if (name == YSTRING("MimeMultipartBody"))
+    if (name == YATOM("MimeMultipartBody"))
 	return const_cast<MimeMultipartBody*>(this);
     void* res = MimeBody::getObject(name);
     if (res)
