@@ -3,21 +3,18 @@
  * This file is part of the YATE Project http://YATE.null.ro
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include "yateclass.h"
@@ -131,7 +128,7 @@ HMUTEX GlobalMutex::s_mutex;
 static GlobalMutex s_global;
 static unsigned long s_maxwait = 0;
 static bool s_unsafe = MUTEX_STATIC_UNSAFE;
-static bool s_safety = true;
+static bool s_safety = false;
 
 volatile int MutexPrivate::s_count = 0;
 volatile int MutexPrivate::s_locks = 0;
@@ -250,12 +247,13 @@ bool MutexPrivate::lock(long maxwait)
 	maxwait = (long)s_maxwait;
 	warn = true;
     }
-    if (s_safety)
+    bool safety = s_safety;
+    if (safety)
 	GlobalMutex::lock();
     Thread* thr = Thread::current();
     if (thr)
 	thr->m_locking = true;
-    if (s_safety) {
+    if (safety) {
 	m_waiting++;
 	GlobalMutex::unlock();
     }
@@ -299,14 +297,14 @@ bool MutexPrivate::lock(long maxwait)
 #endif // HAVE_TIMEDLOCK
     }
 #endif // _WINDOWS
-    if (s_safety) {
+    if (safety) {
 	GlobalMutex::lock();
 	m_waiting--;
     }
     if (thr)
 	thr->m_locking = false;
     if (rval) {
-	if (s_safety)
+	if (safety)
 	    s_locks++;
 	m_locked++;
 	if (thr) {
@@ -316,7 +314,7 @@ bool MutexPrivate::lock(long maxwait)
 	else
 	    m_owner = 0;
     }
-    if (s_safety)
+    if (safety)
 	GlobalMutex::unlock();
     if (warn && !rval)
 	Debug(DebugFail,"Thread '%s' could not lock mutex '%s' owned by '%s' waited by %u others for %lu usec!",
@@ -328,7 +326,8 @@ bool MutexPrivate::unlock()
 {
     bool ok = false;
     // Hope we don't hit a bug related to the debug mutex!
-    if (s_safety)
+    bool safety = s_safety;
+    if (safety)
 	GlobalMutex::lock();
     if (m_locked) {
 	Thread* thr = Thread::current();
@@ -341,7 +340,7 @@ bool MutexPrivate::unlock()
 		    m_name,tname,m_owner,this);
 	    m_owner = 0;
 	}
-	if (s_safety) {
+	if (safety) {
 	    int locks = --s_locks;
 	    if (locks < 0) {
 		// this is very very bad - abort right now
@@ -360,7 +359,7 @@ bool MutexPrivate::unlock()
     }
     else
 	Debug(DebugFail,"MutexPrivate::unlock called on unlocked '%s' [%p]",m_name,this);
-    if (s_safety)
+    if (safety)
 	GlobalMutex::unlock();
     return ok;
 }
@@ -404,12 +403,13 @@ bool SemaphorePrivate::lock(long maxwait)
 	maxwait = (long)s_maxwait;
 	warn = true;
     }
-    if (s_safety)
+    bool safety = s_safety;
+    if (safety)
 	GlobalMutex::lock();
     Thread* thr = Thread::current();
     if (thr)
 	thr->m_locking = true;
-    if (s_safety) {
+    if (safety) {
 	s_locks++;
 	m_waiting++;
 	GlobalMutex::unlock();
@@ -454,7 +454,7 @@ bool SemaphorePrivate::lock(long maxwait)
 #endif // HAVE_TIMEDWAIT
     }
 #endif // _WINDOWS
-    if (s_safety) {
+    if (safety) {
 	GlobalMutex::lock();
 	int locks = --s_locks;
 	if (locks < 0) {
@@ -467,7 +467,7 @@ bool SemaphorePrivate::lock(long maxwait)
     }
     if (thr)
 	thr->m_locking = false;
-    if (s_safety)
+    if (safety)
 	GlobalMutex::unlock();
     if (warn && !rval)
 	Debug(DebugFail,"Thread '%s' could not lock semaphore '%s' waited by %u others for %lu usec!",
@@ -478,7 +478,8 @@ bool SemaphorePrivate::lock(long maxwait)
 bool SemaphorePrivate::unlock()
 {
     if (!s_unsafe) {
-	if (s_safety)
+	bool safety = s_safety;
+	if (safety)
 	    GlobalMutex::lock();
 #ifdef _WINDOWS
 	::ReleaseSemaphore(m_semaphore,1,NULL);
@@ -487,7 +488,7 @@ bool SemaphorePrivate::unlock()
 	if (!::sem_getvalue(&m_semaphore,&val) && (val < (int)m_maxcount))
 	    ::sem_post(&m_semaphore);
 #endif
-	if (s_safety)
+	if (safety)
 	    GlobalMutex::unlock();
     }
     return true;
@@ -521,9 +522,9 @@ void Lockable::startUsingNow()
     s_unsafe = false;
 }
 
-void Lockable::disableSafety()
+void Lockable::enableSafety(bool safe)
 {
-    s_safety = false;
+    s_safety = safe;
 }
 
 void Lockable::wait(unsigned long maxwait)
@@ -607,7 +608,7 @@ int Mutex::count()
 
 int Mutex::locks()
 {
-    return MutexPrivate::s_locks;
+    return s_safety ? MutexPrivate::s_locks : -1;
 }
 
 bool Mutex::efficientTimedLock()
@@ -706,7 +707,7 @@ int Semaphore::count()
 
 int Semaphore::locks()
 {
-    return SemaphorePrivate::s_locks;
+    return s_safety ? SemaphorePrivate::s_locks : -1;
 }
 
 bool Semaphore::efficientTimedLock()
