@@ -3,21 +3,18 @@
  * This file is part of the YATE Project http://YATE.null.ro
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include "yateclass.h"
@@ -45,7 +42,7 @@ ObjList::~ObjList()
 
 void* ObjList::getObject(const String& name) const
 {
-    if (name == YSTRING("ObjList"))
+    if (name == YATOM("ObjList"))
 	return const_cast<ObjList*>(this);
     return GenObject::getObject(name);
 }
@@ -263,6 +260,140 @@ void ObjList::clear()
     TelEngine::destruct(n);
 }
 
+static inline ObjList* skipLastEmpty(ObjList* list)
+{
+    ObjList* l = 0;
+    for (; list; list = list->next()) {
+	if (list->get())
+	    break;
+	l = list;
+    }
+    return l;
+}
+
+// Remove all empty objects in the list
+void ObjList::compact()
+{
+    if (!m_next)
+	return;
+    // Compact list head
+    if (!get()) {
+	ObjList* lastEmpty = skipLastEmpty(this);
+	if (!lastEmpty->next()) {
+	    clear();
+	    return;
+	}
+	ObjList* nonEmpty = lastEmpty->next();
+	ObjList* tmp = m_next;
+	m_next = nonEmpty->next();
+	m_obj = nonEmpty->get();
+	m_delete = nonEmpty->m_delete;
+	nonEmpty->m_obj = 0;
+	nonEmpty->m_next = 0;
+	tmp->destruct();
+    }
+    ObjList* last = this;
+    while (last->next()) {
+	// Find last non empty item
+	for (ObjList* o = last->next(); o; o = o->next()) {
+	    if (o->get())
+		last = o;
+	    else
+		break;
+	}
+	if (!last->next())
+	    break;
+	// Next item after last is empty, find the last empty
+	ObjList* lastEmpty = skipLastEmpty(last->next());
+	if (!lastEmpty->next())
+	    break;
+	ObjList* firstNonEmpty = lastEmpty->next();
+	lastEmpty->m_next = 0;
+	ObjList* tmp = last->m_next;
+	last->m_next = firstNonEmpty;
+	last = firstNonEmpty;
+	tmp->destruct();
+    }
+    if (last && last->next()) {
+	ObjList* tmp = last->next();
+	last->m_next = 0;
+	tmp->destruct();
+    }
+}
+
+static void merge(ObjList* first, ObjList* second,int (*compare)(GenObject* obj1, GenObject* obj2, void* data), void* dat)
+{
+    if (!(first && second))
+	return;
+    if (!first->skipNull()) {
+	while (second->skipNull())
+	    first->append(second->remove(false));
+	return;
+    }
+    ObjList* head = first->skipNull();
+    GenObject* current = head->get();
+    while (second->skipNull()) {
+	GenObject* next = second->remove(false);
+	while (current && compare(current,next,dat) < 1) {
+	    if (!head->skipNext()) {
+		current = 0;
+		break;
+	    }
+	    head = head->skipNext();
+	    current = head->get();
+	}
+	if (!current) {
+	    first->append(next);
+	    continue;
+	}
+	head->insert(next);
+	head = head->skipNext();
+    }
+}
+
+static void splitList(ObjList& list, ObjList& splits, int (*compare)(GenObject* obj1, GenObject* obj2, void* data), void* data)
+{
+    if (!list.skipNull())
+	return;
+    ObjList* slice = new ObjList();
+    splits.append(slice);
+    GenObject* last = list.remove(false);
+    slice->append(last);
+    while (list.skipNull()) {
+	GenObject* next = list.remove(false);
+	if (compare(last,next,data) < 1) {
+	    slice->append(next);
+	    last = next;
+	    continue;
+	}
+	slice = new ObjList();
+	slice->append(next);
+	splits.append(slice);
+	last = next;
+    }
+}
+
+void ObjList::sort(int (*callbackCompare)(GenObject* obj1, GenObject* obj2, void* data), void* data)
+{
+    if (!callbackCompare) {
+	Debug(DebugNote,"ObjList::sort called without callback method!");
+	return;
+    }
+    ObjList splits;
+    splitList(*this,splits,callbackCompare,data);
+    while (splits.skipNull()) {
+	ObjList* first = this;
+	for (ObjList* o = splits.skipNull();o;o = o->skipNext()) {
+	    ObjList* second = static_cast<ObjList*>(o->get());
+	    merge(first,second,callbackCompare,data);
+	    o->remove();
+	    o = o->skipNull();
+	    if (!o)
+		break;
+	    first = static_cast<ObjList*>(o->get());
+	}
+    }
+}
 
 ObjVector::ObjVector(unsigned int maxLen, bool autodelete)
     : m_length(maxLen), m_objects(0), m_delete(autodelete)
@@ -292,7 +423,7 @@ ObjVector::~ObjVector()
 
 void* ObjVector::getObject(const String& name) const
 {
-    if (name == YSTRING("ObjVector"))
+    if (name == YATOM("ObjVector"))
 	return const_cast<ObjVector*>(this);
     return GenObject::getObject(name);
 }
@@ -335,6 +466,16 @@ unsigned int ObjVector::count() const
     return c;
 }
 
+bool ObjVector::null() const
+{
+    if (!m_objects)
+	return true;
+    for (unsigned int i = 0; i < m_length; i++)
+	if (m_objects[i])
+	    return false;
+    return true;
+}
+
 int ObjVector::index(const GenObject* obj) const
 {
     if (!m_objects)
@@ -390,7 +531,7 @@ void ObjVector::clear()
 	for (unsigned int i = 0; i < len; i++)
 	    TelEngine::destruct(objs[i]);
     }
-    delete objs;
+    delete[] objs;
 }
 
 /* vi: set ts=8 sw=4 sts=4 noet: */
