@@ -3,21 +3,18 @@
  * This file is part of the YATE Project http://YATE.null.ro
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include "yateclass.h"
@@ -99,7 +96,6 @@ const char* lookup(int value, const TokenDict* tokens, const char* defvalue)
 }
 
 #define MAX_MATCH 9
-#define INIT_HASH ((unsigned)-1)
 
 class StringMatchPrivate
 {
@@ -140,6 +136,122 @@ static inline char hexEncode(char nib)
 }
 
 
+void UChar::encode()
+{
+    if (m_chr < 0x80) {
+	m_str[0] = (char)m_chr;
+	m_str[1] = '\0';
+    }
+    else if (m_chr < 0x800) {
+	m_str[0] = (char)(0xc0 | ((m_chr >>  6) & 0x1f));
+	m_str[1] = (char)(0x80 | (m_chr & 0x3f));
+	m_str[2] = '\0';
+    }
+    else if (m_chr < 0xffff) {
+	m_str[0] = (char)(0xe0 | ((m_chr >> 12) & 0x0f));
+	m_str[1] = (char)(0x80 | ((m_chr >>  6) & 0x3f));
+	m_str[2] = (char)(0x80 | (m_chr & 0x3f));
+	m_str[3] = '\0';
+    }
+    else if (m_chr < 0x1fffff) {
+	m_str[0] = (char)(0xf0 | ((m_chr >> 18) & 0x07));
+	m_str[1] = (char)(0x80 | ((m_chr >> 12) & 0x3f));
+	m_str[2] = (char)(0x80 | ((m_chr >>  6) & 0x3f));
+	m_str[3] = (char)(0x80 | (m_chr & 0x3f));
+	m_str[4] = '\0';
+    }
+    else if (m_chr < 0x3ffffff) {
+	m_str[0] = (char)(0xf8 | ((m_chr >> 24) & 0x03));
+	m_str[1] = (char)(0x80 | ((m_chr >> 18) & 0x3f));
+	m_str[2] = (char)(0x80 | ((m_chr >> 12) & 0x3f));
+	m_str[3] = (char)(0x80 | ((m_chr >>  6) & 0x3f));
+	m_str[4] = (char)(0x80 | (m_chr & 0x3f));
+	m_str[5] = '\0';
+    }
+    else if (m_chr < 0x7fffffff) {
+	m_str[0] = (char)(0xfc | ((m_chr >> 30) & 0x01));
+	m_str[1] = (char)(0x80 | ((m_chr >> 24) & 0x3f));
+	m_str[2] = (char)(0x80 | ((m_chr >> 18) & 0x3f));
+	m_str[3] = (char)(0x80 | ((m_chr >> 12) & 0x3f));
+	m_str[4] = (char)(0x80 | ((m_chr >>  6) & 0x3f));
+	m_str[5] = (char)(0x80 | (m_chr & 0x3f));
+	m_str[6] = '\0';
+    }
+    else
+	m_str[0] = '\0';
+}
+
+bool UChar::decode(const char*& str, uint32_t maxChar, bool overlong)
+{
+    operator=('\0');
+    if (!str)
+	return false;
+    if (maxChar < 128)
+	maxChar = 0x10ffff; // RFC 3629 default limit
+
+    unsigned int more = 0;
+    uint32_t min = 0;
+    uint32_t val = 0;
+
+    unsigned char c = (unsigned char)*str++;
+    // from 1st byte we find out how many are supposed to follow
+    if (!c)           // don't advance past NUL
+	--str;
+    else if (c < 0x80) // 1 byte, 0...0x7F, ASCII characters
+	val = c & 0x7f;
+    else if (c < 0xc0) // invalid as first UFT-8 byte
+	return false;
+    else if (c < 0xe0) {
+	// 2 bytes, 0x80...0x7FF
+	min = 0x80;
+	val = c & 0x1f;
+	more = 1;
+    }
+    else if (c < 0xf0) {
+	// 3 bytes, 0x800...0xFFFF, Basic Multilingual Plane
+	min = 0x800;
+	val = c & 0x0f;
+	more = 2;
+    }
+    else if (c < 0xf8) {
+	// 4 bytes, 0x10000...0x1FFFFF, RFC 3629 limit (10FFFF)
+	min = 0x10000;
+	val = c & 0x07;
+	more = 3;
+    }
+    else if (c < 0xfc) {
+	// 5 bytes, 0x200000...0x3FFFFFF
+	min = 0x200000;
+	val = c & 0x03;
+	more = 4;
+    }
+    else if (c < 0xfe) {
+	// 6 bytes, 0x4000000...0x7FFFFFFF
+	min = 0x4000000;
+	val = c & 0x01;
+	more = 5;
+    }
+    else
+	return false;
+
+    while (more--) {
+	c = (unsigned char)*str;
+	// all continuation bytes are in range [128..191]
+	if ((c & 0xc0) != 0x80)
+	    return false;
+	val = (val << 6) | (c & 0x3f);
+	++str;
+    }
+    operator=(val);
+    // got full value, check for overlongs and out of range
+    if (val > maxChar)
+	return false;
+    if (val < min && !overlong)
+	return false;
+    return true;
+}
+
+
 StringMatchPrivate::StringMatchPrivate()
 {
     XDebug(DebugAll,"StringMatchPrivate::StringMatchPrivate() [%p]",this);
@@ -182,6 +294,8 @@ void StringMatchPrivate::fixup()
 
 
 static const String s_empty;
+static ObjList s_atoms;
+static Mutex s_mutex(false,"Atom");
 
 const String& String::empty()
 {
@@ -189,13 +303,13 @@ const String& String::empty()
 }
 
 String::String()
-    : m_string(0), m_length(0), m_hash(INIT_HASH), m_matches(0)
+    : m_string(0), m_length(0), m_hash(YSTRING_INIT_HASH), m_matches(0)
 {
     XDebug(DebugAll,"String::String() [%p]",this);
 }
 
 String::String(const char* value, int len)
-    : m_string(0), m_length(0), m_hash(INIT_HASH), m_matches(0)
+    : m_string(0), m_length(0), m_hash(YSTRING_INIT_HASH), m_matches(0)
 {
     XDebug(DebugAll,"String::String(\"%s\",%d) [%p]",value,len,this);
     assign(value,len);
@@ -203,19 +317,21 @@ String::String(const char* value, int len)
 
 String::String(const String& value)
     : GenObject(),
-      m_string(0), m_length(0), m_hash(INIT_HASH), m_matches(0)
+      m_string(0), m_length(0), m_hash(YSTRING_INIT_HASH), m_matches(0)
 {
     XDebug(DebugAll,"String::String(%p) [%p]",&value,this);
     if (!value.null()) {
 	m_string = ::strdup(value.c_str());
 	if (!m_string)
 	    Debug("String",DebugFail,"strdup() returned NULL!");
+	else
+	    m_length = value.length();
 	changed();
     }
 }
 
 String::String(char value, unsigned int repeat)
-    : m_string(0), m_length(0), m_hash(INIT_HASH), m_matches(0)
+    : m_string(0), m_length(0), m_hash(YSTRING_INIT_HASH), m_matches(0)
 {
     XDebug(DebugAll,"String::String('%c',%d) [%p]",value,repeat,this);
     if (value && repeat) {
@@ -223,6 +339,7 @@ String::String(char value, unsigned int repeat)
 	if (m_string) {
 	    ::memset(m_string,value,repeat);
 	    m_string[repeat] = 0;
+	    m_length = repeat;
 	}
 	else
 	    Debug("String",DebugFail,"malloc(%d) returned NULL!",repeat+1);
@@ -230,11 +347,11 @@ String::String(char value, unsigned int repeat)
     }
 }
 
-String::String(int value)
-    : m_string(0), m_length(0), m_hash(INIT_HASH), m_matches(0)
+String::String(int32_t value)
+    : m_string(0), m_length(0), m_hash(YSTRING_INIT_HASH), m_matches(0)
 {
     XDebug(DebugAll,"String::String(%d) [%p]",value,this);
-    char buf[64];
+    char buf[16];
     ::sprintf(buf,"%d",value);
     m_string = ::strdup(buf);
     if (!m_string)
@@ -242,11 +359,23 @@ String::String(int value)
     changed();
 }
 
-String::String(unsigned int value)
-    : m_string(0), m_length(0), m_hash(INIT_HASH), m_matches(0)
+String::String(int64_t value)
+    : m_string(0), m_length(0), m_hash(YSTRING_INIT_HASH), m_matches(0)
+{
+    XDebug(DebugAll,"String::String(" FMT64 ") [%p]",value,this);
+    char buf[24];
+    ::sprintf(buf,FMT64,value);
+    m_string = ::strdup(buf);
+    if (!m_string)
+	Debug("String",DebugFail,"strdup() returned NULL!");
+    changed();
+}
+
+String::String(uint32_t value)
+    : m_string(0), m_length(0), m_hash(YSTRING_INIT_HASH), m_matches(0)
 {
     XDebug(DebugAll,"String::String(%u) [%p]",value,this);
-    char buf[64];
+    char buf[16];
     ::sprintf(buf,"%u",value);
     m_string = ::strdup(buf);
     if (!m_string)
@@ -254,8 +383,20 @@ String::String(unsigned int value)
     changed();
 }
 
+String::String(uint64_t value)
+    : m_string(0), m_length(0), m_hash(YSTRING_INIT_HASH), m_matches(0)
+{
+    XDebug(DebugAll,"String::String(" FMT64U ") [%p]",value,this);
+    char buf[24];
+    ::sprintf(buf,FMT64U,value);
+    m_string = ::strdup(buf);
+    if (!m_string)
+	Debug("String",DebugFail,"strdup() returned NULL!");
+    changed();
+}
+
 String::String(bool value)
-    : m_string(0), m_length(0), m_hash(INIT_HASH), m_matches(0)
+    : m_string(0), m_length(0), m_hash(YSTRING_INIT_HASH), m_matches(0)
 {
     XDebug(DebugAll,"String::String(%u) [%p]",value,this);
     m_string = ::strdup(boolText(value));
@@ -265,13 +406,15 @@ String::String(bool value)
 }
 
 String::String(const String* value)
-    : m_string(0), m_length(0), m_hash(INIT_HASH), m_matches(0)
+    : m_string(0), m_length(0), m_hash(YSTRING_INIT_HASH), m_matches(0)
 {
     XDebug(DebugAll,"String::String(%p) [%p]",&value,this);
     if (value && !value->null()) {
 	m_string = ::strdup(value->c_str());
 	if (!m_string)
 	    Debug("String",DebugFail,"strdup() returned NULL!");
+	else
+	    m_length = value->length();
 	changed();
     }
 }
@@ -311,6 +454,7 @@ String& String::assign(const char* value, int len)
 		data[len] = 0;
 		char* odata = m_string;
 		m_string = data;
+		m_length = len;
 		changed();
 		if (odata)
 		    ::free(odata);
@@ -333,6 +477,7 @@ String& String::assign(char value, unsigned int repeat)
 	    data[repeat] = 0;
 	    char* odata = m_string;
 	    m_string = data;
+	    m_length = repeat;
 	    changed();
 	    if (odata)
 		::free(odata);
@@ -368,6 +513,7 @@ String& String::hexify(void* data, unsigned int len, char sep, bool upCase)
 	    *d = '\0';
 	    char* odata = m_string;
 	    m_string = data;
+	    m_length = repeat;
 	    changed();
 	    if (odata)
 		::free(odata);
@@ -383,8 +529,11 @@ String& String::hexify(void* data, unsigned int len, char sep, bool upCase)
 void String::changed()
 {
     clearMatches();
-    m_hash = INIT_HASH;
-    m_length = m_string ? ::strlen(m_string) : 0;
+    m_hash = YSTRING_INIT_HASH;
+    if (!m_string)
+	m_length = 0;
+    else if (!m_length)
+	m_length = ::strlen(m_string);
 }
 
 void String::clear()
@@ -465,6 +614,27 @@ long int String::toLong(long int defvalue, int base, long int minvalue, long int
     return defvalue;
 }
 
+int64_t String::toInt64(int64_t defvalue, int base, int64_t minvalue, int64_t maxvalue,
+    bool clamp) const
+{
+    if (!m_string)
+	return defvalue;
+    char *eptr = 0;
+
+    errno = 0;
+    int64_t val = ::strtoll(m_string,&eptr,base);
+    // on overflow/underflow mark the entire string as unreadable
+    if ((errno == ERANGE) && eptr)
+	eptr = m_string;
+    if (!eptr || *eptr)
+	return defvalue;
+    if (val >= minvalue && val <= maxvalue)
+	return val;
+    if (clamp)
+	return (val < minvalue) ? minvalue : maxvalue;
+    return defvalue;
+}
+
 double String::toDouble(double defvalue) const
 {
     if (!m_string)
@@ -568,6 +738,7 @@ String& String::operator=(const char* value)
     if (value != c_str()) {
 	char *tmp = m_string;
 	m_string = value ? ::strdup(value) : 0;
+	m_length = 0;
 	if (value && !m_string)
 	    Debug("String",DebugFail,"strdup() returned NULL!");
 	changed();
@@ -577,77 +748,74 @@ String& String::operator=(const char* value)
     return *this;
 }
 
-String& String::operator+=(const char* value)
-{
-    if (value && !*value)
-	value = 0;
-    if (value) {
-	if (m_string) {
-	    int olen = length();
-	    int len = ::strlen(value)+olen;
-	    char *tmp1 = m_string;
-	    char *tmp2 = (char *) ::malloc(len+1);
-	    if (tmp2) {
-		::strncpy(tmp2,m_string,olen);
-		tmp2[olen] = 0;
-		::strncat(tmp2,value,len-olen);
-		tmp2[len] = 0;
-		m_string = tmp2;
-		::free(tmp1);
-	    }
-	    else
-		Debug("String",DebugFail,"malloc(%d) returned NULL!",len+1);
-	}
-	else {
-	    m_string = ::strdup(value);
-	    if (!m_string)
-		Debug("String",DebugFail,"strdup() returned NULL!");
-	}
-	changed();
-    }
-    return *this;
-}
-
 String& String::operator=(char value)
 {
     char buf[2] = {value,0};
     return operator=(buf);
 }
 
-String& String::operator=(int value)
+String& String::operator=(int32_t value)
 {
-    char buf[64];
+    char buf[16];
     ::sprintf(buf,"%d",value);
     return operator=(buf);
 }
 
-String& String::operator=(unsigned int value)
+String& String::operator=(uint32_t value)
 {
-    char buf[64];
+    char buf[16];
     ::sprintf(buf,"%u",value);
     return operator=(buf);
 }
 
+String& String::operator=(int64_t value)
+{
+    char buf[24];
+    ::sprintf(buf,FMT64,value);
+    return operator=(buf);
+}
+
+String& String::operator=(uint64_t value)
+{
+    char buf[24];
+    ::sprintf(buf,FMT64U,value);
+    return operator=(buf);
+}
+
 String& String::operator+=(char value)
 {
     char buf[2] = {value,0};
     return operator+=(buf);
 }
 
-String& String::operator+=(int value)
+String& String::operator+=(int32_t value)
 {
-    char buf[64];
+    char buf[16];
     ::sprintf(buf,"%d",value);
     return operator+=(buf);
 }
 
-String& String::operator+=(unsigned int value)
+String& String::operator+=(uint32_t value)
 {
-    char buf[64];
+    char buf[16];
     ::sprintf(buf,"%u",value);
     return operator+=(buf);
 }
 
+String& String::operator+=(int64_t value)
+{
+    char buf[24];
+    ::sprintf(buf,FMT64,value);
+    return operator+=(buf);
+}
+
+String& String::operator+=(uint64_t value)
+{
+    char buf[24];
+    ::sprintf(buf,FMT64U,value);
+    return operator+=(buf);
+}
+
 String& String::operator>>(const char* skip)
 {
     if (m_string && skip && *skip) {
@@ -667,6 +835,14 @@ String& String::operator>>(char& store)
     return *this;
 }
 
+String& String::operator>>(UChar& store)
+{
+    const char* str = m_string;
+    store.decode(str);
+    assign(str);
+    return *this;
+}
+
 String& String::operator>>(int& store)
 {
     if (m_string) {
@@ -727,6 +903,40 @@ String& String::operator>>(bool& store)
     return *this;
 }
 
+String& String::append(const char* value, int len)
+{
+    if (len && value && *value) {
+	if (len < 0) {
+	    if (!m_string) {
+		m_string = ::strdup(value);
+		m_length = 0;
+		if (!m_string)
+		    Debug("String",DebugFail,"strdup() returned NULL!");
+		changed();
+		return *this;
+	    }
+	    len = ::strlen(value);
+	}
+	int olen = length();
+	len += olen;
+	char *tmp1 = m_string;
+	char *tmp2 = (char *) ::malloc(len+1);
+	if (tmp2) {
+	    if (m_string)
+		::strncpy(tmp2,m_string,olen);
+	    ::strncpy(tmp2+olen,value,len-olen);
+	    tmp2[len] = 0;
+	    m_string = tmp2;
+	    m_length = len;
+	    ::free(tmp1);
+	}
+	else
+	    Debug("String",DebugFail,"malloc(%d) returned NULL!",len+1);
+	changed();
+    }
+    return *this;
+}
+
 String& String::append(const char* value, const char* separator, bool force)
 {
     if (value || force) {
@@ -773,6 +983,7 @@ String& String::append(const ObjList* list, const char* separator, bool force)
     }
     newStr[olen] = 0;
     m_string = newStr;
+    m_length = olen;
     ::free(oldStr);
     changed();
     return *this;
@@ -801,20 +1012,6 @@ bool String::operator!=(const char* value) const
     return (!value) || ::strcmp(m_string,value);
 }
 
-bool String::operator==(const String& value) const
-{
-    if (hash() != value.hash())
-	return false;
-    return operator==(value.c_str());
-}
-
-bool String::operator!=(const String& value) const
-{
-    if (hash() != value.hash())
-	return true;
-    return operator!=(value.c_str());
-}
-
 bool String::operator&=(const char* value) const
 {
     if (!m_string)
@@ -1033,15 +1230,19 @@ String String::msgEscape(const char* str, char extraEsc)
     if (TelEngine::null(str))
 	return s;
     char c;
-    while ((c=*str++)) {
-	if ((unsigned char)c < ' ' || c == ':' || c == extraEsc) {
+    const char* pos = str;
+    char buff[3] =  {'%', '%', '\0'};
+    while ((c=*pos++)) {
+	if ((unsigned char)c < ' ' || c == ':' || c == extraEsc)
 	    c += '@';
-	    s += '%';
-	}
-	else if (c == '%')
-	    s += c;
-	s += c;
+	else if (c != '%')
+	    continue;
+	buff[1] = c;
+	s.append(str,pos - str - 1);
+	s += buff;
+	str = pos;
     }
+    s += str;
     return s;
 }
 
@@ -1058,6 +1259,7 @@ String String::msgUnescape(const char* str, int* errptr, char extraEsc)
 	if ((unsigned char)c < ' ') {
 	    if (errptr)
 		*errptr = (pos-str) - 1;
+	    s.append(str,pos - str - 1);
 	    return s;
 	}
 	else if (c == '%') {
@@ -1067,11 +1269,15 @@ String String::msgUnescape(const char* str, int* errptr, char extraEsc)
 	    else if (c != '%') {
 		if (errptr)
 		    *errptr = (pos-str) - 1;
+		s.append(str,pos - str - 1);
 		return s;
 	    }
+	    s.append(str,pos - str - 2);
+	    s += c;
+	    str = pos;
 	}
-	s += c;
     }
+    s += str;
     if (errptr)
 	*errptr = -1;
     return s;
@@ -1084,8 +1290,10 @@ String String::sqlEscape(const char* str, char extraEsc)
 	return s;
     char c;
     while ((c=*str++)) {
-	if (c == '\\' || c == '\'' || c == extraEsc)
-	    s += '\\';
+	if (c == '\'')
+	    s += "'";
+	else if (c == '\\' || c == extraEsc)
+	    s += "\\";
 	s += c;
     }
     return s;
@@ -1142,13 +1350,6 @@ String String::uriUnescape(const char* str, int* errptr)
     return s;
 }
 
-unsigned int String::hash() const
-{
-    if (m_hash == INIT_HASH)
-	m_hash = hash(m_string);
-    return m_hash;
-}
-
 unsigned int String::hash(const char* value)
 {
     if (!value)
@@ -1161,17 +1362,17 @@ unsigned int String::hash(const char* value)
     return h;
 }
 
-int String::lenUtf8(const char* value, unsigned int maxSeq, bool overlong)
+int String::lenUtf8(const char* value, uint32_t maxChar, bool overlong)
 {
     if (!value)
 	return 0;
-    if (maxSeq < 1)
-	maxSeq = 4; // RFC 3629 default limit
+    if (maxChar < 128)
+	maxChar = 0x10ffff; // RFC 3629 default limit
 
     int count = 0;
     unsigned int more = 0;
-    int32_t min = 0;
-    int32_t val = 0;
+    uint32_t min = 0;
+    uint32_t val = 0;
 
     while (unsigned char c = (unsigned char) *value++) {
 	if (more) {
@@ -1180,9 +1381,11 @@ int String::lenUtf8(const char* value, unsigned int maxSeq, bool overlong)
 		return -1;
 	    val = (val << 6) | (c & 0x3f);
 	    if (!--more) {
+		// got full value, check for overlongs and out of range
+		if (val > maxChar)
+		    return -1;
 		if (overlong)
 		    continue;
-		// got full value, check for overlongs
 		if (val < min)
 		    return -1;
 	    }
@@ -1190,35 +1393,35 @@ int String::lenUtf8(const char* value, unsigned int maxSeq, bool overlong)
 	}
 	count++;
 	// from 1st byte we find out how many are supposed to follow
-	if (c < 128)      // 1 byte, 0...0x7F, ASCII characters, no check
+	if (c < 0x80)      // 1 byte, 0...0x7F, ASCII characters, no check
 	    ;
-	else if (c < 192) // invalid as first UFT-8 byte
+	else if (c < 0xc0) // invalid as first UFT-8 byte
 	    return -1;
-	else if (c < 224) {
+	else if (c < 0xe0) {
 	    // 2 bytes, 0x80...0x7FF
 	    min = 0x80;
 	    val = c & 0x1f;
 	    more = 1;
 	}
-	else if (c < 240) {
+	else if (c < 0xf0) {
 	    // 3 bytes, 0x800...0xFFFF, Basic Multilingual Plane
 	    min = 0x800;
 	    val = c & 0x0f;
 	    more = 2;
 	}
-	else if (c < 248) {
+	else if (c < 0xf8) {
 	    // 4 bytes, 0x10000...0x1FFFFF, RFC 3629 limit (10FFFF)
 	    min = 0x10000;
 	    val = c & 0x07;
 	    more = 3;
 	}
-	else if (c < 252) {
+	else if (c < 0xfc) {
 	    // 5 bytes, 0x200000...0x3FFFFFF
 	    min = 0x200000;
 	    val = c & 0x03;
 	    more = 4;
 	}
-	else if (c < 254) {
+	else if (c < 0xfe) {
 	    // 6 bytes, 0x4000000...0x7FFFFFFF
 	    min = 0x4000000;
 	    val = c & 0x01;
@@ -1226,28 +1429,25 @@ int String::lenUtf8(const char* value, unsigned int maxSeq, bool overlong)
 	}
 	else
 	    return -1;
-	// check if we accept a character with such sequence length
-	if (more >= maxSeq)
-	    return -1;
     }
     if (more)
 	return -1;
     return count;
 }
 
-int String::fixUtf8(const char* replace, unsigned int maxSeq, bool overlong)
+int String::fixUtf8(const char* replace, uint32_t maxChar, bool overlong)
 {
     if (null())
 	return 0;
-    if (maxSeq < 1)
-	maxSeq = 4; // RFC 3629 default limit
+    if (maxChar < 128)
+	maxChar = 0x10ffff; // RFC 3629 default limit
     if (!replace)
 	replace = "\xEF\xBF\xBD";
 
     int count = 0;
     unsigned int more = 0;
-    int32_t min = 0;
-    int32_t val = 0;
+    uint32_t min = 0;
+    uint32_t val = 0;
     unsigned int pos = 0;
     bool bad = false;
     String tmp;
@@ -1255,9 +1455,6 @@ int String::fixUtf8(const char* replace, unsigned int maxSeq, bool overlong)
     for (unsigned int i = 0; i < m_length; i++) {
 	unsigned char c = (unsigned char) at(i);
 	if (more) {
-	    // remember to reject a character with a too long sequence
-	    if (more >= maxSeq)
-		bad = true;
 	    // all continuation bytes are in range [128..191]
 	    if ((c & 0xc0) != 0x80) {
 		// truncated sequence, must search for 1st byte again
@@ -1268,8 +1465,8 @@ int String::fixUtf8(const char* replace, unsigned int maxSeq, bool overlong)
 	    else {
 		val = (val << 6) | (c & 0x3f);
 		if (!--more) {
-		    // got full value, check for overlongs
-		    if ((val < min) && !overlong)
+		    // got full value, check for overlongs and out of range
+		    if ((val > maxChar) || ((val < min) && !overlong))
 			bad = true;
 		    // finished multibyte, add it to temporary
 		    if (bad) {
@@ -1285,35 +1482,35 @@ int String::fixUtf8(const char* replace, unsigned int maxSeq, bool overlong)
 	pos = i;
 	bad = false;
 	// from 1st byte we find out how many are supposed to follow
-	if (c < 128)      // 1 byte, 0...0x7F, ASCII characters, good
+	if (c < 0x80)      // 1 byte, 0...0x7F, ASCII characters, good
 	    ;
-	else if (c < 192) // invalid as first UFT-8 byte
+	else if (c < 0xc0) // invalid as first UFT-8 byte
 	    bad = true;
-	else if (c < 224) {
+	else if (c < 0xe0) {
 	    // 2 bytes, 0x80...0x7FF
 	    min = 0x80;
 	    val = c & 0x1f;
 	    more = 1;
 	}
-	else if (c < 240) {
+	else if (c < 0xf0) {
 	    // 3 bytes, 0x800...0xFFFF, Basic Multilingual Plane
 	    min = 0x800;
 	    val = c & 0x0f;
 	    more = 2;
 	}
-	else if (c < 248) {
+	else if (c < 0xf8) {
 	    // 4 bytes, 0x10000...0x1FFFFF, RFC 3629 limit (10FFFF)
 	    min = 0x10000;
 	    val = c & 0x07;
 	    more = 3;
 	}
-	else if (c < 252) {
+	else if (c < 0xfc) {
 	    // 5 bytes, 0x200000...0x3FFFFFF
 	    min = 0x200000;
 	    val = c & 0x03;
 	    more = 4;
 	}
-	else if (c < 254) {
+	else if (c < 0xfe) {
 	    // 6 bytes, 0x4000000...0x7FFFFFFF
 	    min = 0x4000000;
 	    val = c & 0x01;
@@ -1343,7 +1540,7 @@ int String::fixUtf8(const char* replace, unsigned int maxSeq, bool overlong)
 
 void* String::getObject(const String& name) const
 {
-    if (name == YSTRING("String"))
+    if (name == YATOM("String"))
 	return const_cast<String*>(this);
     return GenObject::getObject(name);
 }
@@ -1353,6 +1550,26 @@ const String& String::toString() const
     return *this;
 }
 
+const String* String::atom(const String*& str, const char* val)
+{
+    if (!str) {
+	s_mutex.lock();
+	if (!str) {
+	    if (TelEngine::null(val))
+		str = &s_empty;
+	    else {
+		str = static_cast<const String*>(s_atoms[val]);
+		if (!str) {
+		    str = new String(val);
+		    s_atoms.insert(str);
+		}
+	    }
+	}
+	s_mutex.unlock();
+    }
+    return str;
+}
+
 
 Regexp::Regexp()
     : m_regexp(0), m_flags(0)
@@ -1467,7 +1684,7 @@ const String& NamedString::toString() const
 
 void* NamedString::getObject(const String& name) const
 {
-    if (name == YSTRING("NamedString"))
+    if (name == YATOM("NamedString"))
 	return (void*)this;
     return String::getObject(name);
 }
@@ -1502,7 +1719,7 @@ GenObject* NamedPointer::takeData()
 
 void* NamedPointer::getObject(const String& name) const
 {
-    if (name == YSTRING("NamedPointer"))
+    if (name == YATOM("NamedPointer"))
 	return (void*)this;
     void* p = NamedString::getObject(name);
     if (p)
