@@ -3,21 +3,18 @@
  * This file is part of the YATE Project http://YATE.null.ro
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include "yateclass.h"
@@ -132,7 +129,7 @@ static const char* const s_levels[11] = {
     "ALL",
 };
 
-static const char* dbg_level(int level)
+static const char* dbg_level(int& level)
 {
     if (level < DebugMin)
 	level = DebugMin;
@@ -157,6 +154,7 @@ static void dbg_colorize_func(const char* buf, int level)
 
 static void (*s_output)(const char*,int) = dbg_stderr_func;
 static void (*s_intout)(const char*,int) = 0;
+static void (*s_alarms)(const char*,int,const char*,const char*) = 0;
 
 static Mutex out_mux(false,"DebugOutput");
 static Mutex ind_mux(false,"DebugIndent");
@@ -180,7 +178,7 @@ static void common_output(int level,char* buf)
 	level = DebugMax;
     int n = ::strlen(buf);
     if (n && (buf[n-1] == '\n'))
-	    n--;
+	n--;
     // serialize the output strings
     out_mux.lock();
     // TODO: detect reentrant calls from foreign threads and main thread
@@ -198,13 +196,17 @@ static void common_output(int level,char* buf)
 	s_output(buf,level);
     if (s_intout)
 	s_intout(buf,level);
+    buf[n] = '\0';
     s_thr = 0;
     out_mux.unlock();
 }
 
-static void dbg_output(int level,const char* prefix, const char* format, va_list ap)
+static void dbg_output(int level,const char* prefix, const char* format, va_list ap,
+    const char* alarmComp = 0, const char* alarmInfo = 0)
 {
-    if (!(s_output || s_intout))
+    bool out = (s_output || s_intout) && (prefix || format);
+    bool alarm = alarmComp && format && s_alarms;
+    if (!(out || alarm))
 	return;
     char buf[OUT_BUFFER_SIZE];
     unsigned int n = Debugger::formatTime(buf,s_fmtstamp);
@@ -219,11 +221,19 @@ static void dbg_output(int level,const char* prefix, const char* format, va_list
 	::strncpy(buf+n,prefix,l);
     n = ::strlen(buf);
     l = sizeof(buf)-n-2;
+    char* msg = buf+n;
     if (format) {
-	::vsnprintf(buf+n,l,format,ap);
+	::vsnprintf(msg,l,format,ap);
 	buf[OUT_BUFFER_SIZE - 2] = 0;
     }
-    common_output(level,buf);
+    if (out)
+	common_output(level,buf);
+    if (alarm) {
+	out_mux.lock();
+	if (s_alarms)
+	    s_alarms(msg,level,alarmComp,alarmInfo);
+	out_mux.unlock();
+    }
 }
 
 void Output(const char* format, ...)
@@ -244,7 +254,7 @@ void Debug(int level, const char* format, ...)
 {
     if (!s_debugging)
 	return;
-    if (level > s_debug)
+    if (level > s_debug || level < DebugMin)
 	return;
     if (reentered())
 	return;
@@ -266,7 +276,7 @@ void Debug(const char* facility, int level, const char* format, ...)
 {
     if (!s_debugging)
 	return;
-    if (level > s_debug)
+    if (level > s_debug || level < DebugMin)
 	return;
     if (reentered())
 	return;
@@ -290,7 +300,7 @@ void Debug(const DebugEnabler* local, int level, const char* format, ...)
 	return;
     const char* facility = 0;
     if (!local) {
-	if (level > s_debug)
+	if (level > s_debug || level < DebugMin)
 	    return;
     }
     else {
@@ -317,6 +327,78 @@ void Debug(const DebugEnabler* local, int level, const char* format, ...)
 	abort();
 }
 
+void Alarm(const char* component, int level, const char* format, ...)
+{
+    if (!format || level < DebugMin || reentered())
+	return;
+    if (TelEngine::null(component))
+	component = "unknown";
+    char buf[64];
+    ::snprintf(buf,sizeof(buf),"<%s:%s> ",component,dbg_level(level));
+    va_list va;
+    va_start(va,format);
+    ind_mux.lock();
+    dbg_output(level,buf,format,va,component);
+    ind_mux.unlock();
+    va_end(va);
+    if (s_abort && (level == DebugFail))
+	abort();
+}
+
+void Alarm(const DebugEnabler* component, int level, const char* format, ...)
+{
+    if (!format || level < DebugMin || reentered())
+	return;
+    const char* name = (component && !TelEngine::null(component->debugName()))
+	? component->debugName() : "unknown";
+    char buf[64];
+    ::snprintf(buf,sizeof(buf),"<%s:%s> ",name,dbg_level(level));
+    va_list va;
+    va_start(va,format);
+    ind_mux.lock();
+    dbg_output(level,buf,format,va,name);
+    ind_mux.unlock();
+    va_end(va);
+    if (s_abort && (level == DebugFail))
+	abort();
+}
+
+void Alarm(const char* component, const char* info, int level, const char* format, ...)
+{
+    if (!format || level < DebugMin || reentered())
+	return;
+    if (TelEngine::null(component))
+	component = "unknown";
+    char buf[64];
+    ::snprintf(buf,sizeof(buf),"<%s:%s> ",component,dbg_level(level));
+    va_list va;
+    va_start(va,format);
+    ind_mux.lock();
+    dbg_output(level,buf,format,va,component,info);
+    ind_mux.unlock();
+    va_end(va);
+    if (s_abort && (level == DebugFail))
+	abort();
+}
+
+void Alarm(const DebugEnabler* component, const char* info, int level, const char* format, ...)
+{
+    if (!format || level < DebugMin || reentered())
+	return;
+    const char* name = (component && !TelEngine::null(component->debugName()))
+	? component->debugName() : "unknown";
+    char buf[64];
+    ::snprintf(buf,sizeof(buf),"<%s:%s> ",name,dbg_level(level));
+    va_list va;
+    va_start(va,format);
+    ind_mux.lock();
+    dbg_output(level,buf,format,va,name,info);
+    ind_mux.unlock();
+    va_end(va);
+    if (s_abort && (level == DebugFail))
+	abort();
+}
+
 void abortOnBug()
 {
     if (s_abort)
@@ -358,6 +440,12 @@ const char* debugColor(int level)
     return s_colors[level];
 }
 
+const char* debugLevelName(int level)
+{
+    return (level < DebugMin) ? "" : dbg_level(level);
+}
+
+
 int DebugEnabler::debugLevel(int level)
 {
     if (level < DebugVis)
@@ -432,6 +520,21 @@ static void dbg_dist_helper(int level, const char* buf, const char* fmt, ...)
     va_end(va);
 }
 
+bool controlReturn(NamedList* params, bool ret, const char* retVal)
+{
+    if (retVal && params)
+	params->setParam("retVal",retVal);
+    if (!params || !params->getObject(YATOM("Message")))
+	return ret;
+    const String* module = params->getParam("module");
+    if (TelEngine::null(module) || YSTRING("rmanager") != *module)
+	return ret;
+    const String s_opStat("operation-status");
+    if (!params->getParam(s_opStat))
+	params->addParam(s_opStat,String::boolText(ret));
+    return ret;
+}
+
 Debugger::~Debugger()
 {
     if (m_name) {
@@ -457,6 +560,13 @@ void Debugger::setIntOut(void (*outFunc)(const char*,int))
     out_mux.unlock();
 }
 
+void Debugger::setAlarmHook(void (*alarmFunc)(const char*,int,const char*,const char*))
+{
+    out_mux.lock();
+    s_alarms = alarmFunc;
+    out_mux.unlock();
+}
+
 void Debugger::enableOutput(bool enable, bool colorize)
 {
     s_debugging = enable;
@@ -486,27 +596,39 @@ unsigned int Debugger::formatTime(char* buf, Formatting format)
 	    t -= s_timestamp;
 	unsigned int s = (unsigned int)(t / 1000000);
 	unsigned int u = (unsigned int)(t % 1000000);
-	if (Textual == format || TextLocal == format) {
-	    time_t sec = (time_t)s;
-	    struct tm tmp;
-	    if (TextLocal == format)
+	switch (format) {
+	    case Textual:
+	    case TextLocal:
+	    case TextSep:
+	    case TextLSep:
+		{
+		    time_t sec = (time_t)s;
+		    struct tm tmp;
+		    if (TextLocal == format || TextLSep == format)
 #ifdef _WINDOWS
-		_localtime_s(&tmp,&sec);
+			_localtime_s(&tmp,&sec);
 #else
-		localtime_r(&sec,&tmp);
+			localtime_r(&sec,&tmp);
 #endif
-	    else
+		    else
 #ifdef _WINDOWS
-		_gmtime_s(&tmp,&sec);
+			_gmtime_s(&tmp,&sec);
 #else
-		gmtime_r(&sec,&tmp);
+			gmtime_r(&sec,&tmp);
 #endif
-	    ::sprintf(buf,"%04d%02d%02d%02d%02d%02d.%06u ",
-		tmp.tm_year+1900,tmp.tm_mon+1,tmp.tm_mday,
-		tmp.tm_hour,tmp.tm_min,tmp.tm_sec,u);
+		    if (Textual == format || TextLocal == format)
+			::sprintf(buf,"%04d%02d%02d%02d%02d%02d.%06u ",
+			    tmp.tm_year+1900,tmp.tm_mon+1,tmp.tm_mday,
+			    tmp.tm_hour,tmp.tm_min,tmp.tm_sec,u);
+		    else
+			::sprintf(buf,"%04d-%02d-%02d_%02d:%02d:%02d.%06u ",
+			    tmp.tm_year+1900,tmp.tm_mon+1,tmp.tm_mday,
+			    tmp.tm_hour,tmp.tm_min,tmp.tm_sec,u);
+		}
+		break;
+	    default:
+		::sprintf(buf,"%07u.%06u ",s,u);
 	}
-	else
-	    ::sprintf(buf,"%07u.%06u ",s,u);
 	return ::strlen(buf);
     }
     buf[0] = '\0';
@@ -607,7 +729,8 @@ unsigned int Time::toEpoch(int year, unsigned int month, unsigned int day,
 
 // Split a given EPOCH time into its date/time components
 bool Time::toDateTime(unsigned int epochTimeSec, int& year, unsigned int& month,
-    unsigned int& day, unsigned int& hour, unsigned int& minute, unsigned int& sec)
+    unsigned int& day, unsigned int& hour, unsigned int& minute, unsigned int& sec,
+    unsigned int* wDay)
 {
 #ifdef _WINDOWS
     FILETIME ft;
@@ -626,6 +749,8 @@ bool Time::toDateTime(unsigned int epochTimeSec, int& year, unsigned int& month,
     hour = st.wHour;
     minute = st.wMinute;
     sec = st.wSecond;
+    if (wDay)
+	*wDay = st.wDayOfWeek;
 #else
     struct tm t;
     time_t time = (time_t)epochTimeSec;
@@ -637,12 +762,35 @@ bool Time::toDateTime(unsigned int epochTimeSec, int& year, unsigned int& month,
     hour = t.tm_hour;
     minute = t.tm_min;
     sec = t.tm_sec;
+    if (wDay)
+	*wDay = t.tm_wday;
 #endif
     DDebug(DebugAll,"Time::toDateTime(%u,%d,%u,%u,%u,%u,%u)",
 	epochTimeSec,year,month,day,hour,minute,sec);
     return true;
 }
 
+int Time::timeZone()
+{
+#ifdef _WINDOWS
+    struct tm t;
+    time_t time = (time_t)secNow();
+    _localtime_s(&t,&time);
+    if (t.tm_isdst)
+	return -(_timezone + _dstbias);
+    return -_timezone;
+#else
+#ifdef HAVE_GMTOFF
+    struct tm t;
+    time_t time = (time_t)secNow();
+    if (localtime_r(&time,&t))
+	return t.tm_gmtoff;
+#endif
+    return -timezone;
+#endif
+}
+
+
 static Random s_random;
 static Mutex s_randomMutex(false,"Random");
 
@@ -698,7 +846,7 @@ RefObject::~RefObject()
 
 void* RefObject::getObject(const String& name) const
 {
-    if (name == YSTRING("RefObject"))
+    if (name == YATOM("RefObject"))
 	return (void*)this;
     return GenObject::getObject(name);
 }
