@@ -3,21 +3,18 @@
  * This file is part of the YATE Project http://YATE.null.ro
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include "yateclass.h"
@@ -228,7 +225,7 @@ ThreadPrivate* ThreadPrivate::create(Thread* t,const char* name,Thread::Priority
     ::pthread_attr_destroy(&attr);
 #endif
     if (e) {
-	Debug(DebugGoOn,"Error %d while creating pthread in '%s' [%p]",e,name,p);
+	Alarm("engine","system",DebugGoOn,"Error %d while creating pthread in '%s' [%p]",e,name,p);
 	p->m_thread = 0;
 	p->destroy();
 	return 0;
@@ -405,10 +402,10 @@ void ThreadPrivate::cleanup()
 	    m_thread->m_private = 0;
 	    m_thread->cleanup();
 	    if (m_thread->locked())
-		Debug(DebugFail,"Thread '%s' destroyed with mutex locks (%d held) [%p]",m_name,m_thread->locks(),m_thread);
+		Alarm("engine","bug",DebugFail,"Thread '%s' destroyed with mutex locks (%d held) [%p]",m_name,m_thread->locks(),m_thread);
 	}
 	else {
-	    Debug(DebugFail,"ThreadPrivate::cleanup() %p '%s' mismatching %p [%p]",m_thread,m_name,m_thread->m_private,this);
+	    Alarm("engine","bug",DebugFail,"ThreadPrivate::cleanup() %p '%s' mismatching %p [%p]",m_thread,m_name,m_thread->m_private,this);
 	    m_thread = 0;
 	}
     }
@@ -643,7 +640,7 @@ void Thread::exit()
     DDebug(DebugAll,"Thread::exit()");
     ThreadPrivate* t = ThreadPrivate::current();
     if (t && t->m_thread && t->m_thread->locked())
-	Debug(DebugFail,"Thread::exit() in '%s' with mutex locks (%d held) [%p]",
+	Alarm("engine","bug",DebugFail,"Thread::exit() in '%s' with mutex locks (%d held) [%p]",
 	    t->m_name,t->m_thread->locks(),t->m_thread);
 #ifdef _WINDOWS
     if (t) {
