@@ -6,21 +6,18 @@
  * Adapted for YATE by Paul Chitescu
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include "yateclass.h"
@@ -269,30 +266,25 @@ static void MD5_Final(unsigned char digest[16], MD5_CTX *ctx)
 using namespace TelEngine;
 
 MD5::MD5()
-    : m_private(0)
 {
 }
 
 MD5::MD5(const void* buf, unsigned int len)
-    : m_private(0)
 {
     update(buf,len);
 }
 
 MD5::MD5(const DataBlock& data)
-    : m_private(0)
 {
     update(data);
 }
 
 MD5::MD5(const String& str)
-    : m_private(0)
 {
     update(str);
 }
 
 MD5::MD5(const MD5& original)
-    : m_private(0)
 {
     m_hex = original.m_hex;
     ::memcpy(m_bin,original.m_bin,sizeof(m_bin));
@@ -355,7 +347,7 @@ void MD5::finalize()
     m_hex = buf;
 }
 
-bool MD5::update(const void* buf, unsigned int len)
+bool MD5::updateInternal(const void* buf, unsigned int len)
 {
     // Don't update an already finalized digest
     if (m_hex)
@@ -369,23 +361,10 @@ bool MD5::update(const void* buf, unsigned int len)
     return true;
 }
 
-MD5& MD5::operator<<(const char* value)
-{
-    if (!null(value))
-	update(value,::strlen(value));
-    return *this;
-}
-
 const unsigned char* MD5::rawDigest()
 {
     finalize();
     return m_bin;
 }
 
-const String& MD5::hexDigest()
-{
-    finalize();
-    return m_hex;
-}
-
 /* vi: set ts=8 sw=4 sts=4 noet: */
