@@ -5,21 +5,18 @@
  * ASN.1 Library
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2010 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include "yateasn.h"
@@ -1270,14 +1267,15 @@ void AsnTag::encode(Class clas, Type type, unsigned int code, DataBlock& data)
     }
     else {
 	u_int8_t last = clas | type | 31;
-	data.append(&last,sizeof(last));
+	DataBlock coding;
+ 	coding.append(&last,sizeof(last));
 	int size = sizeof(unsigned int);
 	bool start = false;
 	while (size > 1) {
 	    u_int8_t msb = (code >> ((size - 1) * 8));
 	    if (start) {
 		msb |= 0x80;
-		data.append(&msb,sizeof(msb));
+		coding.append(&msb,sizeof(msb));
 	    }
 	    else {
 		if (msb == 0) {
@@ -1287,18 +1285,19 @@ void AsnTag::encode(Class clas, Type type, unsigned int code, DataBlock& data)
 		else {
 		    start = true;
 		    msb |= 0x80;
-		    data.append(&msb,sizeof(msb));
+		    coding.append(&msb,sizeof(msb));
 		}
 	    }
 	    size--;
 	}
 	last = code;
-	data.append(&last,sizeof(last));
+	coding.append(&last,sizeof(last));
+	data.insert(coding);
     }
 #ifdef XDEBUG
     String str;
     str.hexify(data.data(),data.length(),' ');
-    XDebug(s_libName.c_str(),DebugAll,"AsnTag::encode(clas=0x%x, type=0x%x, code=%u) tag=%s",clas,type,code,str.c_str());
+    Debug(s_libName.c_str(),DebugAll,"AsnTag::encode(clas=0x%x, type=0x%x, code=%u) tag=%s",clas,type,code,str.c_str());
 #endif
 }
 
