@@ -5,21 +5,18 @@
  * ASN.1 Library
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2010 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #ifndef __YATEASN_H
@@ -101,7 +98,7 @@ public:
      */
     inline DataBlock& fromHexString(const String& value)
     {
-	unHexify(value,value.length());
+	unHexify(value);
 	return *this;
     }
 };
@@ -204,8 +201,8 @@ public:
      * Get the value in the form of a string
      * @return String containing the internal data
      */
-    inline String getValue()
-	{ return m_data;}
+    inline const String& getValue()
+	{ return m_data; }
 
     /**
      * Get the type of the data so that we know how to interpret it
@@ -230,7 +227,7 @@ public:
      * Assign operator
      */
     inline AsnValue& operator=( AsnValue val)
-    {  
+    {
 	m_data = val.getValue();
 	m_type = val.type();
 	return *this;
