@@ -4,22 +4,19 @@
  * This file is part of the YATE Project http://YATE.null.ro
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  * Author: Marian Podgoreanu
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include <yateiax.h>
@@ -37,6 +34,12 @@ using namespace TelEngine;
 // Minimum value for local call numbers
 #define IAX2_MIN_CALLNO 2
 
+// Outgoing data adjust timestamp defaults
+#define IAX2_ADJUSTTSOUT_THRES 120
+#define IAX2_ADJUSTTSOUT_OVER 120
+#define IAX2_ADJUSTTSOUT_UNDER 60
+
+
 // Build an MD5 digest from secret, address, integer value and engine run id
 // MD5(addr.host() + secret + addr.port() + t)
 static void buildSecretDigest(String& buf, const String& secret, unsigned int t,
@@ -49,85 +52,55 @@ static void buildSecretDigest(String& buf, const String& secret, unsigned int t,
 }
 
 
-IAXEngine::IAXEngine(const char* iface, int port, u_int16_t transListCount, u_int16_t retransCount, u_int16_t retransInterval,
-	u_int16_t authTimeout, u_int16_t transTimeout, u_int16_t maxFullFrameDataLen,
-	u_int32_t format, u_int32_t capab, u_int32_t trunkSendInterval, bool authRequired,
-	NamedList* params)
+IAXEngine::IAXEngine(const char* iface, int port, u_int32_t format, u_int32_t capab,
+    const NamedList* params, const char* name)
     : Mutex(true,"IAXEngine"),
+    m_trunking(0),
+    m_name(name),
     m_lastGetEvIndex(0),
-    m_authRequired(authRequired),
-    m_maxFullFrameDataLen(maxFullFrameDataLen),
+    m_exiting(false),
+    m_maxFullFrameDataLen(1400),
     m_startLocalCallNo(0),
-    m_transListCount(0),
-    m_retransCount(retransCount),
-    m_retransInterval(retransInterval),
-    m_authTimeout(authTimeout),
-    m_transTimeout(transTimeout),
+    m_transListCount(64),
+    m_challengeTout(IAX2_CHALLENGETOUT_DEF),
     m_callToken(false),
     m_callTokenAge(10),
     m_showCallTokenFailures(false),
     m_printMsg(true),
+    m_callerNumType(0),
+    m_callingPres(0),
     m_format(format),
     m_formatVideo(0),
     m_capability(capab),
-    m_mutexTrunk(true,"IAXEngine::Trunk"),
-    m_trunkSendInterval(trunkSendInterval)
+    m_adjustTsOutThreshold(IAX2_ADJUSTTSOUT_THRES),
+    m_adjustTsOutOverrun(IAX2_ADJUSTTSOUT_OVER),
+    m_adjustTsOutUnderrun(IAX2_ADJUSTTSOUT_UNDER),
+    m_mutexTrunk(false,"IAXEngine::Trunk"),
+    m_trunkInfoMutex(false,"IAXEngine::TrunkInfo")
 {
-    debugName("iaxengine");
-    Debug(this,DebugAll,"Automatically request authentication set to '%s'.",
-	(authRequired?"YES":"NO"));
+    debugName(m_name);
     if ((port <= 0) || port > 65535)
 	port = 4569;
-    if (transListCount < 4)
-	transListCount = 4;
-    else if (transListCount > 256)
-	transListCount = 256;
-    m_transList = new ObjList*[transListCount];
-    int i;
-    for (i = 0; i < transListCount; i++)
-	m_transList[i] = new ObjList;
-    m_transListCount = transListCount;
-    for(i = 0; i <= IAX2_MAX_CALLNO; i++)
-	m_lUsedCallNo[i] = false;
-    if (params)
+    bool forceBind = true;
+    if (params) {
+	m_transListCount = params->getIntValue("translist_count",64,4,256);
+	m_maxFullFrameDataLen = params->getIntValue("maxfullframedatalen",1400,20);
 	m_callTokenSecret = params->getValue("calltoken_secret");
-    if (!m_callTokenSecret)
-	for (i = 0; i < 3; i++)
-	    m_callTokenSecret << (int)(Random::random() ^ Time::now());
-    m_socket.create(AF_INET,SOCK_DGRAM);
-    SocketAddr addr(AF_INET);
-    addr.host(iface);
-    addr.port(port);
-    m_socket.setBlocking(false);
-    bool ok = m_socket.bind(addr);
-    if (!ok) {
-	bool force = !params || params->getBoolValue("force_bind",true);
-	String tmp;
-	Thread::errorString(tmp,m_socket.error());
-	Debug(this,DebugWarn,"Failed to bind socket on '%s:%d'%s. %d: '%s'",
-	    c_safe(iface),port,force ? " - trying a random port" : "",
-	    m_socket.error(),tmp.c_str());
-	if (force) {
-	    addr.port(0);
-	    ok = m_socket.bind(addr);
-	    if (!ok)
-		Debug(this,DebugWarn,"Failed to bind on any port");
-	    else {
-		ok = m_socket.getSockName(addr);
-		if (!ok)
-		    Debug(this,DebugWarn,"Failed to retrieve bound address");
-	    }
-	}
+	forceBind = params->getBoolValue("force_bind",true);
     }
-    if (ok)
-	Debug(this,DebugInfo,"Bound on '%s:%d'",addr.host().c_str(),addr.port());
+    m_transList = new ObjList*[m_transListCount];
+    for (unsigned int i = 0; i < m_transListCount; i++)
+	m_transList[i] = new ObjList;
+    for(unsigned int i = 0; i <= IAX2_MAX_CALLNO; i++)
+	m_lUsedCallNo[i] = false;
+    if (!m_callTokenSecret)
+	for (unsigned int i = 0; i < 3; i++)
+	    m_callTokenSecret << (int)(Random::random() ^ Time::now());
+    bind(iface,port,forceBind);
     m_startLocalCallNo = 1 + (u_int16_t)(Random::random() % IAX2_MAX_CALLNO);
     if (m_startLocalCallNo < IAX2_MIN_CALLNO)
 	m_startLocalCallNo = IAX2_MIN_CALLNO;
-    if (params)
-	initialize(*params);
-    else
-	initialize(NamedList::empty());
+    initialize(params ? *params : NamedList::empty());
 }
 
 IAXEngine::~IAXEngine()
@@ -141,27 +114,27 @@ IAXTransaction* IAXEngine::addFrame(const SocketAddr& addr, IAXFrame* frame)
 {
     if (!frame)
 	return 0;
-    IAXTransaction* tr;
-    ObjList* l;
+    IAXTransaction* tr = 0;
+    ObjList* l = 0;
     Lock lock(this);
     // Transaction exists for this frame?
-    // Incomplete transactions. They MUST receive a full frame
-    IAXFullFrame* fullFrame = frame->fullFrame();
-    if (fullFrame) {
+    // Incomplete transactions. They MUST receive a full frame with destination call number set
+    IAXFullFrame* full = frame->fullFrame();
+    if (full && full->destCallNo()) {
 	l = m_incompleteTransList.skipNull();
 	for (; l; l = l->next()) {
 	    tr = static_cast<IAXTransaction*>(l->get());
-	    if (!(tr && tr->localCallNo() == fullFrame->destCallNo() && addr == tr->remoteAddr()))
+	    if (!(tr && tr->localCallNo() == full->destCallNo() && addr == tr->remoteAddr()))
 		continue;
 	    // Incomplete outgoing receiving call token
-	    if (fullFrame->type() == IAXFrame::IAX &&
-		fullFrame->subclass() == IAXControl::CallToken) {
+	    if (full->type() == IAXFrame::IAX &&
+		full->subclass() == IAXControl::CallToken) {
 		RefPointer<IAXTransaction> t = tr;
 		lock.drop();
 		if (!t)
 		    return 0;
-		fullFrame->updateIEList(true);
-		IAXIEList* list = fullFrame->ieList();
+		full->updateIEList(true);
+		IAXIEList* list = full->ieList();
 		DataBlock db;
 		if (list)
 		    list->getBinary(IAXInfoElement::CALLTOKEN,db);
@@ -170,25 +143,24 @@ IAXTransaction* IAXEngine::addFrame(const SocketAddr& addr, IAXFrame* frame)
 		return 0;
 	    }
 	    // Complete transaction
-	    if (tr->processFrame(frame)) {
-		tr->m_rCallNo = frame->sourceCallNo();
-		m_incompleteTransList.remove(tr,false);
-		m_transList[frame->sourceCallNo() % m_transListCount]->append(tr);
-		XDebug(this,DebugAll,"New incomplete outgoing transaction completed (%u,%u)",
-		    tr->localCallNo(),tr->remoteCallNo());
-		return tr;
-	    }
-	    break;
+	    tr->m_rCallNo = frame->sourceCallNo();
+	    m_incompleteTransList.remove(tr,false);
+	    m_transList[frame->sourceCallNo() % m_transListCount]->append(tr);
+	    XDebug(this,DebugAll,"New incomplete outgoing transaction completed (%u,%u) [%p]",
+		tr->localCallNo(),tr->remoteCallNo(),this);
+	    return tr->processFrame(frame);
 	}
     }
     // Complete transactions
     l = m_transList[frame->sourceCallNo() % m_transListCount];
-    for (; l; l = l->next()) {
+    if (l)
+	l = l->skipNull();
+    for (; l; l = l->skipNext()) {
 	tr = static_cast<IAXTransaction*>(l->get());
-	if (!(tr && tr->remoteCallNo() == frame->sourceCallNo()))
+	if (tr->remoteCallNo() != frame->sourceCallNo())
 	    continue;
 	// Mini frame
-	if (!fullFrame) {
+	if (!full) {
 	    if (addr == tr->remoteAddr()) {
 		// keep transaction referenced but unlock the engine
 		RefPointer<IAXTransaction> t = tr;
@@ -199,7 +171,7 @@ IAXTransaction* IAXEngine::addFrame(const SocketAddr& addr, IAXFrame* frame)
 	}
 	// Full frame
 	// Has a local number assigned? If not, test socket
-	if (fullFrame->destCallNo() || addr == tr->remoteAddr()) {
+	if (full->destCallNo() || addr == tr->remoteAddr()) {
 	    // keep transaction referenced but unlock the engine
 	    RefPointer<IAXTransaction> t = tr;
 	    lock.drop();
@@ -207,13 +179,21 @@ IAXTransaction* IAXEngine::addFrame(const SocketAddr& addr, IAXFrame* frame)
 	}
     }
     // Frame doesn't belong to an existing transaction
-    // Test if it is a full frame with an IAX control message that needs a new transaction
-    if (!fullFrame || frame->type() != IAXFrame::IAX)
+    if (exiting()) {
+	sendInval(full,addr);
 	return 0;
-    switch (fullFrame->subclass()) {
+    }
+    // Test if it is a full frame with an IAX control message that needs a new transaction
+    if (!full || frame->type() != IAXFrame::IAX) {
+	if (full)
+	    sendInval(full,addr);
+	return 0;
+    }
+    switch (full->subclass()) {
 	case IAXControl::New:
-	     if (!checkCallToken(addr,*fullFrame))
+	    if (!checkCallToken(addr,*full))
 		return 0;
+	    break;
 	case IAXControl::RegReq:
 	case IAXControl::RegRel:
 	case IAXControl::Poke:
@@ -225,28 +205,30 @@ IAXTransaction* IAXEngine::addFrame(const SocketAddr& addr, IAXFrame* frame)
 	    // These are often used as keepalives
 	    return 0;
 	default:
-	    if (fullFrame) {
-	        if (fullFrame->destCallNo() == 0)
-		    Debug(this,DebugAll,"Unsupported incoming transaction Frame(%u,%u). Source call no: %u",
-			frame->type(),fullFrame->subclass(),fullFrame->sourceCallNo());
-		else
-		    Debug(this,DebugAll,"Unmatched Frame(%u,%u) for (%u,%u)",
-			frame->type(),fullFrame->subclass(),fullFrame->destCallNo(),fullFrame->sourceCallNo());
-
-		sendInval(fullFrame,addr);
-	    }
+	    if (full->destCallNo() == 0)
+		Debug(this,DebugAll,
+		    "Unsupported incoming transaction Frame(%u,%u). Source call no: %u [%p]",
+		    frame->type(),full->subclass(),full->sourceCallNo(),this);
+	    else
+		Debug(this,DebugAll,"Unmatched Frame(%u,%u) for (%u,%u) [%p]",
+		    frame->type(),full->subclass(),full->destCallNo(),
+		    full->sourceCallNo(),this);
+	    sendInval(full,addr);
 	    return 0;
     }
     // Generate local number
     u_int16_t lcn = generateCallNo();
-    if (!lcn)
-	return 0;
-    // Create and add transaction
-    tr = IAXTransaction::factoryIn(this,fullFrame,lcn,addr);
-    if (tr)
-	m_transList[frame->sourceCallNo() % m_transListCount]->append(tr);
-    else
-	releaseCallNo(lcn);
+    if (lcn) {
+	// Create and add transaction
+	tr = IAXTransaction::factoryIn(this,full,lcn,addr);
+	if (tr)
+	    m_transList[frame->sourceCallNo() % m_transListCount]->append(tr);
+	else
+	    releaseCallNo(lcn);
+    }
+    if (!tr)
+	Debug(this,DebugInfo,"Failed to build incoming transaction for Frame(%u,%u) [%p]",
+	    frame->type(),full->subclass(),this);
     return tr;
 }
 
@@ -260,7 +242,7 @@ IAXTransaction* IAXEngine::addFrame(const SocketAddr& addr, const unsigned char*
 	SocketAddr local;
 	m_socket.getSockName(local);
 	frame->fullFrame()->toString(s,local,addr,true);
-	Debug(this,DebugInfo,"Received frame.%s",s.c_str());
+	Debug(this,DebugInfo,"Received frame [%p]%s",this,s.c_str());
     }
     IAXTransaction* tr = addFrame(addr,frame);
     if (!tr)
@@ -285,8 +267,12 @@ void IAXEngine::sendInval(IAXFullFrame* frame, const SocketAddr& addr)
 {
     if (!frame)
 	return;
-    DDebug(this,DebugInfo,"Sending INVAL for unmatched frame(%u,%u) with OSeq=%u ISeq=%u",frame->type(),frame->subclass(),
-	frame->oSeqNo(),frame->iSeqNo());
+    // Check for frames that should not receive INVAL
+    if (frame->type() == IAXFrame::IAX && frame->subclass() == IAXControl::Inval)
+	return;
+    DDebug(this,DebugInfo,
+	"Sending INVAL for unmatched frame(%u,%u) with OSeq=%u ISeq=%u [%p]",
+	frame->type(),frame->subclass(),frame->oSeqNo(),frame->iSeqNo(),this);
     IAXFullFrame* f = new IAXFullFrame(IAXFrame::IAX,IAXControl::Inval,frame->destCallNo(),
 	frame->sourceCallNo(),frame->iSeqNo(),frame->oSeqNo(),frame->timeStamp());
     writeSocket(f->data().data(),f->data().length(),addr,f);
@@ -297,13 +283,13 @@ bool IAXEngine::process()
 {
     bool ok = false;
     for (;;) {
-	IAXEvent* event = getEvent(Time::msecNow());
+	IAXEvent* event = getEvent();
 	if (!event)
 	    break;
 	ok = true;
 	if ((event->final() && !event->frameType()) || !event->getTransaction()) {
-	    XDebug(this,DebugAll,"Deleting internal event type %u Frame(%u,%u)",
-		event->type(),event->frameType(),event->subclass());
+	    XDebug(this,DebugAll,"Deleting internal event type %u Frame(%u,%u) [%p]",
+		event->type(),event->frameType(),event->subclass(),this);
 	    delete event;
 	    continue;
 	}
@@ -312,18 +298,107 @@ bool IAXEngine::process()
     return ok;
 }
 
+static inline void roundUp10(unsigned int& value)
+{
+    unsigned int rest = value % 10;
+    if (rest)
+	value += 10 - rest;
+}
+
+// Initialize outgoing data timestamp adjust values
+void IAXEngine::initOutDataAdjust(const NamedList& params, IAXTransaction* tr)
+{
+    const String* thresS = 0;
+    const String* overS = 0;
+    const String* underS = 0;
+    NamedIterator iter(params);
+    for (const NamedString* ns = 0; 0 != (ns = iter.get());) {
+	if (ns->name() == YSTRING("adjust_ts_out_threshold"))
+	    thresS = ns;
+	else if (ns->name() == YSTRING("adjust_ts_out_over"))
+	    overS = ns;
+	else if (ns->name() == YSTRING("adjust_ts_out_under"))
+	    underS = ns;
+    }
+    // No need to set transaction's data if no parameter found
+    if (tr && !(thresS || overS || underS))
+	return;
+    Lock lck(tr ? (Mutex*)tr : (Mutex*)this);
+    unsigned int thresDef = IAX2_ADJUSTTSOUT_THRES;
+    unsigned int overDef = IAX2_ADJUSTTSOUT_OVER;
+    unsigned int underDef = IAX2_ADJUSTTSOUT_UNDER;
+    if (tr) {
+	thresDef = tr->m_adjustTsOutThreshold;
+	overDef = tr->m_adjustTsOutOverrun;
+	underDef = tr->m_adjustTsOutUnderrun;
+    }
+    unsigned int thres = thresS ? thresS->toInteger(thresDef,0,20,300) : thresDef;
+    unsigned int over = overS ? overS->toInteger(overDef,0,10) : overDef;
+    unsigned int under = underS ? underS->toInteger(underDef,0,10) : underDef;
+    bool adjusted = false;
+    // Round down to multiple of 10
+    roundUp10(thres);
+    roundUp10(over);
+    roundUp10(under);
+    // Overrun must not be greater then threshold
+    if (over > thres) {
+	over = thres;
+	adjusted = true;
+    }
+    // Underrun must be less then 2 * threshold
+    unsigned int doubleThres = 2 * thres;
+    if (under >= doubleThres) {
+	under = doubleThres - 10;
+	adjusted = true;
+    }
+    if (tr) {
+	tr->m_adjustTsOutThreshold = thres;
+	tr->m_adjustTsOutOverrun = over;
+	tr->m_adjustTsOutUnderrun = under;
+	Debug(this,DebugAll,
+	    "Transaction(%u,%u) adjust ts out set to thres=%u over=%u under=%u [%p]",
+	    tr->localCallNo(),tr->remoteCallNo(),thres,over,under,tr);
+	return;
+    }
+    m_adjustTsOutThreshold = thres;
+    m_adjustTsOutOverrun = over;
+    m_adjustTsOutUnderrun = under;
+    if (adjusted)
+	Debug(this,DebugConf,
+	    "Adjust ts out set to thres=%u over=%u under=%u from thres=%s over=%s under=%s [%p]",
+	    thres,over,under,TelEngine::c_safe(thresS),
+	    TelEngine::c_safe(overS),TelEngine::c_safe(underS),this);
+    else
+	Debug(this,DebugAll,"Adjust ts out set to thres=%u over=%u under=%u [%p]",
+	    thres,over,under,this);
+}
+
 // (Re)Initialize the engine
 void IAXEngine::initialize(const NamedList& params)
 {
     m_callToken = params.getBoolValue("calltoken_in");
-    int callTokenAge = params.getIntValue("calltoken_age",10);
-    if (callTokenAge > 1 && callTokenAge < 25)
-	m_callTokenAge = callTokenAge;
-    else
-	m_callTokenAge = 10;
+    m_callTokenAge = params.getIntValue("calltoken_age",10,1,25);
     m_showCallTokenFailures = params.getBoolValue("calltoken_printfailure");
     m_rejectMissingCallToken = params.getBoolValue("calltoken_rejectmissing",true);
     m_printMsg = params.getBoolValue("printmsg",true);
+    m_callerNumType = lookup(params["numtype"],IAXInfoElement::s_typeOfNumber);
+    m_callingPres = lookup(params["presentation"],IAXInfoElement::s_presentation) |
+	lookup(params["screening"],IAXInfoElement::s_screening);
+    m_challengeTout = params.getIntValue("challenge_timeout",
+	IAX2_CHALLENGETOUT_DEF,IAX2_CHALLENGETOUT_MIN);
+    initOutDataAdjust(params);
+    IAXTrunkInfo* ti = new IAXTrunkInfo;
+    ti->initTrunking(params,"trunk_");
+    ti->init(params);
+    Lock lck(m_trunkInfoMutex);
+    m_trunkInfoDef = ti;
+#ifdef XDEBUG
+    String tiS;
+    m_trunkInfoDef->dump(tiS,"\r\n");
+    Debug(this,DebugAll,"Initialized trunk info defaults: [%p]\r\n-----\r\n%s\r\n-----",
+	this,tiS.c_str());
+#endif
+    TelEngine::destruct(ti);
 }
 
 void IAXEngine::readSocket(SocketAddr& addr)
@@ -331,15 +406,17 @@ void IAXEngine::readSocket(SocketAddr& addr)
     unsigned char buf[1500];
 
     while (1) {
+	if (Thread::check(false))
+	    break;
 	int len = m_socket.recvFrom(buf,sizeof(buf),addr);
 	if (len == Socket::socketError()) {
 	    if (!m_socket.canRetry()) {
 		String tmp;
 		Thread::errorString(tmp,m_socket.error());
-		Debug(this,DebugWarn,"Socket read error: %s (%d)",
-		    tmp.c_str(),m_socket.error());
+		Debug(this,DebugWarn,"Socket read error: %s (%d) [%p]",
+		    tmp.c_str(),m_socket.error(),this);
 	    }
-	    Thread::idle(true);
+	    Thread::idle(false);
 	    continue;
 	}
 	addFrame(addr,buf,len);
@@ -354,22 +431,22 @@ bool IAXEngine::writeSocket(const void* buf, int len, const SocketAddr& addr,
 	SocketAddr local;
 	m_socket.getSockName(local);
 	frame->toString(s,local,addr,false);
-	Debug(this,DebugInfo,"Sending frame.%s",s.c_str());
+	Debug(this,DebugInfo,"Sending frame [%p]%s",this,s.c_str());
     }
     len = m_socket.sendTo(buf,len,addr);
     if (len == Socket::socketError()) {
 	if (!m_socket.canRetry()) {
 	    String tmp;
 	    Thread::errorString(tmp,m_socket.error());
-	    Debug(this,DebugWarn,"Socket write error: %s (%d)",
-		tmp.c_str(),m_socket.error());
+	    Alarm(this,"socket",DebugWarn,"Socket write error: %s (%d) [%p]",
+		tmp.c_str(),m_socket.error(),this);
 	}
 #ifdef DEBUG
 	else {
 	    String tmp;
 	    Thread::errorString(tmp,m_socket.error());
-	    Debug(this,DebugMild,"Socket temporary unavailable: %s (%d)",
-		tmp.c_str(),m_socket.error());
+	    Debug(this,DebugMild,"Socket temporary unavailable: %s (%d) [%p]",
+		tmp.c_str(),m_socket.error(),this);
 	}
 #endif
 	return false;
@@ -382,10 +459,10 @@ bool IAXEngine::writeSocket(const void* buf, int len, const SocketAddr& addr,
 void IAXEngine::runGetEvents()
 {
     while (1) {
-	if (!process()) {
-	    Thread::idle(true);
-	    continue;
-	}
+	if (Thread::check(false))
+	    break;
+	if (!process())
+	    Thread::idle(false);
     }
 }
 
@@ -397,20 +474,35 @@ void IAXEngine::removeTransaction(IAXTransaction* transaction)
     releaseCallNo(transaction->localCallNo());
     if (!m_incompleteTransList.remove(transaction,false)) {
 	if (m_transList[transaction->remoteCallNo() % m_transListCount]->remove(transaction,false)) {
-	    DDebug(this,DebugAll,"Transaction(%u,%u) removed",
-		transaction->localCallNo(),transaction->remoteCallNo());
+	    DDebug(this,DebugAll,"Transaction(%u,%u) removed [%p]",
+		transaction->localCallNo(),transaction->remoteCallNo(),this);
 	}
 	else {
-	    DDebug(this,DebugAll,"Trying to remove transaction(%u,%u) but does not exist",
-		transaction->localCallNo(),transaction->remoteCallNo());
+	    DDebug(this,DebugAll,
+		"Trying to remove transaction(%u,%u) but does not exist [%p]",
+		transaction->localCallNo(),transaction->remoteCallNo(),this);
 	}
     }
     else {
-	DDebug(this,DebugAll,"Transaction(%u,%u) (incomplete outgoing) removed",
-	    transaction->localCallNo(),transaction->remoteCallNo());
+	DDebug(this,DebugAll,"Transaction(%u,%u) (incomplete outgoing) removed [%p]",
+	    transaction->localCallNo(),transaction->remoteCallNo(),this);
     }
 }
 
+// Check if there are any transactions in the engine
+bool IAXEngine::haveTransactions()
+{
+    Lock lock(this);
+    // Incomplete transactions
+    if (m_incompleteTransList.skipNull())
+	return true;
+    // Complete transactions
+    for (int i = 0; i < m_transListCount; i++)
+	if (m_transList[i]->skipNull())
+	    return true;
+    return false;
+}
+
 u_int32_t IAXEngine::transactionCount()
 {
     u_int32_t n = 0;
@@ -424,7 +516,7 @@ u_int32_t IAXEngine::transactionCount()
     return n;
 }
 
-void IAXEngine::keepAlive(SocketAddr& addr)
+void IAXEngine::keepAlive(const SocketAddr& addr)
 {
 #if 0
     unsigned char buf[12] = {0x80,0,0,0,0,0,0,0,0,0,IAXFrame::IAX,IAXControl::Inval};
@@ -435,33 +527,61 @@ void IAXEngine::keepAlive(SocketAddr& addr)
     f->deref();
 }
 
-bool IAXEngine::processTrunkFrames(u_int32_t time)
+// Decode a DATETIME value
+void IAXEngine::decodeDateTime(u_int32_t dt, unsigned int& year, unsigned int& month,
+    unsigned int& day, unsigned int& hour, unsigned int& minute, unsigned int& sec)
 {
-    Lock lock(&m_mutexTrunk);
+   // RFC 5456 Section 8.6.28
+   year = 2000 + ((dt & 0xfe000000) >> 25);
+   month = (dt & 0x1e00000) >> 21;
+   day = (dt & 0x1f0000) >> 16;
+   hour = (dt & 0xf800) >> 11;
+   minute = (dt & 0x7e0) >> 5;
+   sec = dt & 0x1f;
+}
+
+// Calculate overall timeout from interval and retransmission counter
+unsigned int IAXEngine::overallTout(unsigned int interval, unsigned int nRetrans)
+{
+    unsigned int tmp = interval;
+    for (unsigned int i = 1; i <= nRetrans; i++)
+	tmp += interval * (1 << i);
+    return tmp;
+}
+
+bool IAXEngine::processTrunkFrames(const Time& time)
+{
+    Lock lck(m_mutexTrunk);
     bool sent = false;
-    for (ObjList* l = m_trunkList.skipNull(); l; l = l->skipNext()) {
+    for (ObjList* l = m_trunkList.skipNull(); l;) {
+	if (Thread::check(false))
+	    break;
 	IAXMetaTrunkFrame* frame = static_cast<IAXMetaTrunkFrame*>(l->get());
-	// Frame has mini frame(s) ?
-	if (!frame->timestamp())
+	if (frame->refcount() != 1) {
+	    l = l->skipNext();
+	    if (frame->timerTick(time))
+		sent = true;
 	    continue;
-	int32_t interval = time - frame->timestamp();
-        if (!interval || (interval && (u_int32_t)interval < m_trunkSendInterval))
-	    continue;
-	// If the time wrapped around, send it. Worst case: we'll send an empty frame
-	frame->send(time);
-	sent = true;
+	}
+	Debug(this,DebugAll,
+	    "Removing trunk frame (%p) '%s:%d' timestamps=%s maxlen=%u interval=%ums [%p]",
+	    frame,frame->addr().host().c_str(),frame->addr().port(),
+	    String::boolText(frame->trunkTimestamps()),frame->maxLen(),
+	    frame->sendInterval(),this);
+	l->remove();
+	l = l->skipNull();
     }
     return sent;
 }
 
 void IAXEngine::processEvent(IAXEvent* event)
 {
-    XDebug(this,DebugAll,"Default processing - deleting event %p Subclass %u",
-	event,event->subclass());
+    XDebug(this,DebugAll,"Default processing - deleting event %p Subclass %u [%p]",
+	event,event->subclass(),this);
     delete event;
 }
 
-IAXEvent* IAXEngine::getEvent(u_int64_t time)
+IAXEvent* IAXEngine::getEvent(const Time& now)
 {
     IAXTransaction* tr;
     IAXEvent* ev;
@@ -471,15 +591,18 @@ IAXEvent* IAXEngine::getEvent(u_int64_t time)
     // Find for incomplete transactions
     l = m_incompleteTransList.skipNull();
     for (; l; l = l->next()) {
+	if (Thread::check(false))
+	    break;
 	tr = static_cast<IAXTransaction*>(l->get());
-	if (tr && 0 != (ev = tr->getEvent(time))) {
+	if (tr && 0 != (ev = tr->getEvent(now))) {
 	    unlock();
 	    return ev;
 	}
-	continue;
     }
     // Find for complete transactions, start with current index
     while (m_lastGetEvIndex < m_transListCount) {
+	if (Thread::check(false))
+	    break;
 	l = m_transList[m_lastGetEvIndex++]->skipNull();
 	if (!l)
 	    continue;
@@ -494,7 +617,7 @@ IAXEvent* IAXEngine::getEvent(u_int64_t time)
 	    if (!t)
 		continue;
 	    unlock();
-	    if (0 != (ev = t->getEvent(time)))
+	    if (0 != (ev = t->getEvent(now)))
 		return ev;
 	    lock();
 	}
@@ -522,7 +645,8 @@ u_int16_t IAXEngine::generateCallNo()
 	    m_lUsedCallNo[i] = true;
 	    return i;
 	}
-    Debug(this,DebugWarn,"Unable to generate call number. Transaction count: %u",transactionCount());
+    Debug(this,DebugWarn,"Unable to generate call number. Transaction count: %u [%p]",
+	transactionCount(),this);
     return 0;
 }
 
@@ -531,28 +655,95 @@ void IAXEngine::releaseCallNo(u_int16_t lcallno)
     m_lUsedCallNo[lcallno] = false;
 }
 
-IAXTransaction* IAXEngine::startLocalTransaction(IAXTransaction::Type type, const SocketAddr& addr, IAXIEList& ieList, bool trunking)
+IAXTransaction* IAXEngine::startLocalTransaction(IAXTransaction::Type type,
+    const SocketAddr& addr, IAXIEList& ieList, bool refTrans, bool startTrans)
 {
-    Lock lock(this);
+    Lock lck(this);
+    if (exiting())
+	return 0;
     u_int16_t lcn = generateCallNo();
     if (!lcn)
 	return 0;
     IAXTransaction* tr = IAXTransaction::factoryOut(this,type,lcn,addr,ieList);
     if (tr) {
-	m_incompleteTransList.append(tr);
-	if (trunking)
-	    enableTrunking(tr);
+	if (!refTrans || tr->ref()) {
+	    m_incompleteTransList.append(tr);
+	    if (startTrans)
+		tr->start();
+	}
+	else
+	    TelEngine::destruct(tr);
     }
-    else
+    if (!tr)
 	releaseCallNo(lcn);
     return tr;
 }
 
+// Bind the socket. Terminate it before trying
+bool IAXEngine::bind(const char* iface, int port, bool force)
+{
+    if (m_socket.valid())
+	m_socket.terminate();
+    m_addr.clear();
+    if (!m_socket.create(AF_INET,SOCK_DGRAM)) {
+	String tmp;
+	Thread::errorString(tmp,m_socket.error());
+	Alarm(this,"socket",DebugWarn,"Failed to create socket. %d: '%s' [%p]",
+	    m_socket.error(),tmp.c_str(),this);
+	return false;
+    }
+    if (!m_socket.setBlocking(false)) {
+	String tmp;
+	Thread::errorString(tmp,m_socket.error());
+	Alarm(this,"socket",DebugWarn,
+	    "Failed to set socket non blocking operation mode. %d: '%s' [%p]",
+	    m_socket.error(),tmp.c_str(),this);
+	m_socket.terminate();
+	return false;
+    }
+    SocketAddr addr(AF_INET);
+    addr.host(iface);
+    addr.port(port ? port : 4569);
+    bool ok = m_socket.bind(addr);
+    if (!ok) {
+	String tmp;
+	Thread::errorString(tmp,m_socket.error());
+	Alarm(this,"socket",DebugWarn,"Failed to bind socket on '%s:%d'%s. %d: '%s' [%p]",
+	    c_safe(iface),port,force ? " - trying a random port" : "",
+	    m_socket.error(),tmp.c_str(),this);
+	if (force) {
+	    addr.port(0);
+	    ok = m_socket.bind(addr);
+	    if (!ok)
+		Alarm(this,"socket",DebugWarn,"Failed to bind on any port for iface='%s' [%p]",
+		    iface,this);
+	    else {
+		ok = m_socket.getSockName(addr);
+		if (!ok)
+		    Debug(this,DebugWarn,"Failed to retrieve bound address [%p]",this);
+	    }
+	}
+    }
+    if (!ok) {
+	m_socket.terminate();
+	return false;
+    }
+    m_addr = addr;
+    if (!m_addr.host())
+	m_addr.host("0.0.0.0");
+    String s;
+    if (addr.host() != iface && !TelEngine::null(iface))
+	s << " (" << iface << ")";
+    Debug(this,DebugInfo,"Bound on '%s:%d'%s [%p]",
+	m_addr.host().c_str(),m_addr.port(),s.safe(),this);
+    return true;
+}
+
 // Check call token on incoming call requests.
 bool IAXEngine::checkCallToken(const SocketAddr& addr, IAXFullFrame& frame)
 {
-    XDebug(this,DebugAll,"IAXEngine::checkCallToken('%s:%d') calltoken=%u",
-	addr.host().c_str(),addr.port(),m_callToken);
+    XDebug(this,DebugAll,"IAXEngine::checkCallToken('%s:%d') calltoken=%u [%p]",
+	addr.host().c_str(),addr.port(),m_callToken,this);
     if (!m_callToken)
 	return true;
     frame.updateIEList(true);
@@ -564,9 +755,9 @@ bool IAXEngine::checkCallToken(const SocketAddr& addr, IAXFullFrame& frame)
     if (!ct) {
 	if (m_showCallTokenFailures)
 	    Debug(this,DebugNote,
-		"Missing required %s parameter in call request %u from '%s:%d'",
+		"Missing required %s parameter in call request %u from '%s:%d' [%p]",
 		IAXInfoElement::ieText(IAXInfoElement::CALLTOKEN),frame.sourceCallNo(),
-		addr.host().c_str(),addr.port());
+		addr.host().c_str(),addr.port(),this);
 	if (m_rejectMissingCallToken) {
 	    IAXIEList* ies = new IAXIEList;
 	    ies->appendString(IAXInfoElement::CAUSE,"CALLTOKEN support required");
@@ -582,15 +773,15 @@ bool IAXEngine::checkCallToken(const SocketAddr& addr, IAXFullFrame& frame)
     if (ct->data().length()) {
 	String tmp((char*)ct->data().data(),ct->data().length());
 	int age = addrSecretAge(tmp,m_callTokenSecret,addr);
-	XDebug(this,DebugAll,"Call request %u from '%s:%d' with call token age=%d",
-	    frame.sourceCallNo(),addr.host().c_str(),addr.port(),age);
+	XDebug(this,DebugAll,"Call request %u from '%s:%d' with call token age=%d [%p]",
+	    frame.sourceCallNo(),addr.host().c_str(),addr.port(),age,this);
 	if (age >= 0 && age <= m_callTokenAge)
 	    return true;
 	if (m_showCallTokenFailures)
 	    Debug(this,DebugNote,
-		"Ignoring call request %u from '%s:%d' with %s call token age=%d",
+		"Ignoring call request %u from '%s:%d' with %s call token age=%d [%p]",
 		frame.sourceCallNo(),addr.host().c_str(),addr.port(),
-		(age > 0) ? "old" : "invalid",age);
+		(age > 0) ? "old" : "invalid",age,this);
 	return false;
     }
     // Request with empty call token: send one
@@ -612,8 +803,8 @@ bool IAXEngine::acceptFormatAndCapability(IAXTransaction* trans, unsigned int* c
     u_int32_t transCapsNonType = IAXFormat::clear(trans->m_capability,type);
     IAXFormat* fmt = trans->getFormat(type);
     if (!fmt) {
-	DDebug(this,DebugStub,"acceptFormatAndCapability() No media %s in transaction",
-	    IAXFormat::typeName(type));
+	DDebug(this,DebugStub,"acceptFormatAndCapability() No media %s in transaction [%p]",
+	    IAXFormat::typeName(type),this);
 	trans->m_capability = transCapsNonType;
 	return false;
     }
@@ -623,10 +814,10 @@ bool IAXEngine::acceptFormatAndCapability(IAXTransaction* trans, unsigned int* c
 	capability &= IAXFormat::mask(*caps,type);
     trans->m_capability = transCapsNonType | capability;
     XDebug(this,DebugAll,
-	"acceptFormatAndCapability trans(%u,%u) type=%s caps(trans/our/param/result)=%u/%u/%u/%u",
+	"acceptFormatAndCapability trans(%u,%u) type=%s caps(trans/our/param/result)=%u/%u/%u/%u [%p]",
 	trans->localCallNo(),trans->remoteCallNo(),
 	fmt->typeName(),transCapsType,IAXFormat::mask(m_capability,type),
-	caps ? IAXFormat::mask(*caps,type) : 0,capability);
+	caps ? IAXFormat::mask(*caps,type) : 0,capability,this);
     // Valid capability ?
     if (!capability) {
 	// Warn if we should have media
@@ -661,8 +852,9 @@ bool IAXEngine::acceptFormatAndCapability(IAXTransaction* trans, unsigned int* c
 
 void IAXEngine::defaultEventHandler(IAXEvent* event)
 {
-    DDebug(this,DebugAll,"defaultEventHandler - Event type: %u. Frame - Type: %u Subclass: %u",
-	event->type(),event->frameType(),event->subclass());
+    DDebug(this,DebugAll,
+	"defaultEventHandler - Event type: %u. Frame - Type: %u Subclass: %u [%p]",
+	event->type(),event->frameType(),event->subclass(),this);
     IAXTransaction* tr = event->getTransaction();
     switch (event->type()) {
 	case IAXEvent::New:
@@ -672,38 +864,107 @@ void IAXEngine::defaultEventHandler(IAXEvent* event)
     }
 }
 
-void IAXEngine::enableTrunking(IAXTransaction* trans)
+// Set the exiting flag
+void IAXEngine::setExiting()
+{
+    Lock lck(this);
+    m_exiting = true;
+}
+
+static bool getTrunkingInfo(RefPointer<IAXTrunkInfo>& ti, IAXEngine* engine,
+    const NamedList* params, const String& prefix, bool out)
+{
+    if (!engine->trunkInfo(ti))
+	return false;
+    if (!params)
+	return true;
+    IAXTrunkInfo* tmp = new IAXTrunkInfo;
+    tmp->initTrunking(*params,prefix,ti,out,!out);
+    ti = tmp;
+    TelEngine::destruct(tmp);
+    return true;
+}
+
+void IAXEngine::enableTrunking(IAXTransaction* trans, const NamedList* params,
+    const String& prefix)
 {
     if (!trans || trans->type() != IAXTransaction::New)
 	return;
-    Lock lock(&m_mutexTrunk);
+    RefPointer<IAXTrunkInfo> ti;
+    if (getTrunkingInfo(ti,this,params,prefix,true))
+	enableTrunking(trans,*ti);
+    ti = 0;
+}
+
+// Enable trunking for the given transaction. Allocate a trunk meta frame if needed.
+void IAXEngine::enableTrunking(IAXTransaction* trans, IAXTrunkInfo& data)
+{
+    if (!trans || trans->type() != IAXTransaction::New)
+	return;
+    Lock lock(m_mutexTrunk);
+    if (m_trunking >= 0) {
+	m_trunking++;
+	if (m_trunking == 1 || 0 == ((m_trunking - 1) % 200))
+	    Debug(this,DebugNote,"Failed to enable trunking: not available [%p]",this);
+	return;
+    }
     IAXMetaTrunkFrame* frame;
     // Already enabled ?
-    for (ObjList* l = m_trunkList.skipNull(); l; l = l->next()) {
+    for (ObjList* l = m_trunkList.skipNull(); l; l = l->skipNext()) {
 	frame = static_cast<IAXMetaTrunkFrame*>(l->get());
-	if (frame && frame->addr() == trans->remoteAddr()) {
-	    trans->enableTrunking(frame);
+	if (frame->addr() == trans->remoteAddr()) {
+	    trans->enableTrunking(frame,data.m_efficientUse);
 	    return;
 	}
     }
-    frame = new IAXMetaTrunkFrame(this,trans->remoteAddr());
-    if (trans->enableTrunking(frame))
+    frame = new IAXMetaTrunkFrame(this,trans->remoteAddr(),data.m_timestamps,
+	data.m_maxLen,data.m_sendInterval);
+    if (trans->enableTrunking(frame,data.m_efficientUse)) {
 	m_trunkList.append(frame);
-    // Deref frame: Only transactions are allowed to keep references for it
-    frame->deref();
+	Debug(this,DebugAll,
+	    "Added trunk frame (%p) '%s:%d' timestamps=%s maxlen=%u interval=%ums [%p]",
+	    frame,frame->addr().host().c_str(),frame->addr().port(),
+	    String::boolText(frame->trunkTimestamps()),frame->maxLen(),
+	    frame->sendInterval(),this);
+    }
+    else
+	TelEngine::destruct(frame);
 }
 
-void IAXEngine::removeTrunkFrame(IAXMetaTrunkFrame* trunkFrame)
+// Init incoming trunking data for a given transaction
+void IAXEngine::initTrunkIn(IAXTransaction* trans, const NamedList* params,
+    const String& prefix)
 {
-    Lock lock(&m_mutexTrunk);
-    m_trunkList.remove(trunkFrame,false);
+    if (!trans)
+	return;
+    RefPointer<IAXTrunkInfo> ti;
+    if (getTrunkingInfo(ti,this,params,prefix,false))
+	initTrunkIn(trans,*ti);
+    ti = 0;
+}
+
+// Init incoming trunking data for a given transaction
+void IAXEngine::initTrunkIn(IAXTransaction* trans, IAXTrunkInfo& data)
+{
+    if (!trans)
+	return;
+    trans->m_trunkInSyncUsingTs = data.m_trunkInSyncUsingTs;
+    trans->m_trunkInTsDiffRestart = data.m_trunkInTsDiffRestart;
+#ifdef XDEBUG
+    String tmp;
+    data.dump(tmp," ",false,true,false);
+    Debug(this,DebugAll,"initTrunkIn(%p) callno=%u set %s [%p]",
+	trans,trans->localCallNo(),tmp.c_str(),this);
+#endif
 }
 
 void IAXEngine::runProcessTrunkFrames()
 {
     while (1) {
+	if (Thread::check(false))
+	    break;
 	processTrunkFrames();
-	Thread::msleep(2,true);
+	Thread::msleep(2,false);
     }
 }
 
