@@ -4,21 +4,18 @@
  * This file is part of the YATE Project http://YATE.null.ro
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include <yatejabber.h>
@@ -2218,7 +2215,7 @@ bool JBStreamSetList::add(JBStream* client)
     if (!client || m_engine->exiting())
 	return false;
     Lock lock(this);
-    for (ObjList* o = m_sets.skipNull(); o; o = o->skipNull()) {
+    for (ObjList* o = m_sets.skipNull(); o; o = o->skipNext()) {
 	if ((static_cast<JBStreamSet*>(o->get()))->add(client)) {
 	    m_streamCount++;
 	    return true;
@@ -2562,7 +2559,7 @@ bool JBEntityCapsList::processCaps(String& capsId, XmlElement* xml, JBStream* st
 void JBEntityCapsList::addCaps(NamedList& list, JBEntityCaps& caps)
 {
 #define CHECK_NS(ns,param) \
-    if (caps.m_features.get(ns)) { \
+    if (caps.hasFeature(ns)) { \
 	params->append(param,","); \
 	list.addParam(param,String::boolText(true)); \
     }
@@ -2590,6 +2587,8 @@ void JBEntityCapsList::addCaps(NamedList& list, JBEntityCaps& caps)
 	    case 0:
 		break;
 	}
+	CHECK_NS(XMPPNamespace::FileInfoShare,"caps.fileinfoshare");
+	CHECK_NS(XMPPNamespace::ResultSetMngt,"caps.resultsetmngt");
     }
     CHECK_NS(XMPPNamespace::Muc,"caps.muc");
 #undef CHECK_NS
