@@ -4,21 +4,18 @@
  * This file is part of the YATE Project http://YATE.null.ro
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include <yatejingle.h>
@@ -971,7 +968,7 @@ JGStreamHost* JGStreamHost::fromXml(XmlElement* xml)
     const char* jid = xml->attribute("jid");
     if (TelEngine::null(jid))
 	return 0;
-    return new JGStreamHost(jid,xml->attribute("host"),
+    return new JGStreamHost(false,jid,xml->attribute("host"),
 	String(xml->attribute("port")).toInteger(-1),xml->attribute("zeroconf"));
 }
 
@@ -1363,7 +1360,7 @@ void JGSession::destroyed()
 
 // Send a stanza to the remote peer
 bool JGSession::sendStanza(XmlElement* stanza, String* stanzaId, bool confirmation,
-    bool ping)
+    bool ping, unsigned int toutMs)
 {
     if (!stanza)
 	return false;
@@ -1398,12 +1395,24 @@ bool JGSession::sendStanza(XmlElement* stanza, String* stanzaId, bool confirmati
 	}
 	String id = m_localSid;
 	id << "_" << (unsigned int)m_stanzaId++;
-	JGSentStanza* sent = new JGSentStanza(id,
-	    m_engine->stanzaTimeout() + Time::msecNow(),stanzaId != 0,ping,act);
+	u_int64_t tout = Time::msecNow() + (toutMs ? toutMs : m_engine->stanzaTimeout());
+	JGSentStanza* sent = new JGSentStanza(id,tout,stanzaId != 0,ping,act);
 	stanza->setAttribute("id",*sent);
 	if (stanzaId)
 	    *stanzaId = *sent;
-	m_sentStanza.append(sent);
+	// Insert stanza in timeout ascending order
+	ObjList* last = &m_sentStanza;
+	for (ObjList* o = last->skipNull(); o; o = o->skipNext()) {
+	    JGSentStanza* tmp = static_cast<JGSentStanza*>(o->get());
+	    if (tout < tmp->timeout()) {
+		o->insert(sent);
+		sent = 0;
+		break;
+	    }
+	    last = o;
+	}
+	if (sent)
+	    last->append(sent);
     }
     return m_engine->sendStanza(this,stanza);
 }
@@ -2137,7 +2146,7 @@ bool JGSession1::sendStreamHosts(const ObjList& hosts, String* stanzaId)
 	return false;
     XmlElement* xml = XMPPUtils::createIq(XMPPUtils::IqSet,m_local,m_remote,0);
     xml->addChild(JGStreamHost::buildHosts(hosts,m_sid));
-    return sendStanza(xml,stanzaId);
+    return sendStanza(xml,stanzaId,true,false,(unsigned int)m_engine->streamHostTimeout());
 }
 
 // Send a stanza with a stream host used
