@@ -4,21 +4,18 @@
  * This file is part of the YATE Project http://YATE.null.ro
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #ifndef __YATEJABBER_H
@@ -2791,14 +2788,22 @@ public:
 	m_version(version), m_node(node), m_data(data)
 	{}
 
+    /**
+     * Check if a given feature is found in the list
+     * @param ns The feature to check
+     * @return True if the feature was found in the list
+     */
+    inline bool hasFeature(int ns)
+	{ return 0 != m_features.get(ns); }
+
     /**
      * Check if an audio capability is present
      * @return True if an audio capability is present
      */
     inline bool hasAudio() {
-	    return m_features.get(XMPPNamespace::JingleAppsRtpAudio) ||
-		m_features.get(XMPPNamespace::JingleAudio) ||
-		m_features.get(XMPPNamespace::JingleVoiceV1);
+	    return hasFeature(XMPPNamespace::JingleAppsRtpAudio) ||
+		hasFeature(XMPPNamespace::JingleAudio) ||
+		hasFeature(XMPPNamespace::JingleVoiceV1);
 	}
 
     /**
