@@ -4,21 +4,18 @@
  * This file is part of the YATE Project http://YATE.null.ro 
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include <yatemgcp.h>
@@ -195,7 +192,7 @@ void MGCPEngine::initialize(const NamedList& params)
 	}
 
 	if (!m_socket.bind(m_address)) {
-	    Debug(this,DebugWarn,"Failed to bind socket to %s:%d. Error: %d: %s",
+	    Alarm(this,"socket",DebugWarn,"Failed to bind socket to %s:%d. Error: %d: %s",
 		m_address.host().safe(),m_address.port(),
 		m_socket.error(),::strerror(m_socket.error()));
 	    m_socket.terminate();
@@ -626,7 +623,7 @@ bool MGCPEngine::sendData(const String& msg, const SocketAddr& address)
     if (len != Socket::socketError())
 	return true;
     if (!m_socket.canRetry())
-	Debug(this,DebugWarn,"Socket write error: %d: %s",
+	Alarm(this,"socket",DebugWarn,"Socket write error: %d: %s",
 	    m_socket.error(),::strerror(m_socket.error()));
     else
 	DDebug(this,DebugMild,"Socket temporary unavailable: %d: %s",
