@@ -3,21 +3,18 @@
  * This file is part of the YATE Project http://YATE.null.ro
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include "yateclass.h"
@@ -143,15 +140,26 @@ bool ExpExtender::runAssign(ObjList& stack, const ExpOperation& oper, GenObject*
     return false;
 }
 
+ParsePoint& ParsePoint::operator=(ParsePoint& parsePoint)
+{
+    m_expr = parsePoint.m_expr;
+    m_count = parsePoint.m_count;
+    m_searchedSeps = parsePoint.m_searchedSeps;
+    m_fileName = parsePoint.m_fileName;
+    m_lineNo = parsePoint.m_lineNo;
+    if (m_eval)
+	m_eval->m_lineNo = m_lineNo;
+    return *this;
+}
 
 ExpEvaluator::ExpEvaluator(const TokenDict* operators, const TokenDict* unaryOps)
-    : m_operators(operators), m_unaryOps(unaryOps),
+    : m_operators(operators), m_unaryOps(unaryOps), m_lastOpcode(&m_opcodes),
       m_inError(false), m_lineNo(1), m_extender(0)
 {
 }
 
 ExpEvaluator::ExpEvaluator(ExpEvaluator::Parser style)
-    : m_operators(0), m_unaryOps(0),
+    : m_operators(0), m_unaryOps(0), m_lastOpcode(&m_opcodes),
     m_inError(false), m_lineNo(1), m_extender(0)
 {
     switch (style) {
@@ -167,13 +175,13 @@ ExpEvaluator::ExpEvaluator(ExpEvaluator::Parser style)
 }
 
 ExpEvaluator::ExpEvaluator(const ExpEvaluator& original)
-    : m_operators(original.m_operators), m_unaryOps(original.unaryOps()),
+    : m_operators(original.m_operators), m_unaryOps(original.unaryOps()), m_lastOpcode(&m_opcodes),
       m_inError(false), m_lineNo(original.lineNumber()), m_extender(0)
 {
     extender(original.extender());
     for (ObjList* l = original.m_opcodes.skipNull(); l; l = l->skipNext()) {
 	const ExpOperation* o = static_cast<const ExpOperation*>(l->get());
-	m_opcodes.append(o->clone());
+	m_lastOpcode = m_lastOpcode->append(o->clone());
     }
 }
 
@@ -182,6 +190,11 @@ ExpEvaluator::~ExpEvaluator()
     extender(0);
 }
 
+bool ExpEvaluator::null() const
+{
+    return !m_opcodes.skipNull();
+}
+
 void ExpEvaluator::extender(ExpExtender* ext)
 {
     if (ext == m_extender)
@@ -194,9 +207,9 @@ void ExpEvaluator::extender(ExpExtender* ext)
 	TelEngine::destruct(tmp->refObj());
 }
 
-char ExpEvaluator::skipWhites(const char*& expr)
+char ExpEvaluator::skipWhites(ParsePoint& expr)
 {
-    if (!expr)
+    if (!expr.m_expr)
 	return 0;
     for (; ; expr++) {
 	char c = *expr;
@@ -205,12 +218,12 @@ char ExpEvaluator::skipWhites(const char*& expr)
 	    case '\t':
 		continue;
 	    case '\r':
-		m_lineNo++;
+		expr.m_lineNo = ++m_lineNo;
 		if (expr[1] == '\n')
 		    expr++;
 		continue;
 	    case '\n':
-		m_lineNo++;
+		expr.m_lineNo = ++m_lineNo;
 		if (expr[1] == '\r')
 		    expr++;
 		continue;
@@ -220,18 +233,27 @@ char ExpEvaluator::skipWhites(const char*& expr)
     }
 }
 
-bool ExpEvaluator::keywordChar(char c) const
+bool ExpEvaluator::keywordLetter(char c) const
 {
-    return ('a' <= c && c <= 'z') || ('A' <= c && c <= 'Z') ||
-	('0' <= c && c <= '9') || (c == '_');
+    return ('a' <= c && c <= 'z') || ('A' <= c && c <= 'Z') || (c == '_');
 }
 
-char ExpEvaluator::skipComments(const char*& expr, GenObject* context)
+bool ExpEvaluator::keywordDigit(char c) const
+{
+    return ('0' <= c && c <= '9');
+}
+
+bool ExpEvaluator::keywordChar(char c) const
+{
+    return keywordLetter(c) || keywordDigit(c);
+}
+
+char ExpEvaluator::skipComments(ParsePoint& expr, GenObject* context)
 {
     return skipWhites(expr);
 }
 
-int ExpEvaluator::preProcess(const char*& expr, GenObject* context)
+int ExpEvaluator::preProcess(ParsePoint& expr, GenObject* context)
 {
     return -1;
 }
@@ -285,16 +307,26 @@ void ExpEvaluator::formatLineNo(String& buf, unsigned int line) const
     buf << "line " << line;
 }
 
-bool ExpEvaluator::getInstruction(const char*& expr, char stop, GenObject* nested)
+bool ExpEvaluator::getInstruction(ParsePoint& expr, char stop, GenObject* nested)
 {
     return false;
 }
 
-bool ExpEvaluator::getOperand(const char*& expr, bool endOk)
+bool ExpEvaluator::getOperand(ParsePoint& expr, bool endOk, int precedence)
 {
     if (inError())
 	return false;
-    XDebug(this,DebugAll,"getOperand '%.30s'",expr);
+    XDebug(this,DebugAll,"getOperand '%.30s'",(const char*)expr);
+    if (!getOperandInternal(expr, endOk, precedence))
+	return false;
+    Opcode oper;
+    while ((oper = getPostfixOperator(expr,precedence)) != OpcNone)
+	addOpcode(oper);
+    return true;
+}
+
+bool ExpEvaluator::getOperandInternal(ParsePoint& expr, bool endOk, int precedence)
+{
     char c = skipComments(expr);
     if (!c)
 	// end of string
@@ -310,7 +342,7 @@ bool ExpEvaluator::getOperand(const char*& expr, bool endOk)
     }
     Opcode op = getUnaryOperator(expr);
     if (op != OpcNone) {
-	if (!getOperand(expr,false))
+	if (!getOperand(expr,false,getPrecedence(op)))
 	    return false;
 	addOpcode(op);
 	return true;
@@ -320,31 +352,31 @@ bool ExpEvaluator::getOperand(const char*& expr, bool endOk)
     return gotError("Expecting operand",expr);
 }
 
-bool ExpEvaluator::getSimple(const char*& expr, bool constOnly)
+bool ExpEvaluator::getSimple(ParsePoint& expr, bool constOnly)
 {
     return getString(expr) || getNumber(expr);
 }
 
-bool ExpEvaluator::getNumber(const char*& expr)
+bool ExpEvaluator::getNumber(ParsePoint& expr)
 {
     if (inError())
 	return false;
-    XDebug(this,DebugAll,"getNumber '%.30s'",expr);
+    XDebug(this,DebugAll,"getNumber '%.30s'",(const char*)expr);
     char* endp = 0;
-    long int val = ::strtol(expr,&endp,0);
+    int64_t val = ::strtoll(expr,&endp,0);
     if (!endp || (endp == expr))
 	return false;
     expr = endp;
-    DDebug(this,DebugAll,"Found %ld",val);
+    DDebug(this,DebugAll,"Found " FMT64,val);
     addOpcode(val);
     return true;
 }
 
-bool ExpEvaluator::getString(const char*& expr)
+bool ExpEvaluator::getString(ParsePoint& expr)
 {
     if (inError())
 	return false;
-    XDebug(this,DebugAll,"getString '%.30s'",expr);
+    XDebug(this,DebugAll,"getString '%.30s'",(const char*)expr);
     char c = skipComments(expr);
     if (c == '"' || c == '\'') {
 	String str;
@@ -419,17 +451,20 @@ int ExpEvaluator::getKeyword(const char* str) const
     return len;
 }
 
-bool ExpEvaluator::getFunction(const char*& expr)
+bool ExpEvaluator::getFunction(ParsePoint& expr)
 {
     if (inError())
 	return false;
-    XDebug(this,DebugAll,"getFunction '%.30s'",expr);
+    XDebug(this,DebugAll,"getFunction '%.30s'",(const char*)expr);
     skipComments(expr);
     int len = getKeyword(expr);
-    const char* s = expr+len;
+    ParsePoint s = expr;
+    s.m_expr = s.m_expr+len;
     skipComments(expr);
-    if ((len <= 0) || (skipComments(s) != '('))
+    if ((len <= 0) || (skipComments(s) != '(')) {
+	expr.m_lineNo = s.m_lineNo;
 	return false;
+    }
     s++;
     int argc = 0;
     // parameter list
@@ -437,6 +472,7 @@ bool ExpEvaluator::getFunction(const char*& expr)
 	if (!runCompile(s,')')) {
 	    if (!argc && (skipComments(s) == ')'))
 		break;
+	    expr.m_lineNo = s.m_lineNo;
 	    return false;
 	}
 	argc++;
@@ -444,17 +480,17 @@ bool ExpEvaluator::getFunction(const char*& expr)
     if (skipComments(s) != ')')
 	return gotError("Expecting ')' after function",s);
     String str(expr,len);
-    expr = s+1;
+    expr.m_expr = s.m_expr+1;
     DDebug(this,DebugAll,"Found %s()",str.safe());
     addOpcode(OpcFunc,str,argc);
     return true;
 }
 
-bool ExpEvaluator::getField(const char*& expr)
+bool ExpEvaluator::getField(ParsePoint& expr)
 {
     if (inError())
 	return false;
-    XDebug(this,DebugAll,"getField '%.30s'",expr);
+    XDebug(this,DebugAll,"getField '%.30s'",(const char*)expr);
     skipComments(expr);
     int len = getKeyword(expr);
     if (len <= 0)
@@ -468,19 +504,19 @@ bool ExpEvaluator::getField(const char*& expr)
     return true;
 }
 
-ExpEvaluator::Opcode ExpEvaluator::getOperator(const char*& expr)
+ExpEvaluator::Opcode ExpEvaluator::getOperator(ParsePoint& expr)
 {
     skipComments(expr);
     return getOperator(expr,m_operators);
 }
 
-ExpEvaluator::Opcode ExpEvaluator::getUnaryOperator(const char*& expr)
+ExpEvaluator::Opcode ExpEvaluator::getUnaryOperator(ParsePoint& expr)
 {
     skipComments(expr);
     return getOperator(expr,m_unaryOps);
 }
 
-ExpEvaluator::Opcode ExpEvaluator::getPostfixOperator(const char*& expr)
+ExpEvaluator::Opcode ExpEvaluator::getPostfixOperator(ParsePoint& expr, int priority)
 {
     return OpcNone;
 }
@@ -498,40 +534,41 @@ int ExpEvaluator::getPrecedence(ExpEvaluator::Opcode oper) const
 	case OpcDecPre:
 	case OpcIncPost:
 	case OpcDecPost:
+	    return 120;
 	case OpcNeg:
 	case OpcNot:
-	    return 11;
+	case OpcLNot:
+	    return 110;
 	case OpcMul:
 	case OpcDiv:
 	case OpcMod:
 	case OpcAnd:
-	    return 10;
+	    return 100;
 	case OpcAdd:
 	case OpcSub:
 	case OpcOr:
 	case OpcXor:
-	    return 9;
+	    return 90;
 	case OpcShl:
 	case OpcShr:
-	    return 8;
+	    return 80;
 	case OpcCat:
-	    return 7;
-	// ANY, ALL, SOME = 6
-	case OpcLNot:
-	    return 5;
+	    return 70;
+	// ANY, ALL, SOME = 60
 	case OpcLt:
 	case OpcGt:
 	case OpcLe:
 	case OpcGe:
+	    return 50;
 	case OpcEq:
 	case OpcNe:
-	    return 4;
-	// IN, BETWEEN, LIKE, MATCHES = 3
+	    return 40;
+	// IN, BETWEEN, LIKE, MATCHES = 30
 	case OpcLAnd:
-	    return 2;
+	    return 20;
 	case OpcLOr:
 	case OpcLXor:
-	    return 1;
+	    return 10;
 	default:
 	    return 0;
     }
@@ -553,7 +590,7 @@ bool ExpEvaluator::getRightAssoc(ExpEvaluator::Opcode oper) const
     }
 }
 
-bool ExpEvaluator::getSeparator(const char*& expr, bool remove)
+bool ExpEvaluator::getSeparator(ParsePoint& expr, bool remove)
 {
     if (skipComments(expr) != ',')
 	return false;
@@ -562,7 +599,7 @@ bool ExpEvaluator::getSeparator(const char*& expr, bool remove)
     return true;
 }
 
-bool ExpEvaluator::runCompile(const char*& expr, char stop, GenObject* nested)
+bool ExpEvaluator::runCompile(ParsePoint& expr, char stop, GenObject* nested)
 {
     char buf[2];
     const char* stopStr = 0;
@@ -574,7 +611,7 @@ bool ExpEvaluator::runCompile(const char*& expr, char stop, GenObject* nested)
     return runCompile(expr,stopStr,nested);
 }
 
-bool ExpEvaluator::runCompile(const char*& expr, const char* stop, GenObject* nested)
+bool ExpEvaluator::runCompile(ParsePoint& expr, const char* stop, GenObject* nested)
 {
     typedef struct {
 	Opcode code;
@@ -583,7 +620,7 @@ bool ExpEvaluator::runCompile(const char*& expr, const char* stop, GenObject* ne
     StackedOpcode stack[10];
     unsigned int stackPos = 0;
 #ifdef DEBUG
-    Debugger debug(DebugInfo,"runCompile()"," '%s' %p '%.30s'",TelEngine::c_safe(stop),nested,expr);
+    Debugger debug(DebugInfo,"runCompile()"," '%s' %p '%.30s'",TelEngine::c_safe(stop),nested,(const char*)expr);
 #endif
     if (skipComments(expr) == ')')
 	return false;
@@ -596,12 +633,15 @@ bool ExpEvaluator::runCompile(const char*& expr, const char* stop, GenObject* ne
     char stopChar = stop ? stop[0] : '\0';
     for (;;) {
 	while (!stackPos && skipComments(expr) && (!stop || !::strchr(stop,*expr)) && getInstruction(expr,stopChar,nested))
-	    ;
+	   if (!expr.m_count && expr.m_searchedSeps && expr.m_foundSep && ::strchr(expr.m_searchedSeps,expr.m_foundSep))
+		return true;
 	if (inError())
 	    return false;
 	char c = skipComments(expr);
-	if (c && stop && ::strchr(stop,c))
+	if (c && stop && ::strchr(stop,c)) {
+	    expr.m_foundSep = c;
 	    return true;
+	}
 	if (!getOperand(expr))
 	    return false;
 	Opcode oper;
@@ -679,7 +719,7 @@ bool ExpEvaluator::trySimplify()
 		    if (o->opcode() == OpcLAnd || o->opcode() == OpcAnd || o->opcode() == OpcMul) {
 			if ((op1->opcode() == OpcPush && !op1->number() && op2->opcode() == OpcField) ||
 			    (op2->opcode() == OpcPush && !op2->number() && op1->opcode() == OpcField)) {
-			    ExpOperation* newOp = (o->opcode() == OpcLAnd) ? new ExpOperation(false) : new ExpOperation((long int)0);
+			    ExpOperation* newOp = (o->opcode() == OpcLAnd) ? new ExpOperation(false) : new ExpOperation((int64_t)0);
 			    newOp->lineNumber(o->lineNumber());
 			    (m_opcodes+i)->set(newOp);
 			    m_opcodes.remove(op1);
@@ -752,6 +792,7 @@ bool ExpEvaluator::trySimplify()
 		break;
 	}
     }
+    m_lastOpcode = m_opcodes.last();
     return done;
 }
 
@@ -763,7 +804,7 @@ void ExpEvaluator::addOpcode(ExpOperation* oper, unsigned int line)
     if (!line)
 	line = lineNumber();
     oper->lineNumber(line);
-    m_opcodes.append(oper);
+    m_lastOpcode = m_lastOpcode->append(oper);
 }
 
 ExpOperation* ExpEvaluator::addOpcode(ExpEvaluator::Opcode oper, bool barrier)
@@ -781,25 +822,25 @@ ExpOperation* ExpEvaluator::addOpcode(ExpEvaluator::Opcode oper, bool barrier)
     }
     ExpOperation* op = new ExpOperation(oper,0,ExpOperation::nonInteger(),barrier);
     op->lineNumber(lineNumber());
-    m_opcodes.append(op);
+    m_lastOpcode = m_lastOpcode->append(op);
     return op;
 }
 
-ExpOperation* ExpEvaluator::addOpcode(ExpEvaluator::Opcode oper, long int value, bool barrier)
+ExpOperation* ExpEvaluator::addOpcode(ExpEvaluator::Opcode oper, int64_t value, bool barrier)
 {
-    DDebug(this,DebugAll,"addOpcode %u (%s) %lu",oper,getOperator(oper),value);
+    DDebug(this,DebugAll,"addOpcode %u (%s) " FMT64,oper,getOperator(oper),value);
     ExpOperation* op = new ExpOperation(oper,0,value,barrier);
     op->lineNumber(lineNumber());
-    m_opcodes.append(op);
+    m_lastOpcode = m_lastOpcode->append(op);
     return op;
 }
 
-ExpOperation* ExpEvaluator::addOpcode(ExpEvaluator::Opcode oper, const String& name, long int value, bool barrier)
+ExpOperation* ExpEvaluator::addOpcode(ExpEvaluator::Opcode oper, const String& name, int64_t value, bool barrier)
 {
-    DDebug(this,DebugAll,"addOpcode %u (%s) '%s' %ld",oper,getOperator(oper),name.c_str(),value);
+    DDebug(this,DebugAll,"addOpcode %u (%s) '%s' " FMT64,oper,getOperator(oper),name.c_str(),value);
     ExpOperation* op = new ExpOperation(oper,name,value,barrier);
     op->lineNumber(lineNumber());
-    m_opcodes.append(op);
+    m_lastOpcode = m_lastOpcode->append(op);
     return op;
 }
 
@@ -808,16 +849,16 @@ ExpOperation* ExpEvaluator::addOpcode(const String& value)
     DDebug(this,DebugAll,"addOpcode ='%s'",value.c_str());
     ExpOperation* op = new ExpOperation(value);
     op->lineNumber(lineNumber());
-    m_opcodes.append(op);
+    m_lastOpcode = m_lastOpcode->append(op);
     return op;
 }
 
-ExpOperation* ExpEvaluator::addOpcode(long int value)
+ExpOperation* ExpEvaluator::addOpcode(int64_t value)
 {
-    DDebug(this,DebugAll,"addOpcode =%ld",value);
+    DDebug(this,DebugAll,"addOpcode =" FMT64,value);
     ExpOperation* op = new ExpOperation(value);
     op->lineNumber(lineNumber());
-    m_opcodes.append(op);
+    m_lastOpcode = m_lastOpcode->append(op);
     return op;
 }
 
@@ -826,7 +867,7 @@ ExpOperation* ExpEvaluator::addOpcode(bool value)
     DDebug(this,DebugAll,"addOpcode =%s",String::boolText(value));
     ExpOperation* op = new ExpOperation(value);
     op->lineNumber(lineNumber());
-    m_opcodes.append(op);
+    m_lastOpcode = m_lastOpcode->append(op);
     return op;
 }
 
@@ -873,13 +914,7 @@ ExpOperation* ExpEvaluator::popOne(ObjList& stack)
     }
     stack.remove(o,false);
 #ifdef DEBUG
-#ifdef XDEBUG
-    Debug(DebugAll,"popOne: %p%s%s",o,
-	(YOBJECT(ExpFunction,o) ? " function" : ""),
-	(YOBJECT(ExpWrapper,o) ? " wrapper" : ""));
-#else
-    Debug(DebugAll,"popOne: %p",o);
-#endif
+    Debug(DebugAll,"popOne: %p%s%s",o,(o ? " " : ""),(o ? o->typeOf() : ""));
 #endif
     return o;
 }
@@ -896,14 +931,8 @@ ExpOperation* ExpEvaluator::popAny(ObjList& stack)
     }
     stack.remove(o,false);
 #ifdef DEBUG
-#ifdef XDEBUG
-    Debug(DebugAll,"popAny: %p%s%s '%s'",o,
-	(YOBJECT(ExpFunction,o) ? " function" : ""),
-	(YOBJECT(ExpWrapper,o) ? " wrapper" : ""),
-	(o ? o->name().safe() : (const char*)0));
-#else
-    Debug(DebugAll,"popAny: %p '%s'",o,(o ? o->name().safe() : (const char*)0));
-#endif
+    Debug(DebugAll,"popAny: %p%s%s '%s'",o,(o ? " " : ""),
+	(o ? o->typeOf() : ""),(o ? o->name().safe() : (const char*)0));
 #endif
     return o;
 }
@@ -1004,7 +1033,7 @@ bool ExpEvaluator::runOperation(ObjList& stack, const ExpOperation& oper, GenObj
 		    default:
 			break;
 		}
-		long int val = 0;
+		int64_t val = 0;
 		switch (oper.opcode()) {
 		    case OpcAnd:
 			val = op1->valInteger() & op2->valInteger();
@@ -1064,7 +1093,7 @@ bool ExpEvaluator::runOperation(ObjList& stack, const ExpOperation& oper, GenObj
 		    pushOne(stack,new ExpOperation(val != 0));
 		}
 		else {
-		    DDebug(this,DebugAll,"Numeric result: %lu",val);
+		    DDebug(this,DebugAll,"Numeric result: " FMT64,val);
 		    pushOne(stack,new ExpOperation(val));
 		}
 	    }
@@ -1170,7 +1199,7 @@ bool ExpEvaluator::runOperation(ObjList& stack, const ExpOperation& oper, GenObj
 		    TelEngine::destruct(fld);
 		    return false;
 		}
-		long int num = val->valInteger();
+		int64_t num = val->valInteger();
 		switch (oper.opcode()) {
 		    case OpcIncPre:
 			num++;
@@ -1246,9 +1275,11 @@ bool ExpEvaluator::runOperation(ObjList& stack, const ExpOperation& oper, GenObj
 		pushOne(stack,val);
 		ExpOperation op((Opcode)(oper.opcode() & ~OpcAssign),
 		    oper.name(),oper.number(),oper.barrier());
+		op.lineNumber(oper.lineNumber());
 		if (!runOperation(stack,op,context))
 		    return false;
-		static const ExpOperation assign(OpcAssign);
+		ExpOperation assign(OpcAssign);
+		assign.lineNumber(oper.lineNumber());
 		return runOperation(stack,assign,context);
 	    }
 	    Debug(this,DebugStub,"Please implement operation %u '%s'",
@@ -1260,11 +1291,11 @@ bool ExpEvaluator::runOperation(ObjList& stack, const ExpOperation& oper, GenObj
 
 bool ExpEvaluator::runFunction(ObjList& stack, const ExpOperation& oper, GenObject* context) const
 {
-    DDebug(this,DebugAll,"runFunction(%p,'%s' %ld, %p) ext=%p",
+    DDebug(this,DebugAll,"runFunction(%p,'%s' "FMT64", %p) ext=%p",
 	&stack,oper.name().c_str(),oper.number(),context,(void*)m_extender);
     if (oper.name() == YSTRING("chr")) {
 	String res;
-	for (long int i = oper.number(); i; i--) {
+	for (int i = (int)oper.number(); i; i--) {
 	    ExpOperation* o = popValue(stack,context);
 	    if (!o)
 		return gotError("ExpEvaluator stack underflow",oper.lineNumber());
@@ -1277,7 +1308,7 @@ bool ExpEvaluator::runFunction(ObjList& stack, const ExpOperation& oper, GenObje
     if (oper.name() == YSTRING("now")) {
 	if (oper.number())
 	    return gotError("Function expects no arguments",oper.lineNumber());
-	pushOne(stack,new ExpOperation((long int)Time::secNow()));
+	pushOne(stack,new ExpOperation((int64_t)Time::secNow()));
 	return true;
     }
     return m_extender && m_extender->runFunction(stack,oper,context);
@@ -1348,8 +1379,10 @@ bool ExpEvaluator::runAllFields(ObjList& stack, GenObject* context) const
     return ok;
 }
 
-int ExpEvaluator::compile(const char* expr, GenObject* context)
+int ExpEvaluator::compile(ParsePoint& expr, GenObject* context)
 {
+    if (!expr.m_eval)
+	expr.m_eval = this;
     if (!skipComments(expr,context))
 	return 0;
     int res = 0;
@@ -1411,16 +1444,11 @@ int ExpEvaluator::evaluate(Array& results, unsigned int index, GenObject* contex
 
 void ExpEvaluator::dump(const ExpOperation& oper, String& res) const
 {
-    const char* name = getOperator(oper.opcode());
-    if (name) {
-	res << name;
-	return;
-    }
     switch (oper.opcode()) {
 	case OpcPush:
 	case OpcCopy:
 	    if (oper.isInteger())
-		res << (int)oper.number();
+		res << oper.number();
 	    else
 		res << "'" << oper << "'";
 	    break;
@@ -1428,12 +1456,18 @@ void ExpEvaluator::dump(const ExpOperation& oper, String& res) const
 	    res << oper.name();
 	    break;
 	case OpcFunc:
-	    res << oper.name() << "(" << (int)oper.number() << ")";
+	    res << oper.name() << "(" << oper.number() << ")";
 	    break;
 	default:
-	    res << "[" << oper.opcode() << "]";
+	    {
+	        const char* name = getOperator(oper.opcode());
+		if (name)
+		    res << name;
+		else 
+		    res << "[" << oper.opcode() << "]";
+	    }
 	    if (oper.number() && oper.isInteger())
-		res << "(" << (int)oper.number() << ")";
+		res << "(" << oper.number() << ")";
     }
 }
 
@@ -1447,7 +1481,12 @@ void ExpEvaluator::dump(const ObjList& codes, String& res) const
     }
 }
 
-long int ExpOperation::valInteger() const
+void ExpEvaluator::dump(String& res) const
+{
+    return dump(m_opcodes,res);
+}
+
+int64_t ExpOperation::valInteger() const
 {
     return isInteger() ? number() : 0;
 }
@@ -1457,6 +1496,19 @@ bool ExpOperation::valBoolean() const
     return isInteger() ? (number() != 0) : !null();
 }
 
+const char* ExpOperation::typeOf() const
+{
+    switch (opcode()) {
+	case ExpEvaluator::OpcPush:
+	case ExpEvaluator::OpcCopy:
+	    return isInteger() ? "number" : "string";
+	case ExpEvaluator::OpcFunc:
+	    return "function";
+	default:
+	    return "internal";
+    }
+}
+
 ExpOperation* ExpOperation::clone(const char* name) const
 {
     ExpOperation* op = new ExpOperation(*this,name);
@@ -1468,7 +1520,7 @@ ExpOperation* ExpOperation::clone(const char* name) const
 ExpOperation* ExpFunction::clone(const char* name) const
 {
     XDebug(DebugInfo,"ExpFunction::clone('%s') [%p]",name,this);
-    ExpFunction* op = new ExpFunction(name,number());
+    ExpFunction* op = new ExpFunction(name,(long int)number());
     op->lineNumber(lineNumber());
     return op;
 }
@@ -1499,6 +1551,17 @@ ExpOperation* ExpWrapper::copy(Mutex* mtx) const
     return op;
 }
 
+const char* ExpWrapper::typeOf() const
+{
+    switch (opcode()) {
+	case ExpEvaluator::OpcPush:
+	case ExpEvaluator::OpcCopy:
+	    return object() ? "object" : "undefined";
+	default:
+	    return ExpOperation::typeOf();
+    }
+}
+
 bool ExpWrapper::valBoolean() const
 {
     if (!m_object)
@@ -1508,7 +1571,7 @@ bool ExpWrapper::valBoolean() const
 
 void* ExpWrapper::getObject(const String& name) const
 {
-    if (name == YSTRING("ExpWrapper"))
+    if (name == YATOM("ExpWrapper"))
 	return const_cast<ExpWrapper*>(this);
     void* obj = ExpOperation::getObject(name);
     if (obj)
@@ -1583,7 +1646,7 @@ unsigned int TableEvaluator::evalLimit(GenObject* context)
 	    const ExpOperation* o = static_cast<const ExpOperation*>(first->get());
 	    if (o->opcode() != ExpEvaluator::OpcPush)
 		break;
-	    int lim = o->number();
+	    int lim = (int)o->number();
 	    if (lim < 0)
 		lim = 0;
 	    m_limitVal = lim;
