@@ -4,24 +4,22 @@
  * This file is part of the YATE Project http://YATE.null.ro
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2011 Null Team
+ * Copyright (C) 2011-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include "yatescript.h"
+#include <string.h>
 
 using namespace TelEngine;
 
@@ -46,7 +44,8 @@ class JsDate : public JsObject
     YCLASS(JsDate,JsObject)
 public:
     inline JsDate(Mutex* mtx)
-	: JsObject("Date",mtx,true)
+	: JsObject("Date",mtx,true),
+	  m_time(0), m_msec(0), m_offs(0)
 	{
 	    params().addParam(new ExpFunction("getDate"));
 	    params().addParam(new ExpFunction("getDay"));
@@ -57,6 +56,7 @@ public:
 	    params().addParam(new ExpFunction("getMonth"));
 	    params().addParam(new ExpFunction("getSeconds"));
 	    params().addParam(new ExpFunction("getTime"));
+	    params().addParam(new ExpFunction("getTimezoneOffset"));
 
 	    params().addParam(new ExpFunction("getUTCDate"));
 	    params().addParam(new ExpFunction("getUTCDay"));
@@ -71,13 +71,23 @@ public:
 	{
 	    construct->params().addParam(new ExpFunction("now"));
 	}
+    virtual JsObject* runConstructor(ObjList& stack, const ExpOperation& oper, GenObject* context);
 protected:
-    inline JsDate(Mutex* mtx, const char* name)
-	: JsObject(mtx,name)
+    inline JsDate(Mutex* mtx, u_int64_t msecs, bool local = false)
+	: JsObject("Date",mtx),
+	  m_time((unsigned int)(msecs / 1000)), m_msec((unsigned int)(msecs % 1000)), m_offs(Time::timeZone())
+	{ if (local) m_time -= m_offs; }
+    inline JsDate(Mutex* mtx, const char* name, unsigned int time, unsigned int msec, unsigned int offs)
+	: JsObject(mtx,name),
+	  m_time(time), m_msec(msec), m_offs(offs)
 	{ }
     virtual JsObject* clone(const char* name) const
-	{ return new JsDate(mutex(),name); }
+	{ return new JsDate(mutex(),name,m_time,m_msec,m_offs); }
     bool runNative(ObjList& stack, const ExpOperation& oper, GenObject* context);
+private:
+    unsigned int m_time;
+    unsigned int m_msec;
+    int m_offs;
 };
 
 // Math class - not really an object, all methods are static
@@ -91,6 +101,7 @@ public:
 	    params().addParam(new ExpFunction("abs"));
 	    params().addParam(new ExpFunction("max"));
 	    params().addParam(new ExpFunction("min"));
+	    params().addParam(new ExpFunction("random"));
 	}
 protected:
     bool runNative(ObjList& stack, const ExpOperation& oper, GenObject* context);
@@ -310,8 +321,13 @@ bool JsObject::runField(ObjList& stack, const ExpOperation& oper, GenObject* con
 	    ExpWrapper* w = YOBJECT(ExpWrapper,param);
 	    if (w)
 		ExpEvaluator::pushOne(stack,w->clone(oper.name()));
-	    else
-		ExpEvaluator::pushOne(stack,new ExpOperation(*param,oper.name(),true));
+	    else {
+		bool num = true;
+		ExpOperation* o = YOBJECT(ExpOperation,param);
+		if (o && !o->isInteger())
+		    num = false;
+		ExpEvaluator::pushOne(stack,new ExpOperation(*param,oper.name(),num));
+	    }
 	}
     }
     else
@@ -321,8 +337,8 @@ bool JsObject::runField(ObjList& stack, const ExpOperation& oper, GenObject* con
 
 bool JsObject::runAssign(ObjList& stack, const ExpOperation& oper, GenObject* context)
 {
-    XDebug(DebugAll,"JsObject::runAssign() '%s'='%s' in '%s' [%p]",
-	oper.name().c_str(),oper.c_str(),toString().c_str(),this);
+    XDebug(DebugAll,"JsObject::runAssign() '%s'='%s' (%s) in '%s' [%p]",
+	oper.name().c_str(),oper.c_str(),oper.typeOf(),toString().c_str(),this);
     if (frozen()) {
 	Debug(DebugWarn,"Object '%s' is frozen",toString().c_str());
 	return false;
@@ -333,14 +349,13 @@ bool JsObject::runAssign(ObjList& stack, const ExpOperation& oper, GenObject* co
     else {
 	ExpWrapper* w = YOBJECT(ExpWrapper,&oper);
 	if (w) {
-	    GenObject* o = w->object();
-	    if (o)
-		params().setParam(w->clone(oper.name()));
-	    else
-		params().clearParam(oper.name());
+	    JsFunction* jsf = YOBJECT(JsFunction,w->object());
+	    if (jsf)
+		jsf->firstName(oper.name());
+	    params().setParam(w->clone(oper.name()));
 	}
 	else
-	    params().setParam(new NamedString(oper.name(),oper));
+	    params().setParam(oper.clone());
     }
     return true;
 }
@@ -357,7 +372,7 @@ bool JsObject::runNative(ObjList& stack, const ExpOperation& oper, GenObject* co
 	ExpEvaluator::pushOne(stack,new ExpOperation(params()));
     else if (oper.name() == YSTRING("hasOwnProperty")) {
 	bool ok = true;
-	for (long int i = oper.number(); i; i--) {
+	for (int i = (int)oper.number(); i; i--) {
 	    ExpOperation* op = popValue(stack,context);
 	    if (!op)
 		continue;
@@ -404,11 +419,11 @@ int JsObject::extractArgs(JsObject* obj, ObjList& stack, const ExpOperation& ope
 {
     if (!obj || !oper.number())
 	return 0;
-    for (long int i = oper.number(); i;  i--) {
+    for (int i = (int)oper.number(); i;  i--) {
 	ExpOperation* op = obj->popValue(stack,context);
 	arguments.insert(op);
     }
-    return oper.number();
+    return (int)oper.number();
 }
 
 // Static helper method that deep copies all parameters
@@ -471,6 +486,7 @@ JsArray::JsArray(Mutex* mtx)
     params().addParam(new ExpFunction("slice"));
     params().addParam(new ExpFunction("splice"));
     params().addParam(new ExpFunction("sort"));
+    params().addParam(new ExpFunction("indexOf"));
     params().addParam("length","0");
 }
 
@@ -494,6 +510,39 @@ void JsArray::push(ExpOperation* item)
     setLength(pos + 1);
 }
 
+bool JsArray::runAssign(ObjList& stack, const ExpOperation& oper, GenObject* context)
+{
+    XDebug(DebugAll,"JsArray::runAssign() '%s'='%s' (%s) in '%s' [%p]",
+	oper.name().c_str(),oper.c_str(),oper.typeOf(),toString().c_str(),this);
+    if (oper.name() == YSTRING("length")) {
+	int newLen = oper.toInteger(-1);
+	if (newLen < 0)
+	    return false;
+	for (int i = newLen; i < length(); i++)
+	    params().clearParam(String(i));
+	setLength(newLen);
+	return true;
+    }
+    else if (!JsObject::runAssign(stack,oper,context))
+	return false;
+    int idx = oper.toString().toInteger(-1) + 1;
+    if (idx && idx > m_length)
+	setLength(idx);
+    return true;
+}
+
+bool JsArray::runField(ObjList& stack, const ExpOperation& oper, GenObject* context)
+{
+    XDebug(DebugAll,"JsArray::runField() '%s' in '%s' [%p]",
+	oper.name().c_str(),toString().c_str(),this);
+    if (oper.name() == YSTRING("length")) {
+	// Reflects the number of elements in an array.
+	ExpEvaluator::pushOne(stack,new ExpOperation((int64_t)length()));
+	return true;
+    }
+    return JsObject::runField(stack,oper,context);
+}
+
 bool JsArray::runNative(ObjList& stack, const ExpOperation& oper, GenObject* context)
 {
     XDebug(DebugAll,"JsArray::runNative() '%s' in '%s' [%p]",
@@ -507,8 +556,7 @@ bool JsArray::runNative(ObjList& stack, const ExpOperation& oper, GenObject* con
 	    const_cast<String&>(op->name()) = (unsigned int)m_length++;
 	    params().addParam(op);
 	}
-	setLength();
-	ExpEvaluator::pushOne(stack,new ExpOperation(length()));
+	ExpEvaluator::pushOne(stack,new ExpOperation((int64_t)length()));
     }
     else if (oper.name() == YSTRING("pop")) {
 	// Removes the last element from an array and returns that element
@@ -517,26 +565,21 @@ bool JsArray::runNative(ObjList& stack, const ExpOperation& oper, GenObject* con
 
 	NamedString* last = 0;
 	while (!last) {
-	    last = params().getParam(String((int)--m_length));
+	    last = params().getParam(String(--m_length));
 	    if (m_length == 0)
 		break;
 	}
 	if (!last)
 	    ExpEvaluator::pushOne(stack,new ExpWrapper(0,0));
 	else {
-	    NamedPointer* np = (NamedPointer*)last->getObject(YSTRING("NamedPointer"));
-	    if (!np)
-		ExpEvaluator::pushOne(stack,new ExpOperation(last->toString()));
+	    ExpWrapper* w = YOBJECT(ExpWrapper,last);
+	    if (!w)
+		ExpEvaluator::pushOne(stack,new ExpOperation(*last));
 	    else
-		ExpEvaluator::pushOne(stack,new ExpWrapper(np->userData(),0));
+		ExpEvaluator::pushOne(stack,w->clone(w->name()));
 	}
 	// clear last
 	params().clearParam(last);
-	setLength();
-    }
-    else if (oper.name() == YSTRING("length")) {
-	// Reflects the number of elements in an array.
-	ExpEvaluator::pushOne(stack,new ExpOperation(length()));
     }
     else if (oper.name() == YSTRING("concat")) {
 	// Returns a new array comprised of this array joined with other array(s) and/or value(s).
@@ -554,25 +597,25 @@ bool JsArray::runNative(ObjList& stack, const ExpOperation& oper, GenObject* con
 	    return false;
 	JsArray* array = new JsArray(mutex());
 	// copy this array
-	for (long int i = 0; i < m_length; i++)
-	    array->params().addParam(params().getParam(String((int)i)));
+	for (int i = 0; i < m_length; i++)
+	    array->params().addParam(params().getParam(String(i)));
 	array->setLength(length());
 	// add parameters (JsArray of JsObject)
-	for (long int i = oper.number(); i; i--) {
+	for (int i = (int)oper.number(); i; i--) {
 	    ExpOperation* op = popValue(stack,context);
 	    ExpWrapper* obj = YOBJECT(ExpWrapper,op);
 	    if (!obj)
 		continue;
-	    JsArray* ja = (JsArray*)obj->getObject(YSTRING("JsArray"));
+	    JsArray* ja = (JsArray*)obj->getObject(YATOM("JsArray"));
 	    if (ja) {
-		for (long int i = 0; i < ja->length(); i++)
-		    array->params().addParam(String((int)(i + array->length())),ja->params().getValue(String((int)i)));
+		for (int j = 0; j < ja->length(); j++)
+		    array->params().addParam(String(j + array->length()),ja->params().getValue(String(j)));
 		array->setLength(array->length() + ja->length());
 	    }
 	    else {
-		JsObject* jo = (JsObject*)obj->getObject(YSTRING("JsObject"));
+		JsObject* jo = (JsObject*)obj->getObject(YATOM("JsObject"));
 		if (jo) {
-		    array->params().addParam(new NamedPointer(String((unsigned int)array->length()),jo));
+		    array->params().addParam(new NamedPointer(String(array->length()),jo));
 		    array->setLength(array->length() + 1);
 		    jo->ref();
 		}
@@ -592,11 +635,12 @@ bool JsArray::runNative(ObjList& stack, const ExpOperation& oper, GenObject* con
 	String separator = ",";
 	if (oper.number()) {
 	    ExpOperation* op = popValue(stack,context);
-	    separator = op->toString();
+	    separator = *op;
+	    TelEngine::destruct(op);
 	}
 	String result;
-	for (long int i = 0; i < length(); i++)
-	    result.append(params()[String((int)i)],separator);
+	for (int32_t i = 0; i < length(); i++)
+	    result.append(params()[String(i)],separator);
 	ExpEvaluator::pushOne(stack,new ExpOperation(result));
     }
     else if (oper.name() == YSTRING("reverse")) {
@@ -606,11 +650,11 @@ bool JsArray::runNative(ObjList& stack, const ExpOperation& oper, GenObject* con
 	NamedList reversed("");
 	String separator = ",";
 	String toCopy;
-	for (long int i = 0; i < length(); i++)
-	    toCopy.append(params()[String((int)i)],separator);
+	for (int32_t i = 0; i < length(); i++)
+	    toCopy.append(params()[String(i)],separator);
 	reversed.copyParams(params(),toCopy);
-	for (long int i = length(); i; i--)
-	    params().setParam(String((int)(length() - i)),reversed.getValue(String((int)(i - 1))));
+	for (int32_t i = length(); i; i--)
+	    params().setParam(String(length() - i),reversed.getValue(String(i - 1)));
     }
     else if (oper.name() == YSTRING("shift")) {
 	// Removes the first element from an array and returns that element
@@ -629,9 +673,9 @@ bool JsArray::runNative(ObjList& stack, const ExpOperation& oper, GenObject* con
 	else {
 	    ExpEvaluator::pushOne(stack,new ExpOperation(params().getValue("0")));
 	    // shift : value n+1 becomes value n
-	    for (long int i = 0; i < length() - 1; i++)
-		params().setParam(String((int)i),params().getValue(String((int)i + 1)));
-	    params().clearParam(String((int)(length() - 1)));
+	    for (int32_t i = 0; i < length() - 1; i++)
+		params().setParam(String(i),params().getValue(String(i + 1)));
+	    params().clearParam(String(length() - 1));
 	    setLength(length() - 1);
 	}
     }
@@ -647,24 +691,24 @@ bool JsArray::runNative(ObjList& stack, const ExpOperation& oper, GenObject* con
 	// myFish after: ["drum", "lion", "angel", "clown"]
 	// New length: 4
 	// shift array
-	long shift = oper.number();
-	for (long int i = length(); i; i--)
-	    params().setParam(String((int)(i - 1 + shift)),params().getValue(String((int)(i - 1))));
-	for (long int i = shift; i; i--) {
+	int32_t shift = (int32_t)oper.number();
+	for (int32_t i = length(); i; i--)
+	    params().setParam(String(i - 1 + shift),params().getValue(String(i - 1)));
+	for (int32_t i = shift; i; i--) {
 	    ExpOperation* op = popValue(stack,context);
 	    ExpWrapper* obj = YOBJECT(ExpWrapper,op);
 	    if (!obj)
 		continue;
-	    JsObject* jo = (JsObject*)obj->getObject(YSTRING("JsObject"));
+	    JsObject* jo = (JsObject*)obj->getObject(YATOM("JsObject"));
 	    if (!jo)
 		continue;
 	    jo->ref();
-	    params().clearParam(String((int)(i - 1)));
-	    params().setParam(new NamedPointer(String((int)(i - 1)),jo));
+	    params().clearParam(String(i - 1));
+	    params().setParam(new NamedPointer(String(i - 1),jo));
 	    TelEngine::destruct(op);
 	}
 	setLength(length() + shift);
-	ExpEvaluator::pushOne(stack,new ExpOperation(length()));
+	ExpEvaluator::pushOne(stack,new ExpOperation((int64_t)length()));
     }
     else if (oper.name() == YSTRING("slice"))
 	return runNativeSlice(stack,oper,context);
@@ -679,9 +723,45 @@ bool JsArray::runNative(ObjList& stack, const ExpOperation& oper, GenObject* con
 	// var myVar = monthNames.toString(); // assigns "Jan,Feb,Mar,Apr" to myVar.
 	String separator = ",";
 	String result;
-	for (long int i = 0; i < length(); i++)
-	    result.append(params()[String((int)i)],separator);
+	for (int32_t i = 0; i < length(); i++)
+	    result.append(params()[String(i)],separator);
 	ExpEvaluator::pushOne(stack,new ExpOperation(result));
+    } else if (oper.name() == YSTRING("indexOf")) {
+	ObjList args;
+	if (!extractArgs(this,stack,oper,context,args)) {
+	    Debug(DebugWarn,"Failed to extract arguments!");
+	    return false;
+	}
+	ExpOperation* op1 = static_cast<ExpOperation*>(args.remove(false));
+	if (!op1)
+	    return false;
+	int pos = 0;
+	if (args.skipNull()) {
+	    String* spos = static_cast<String*>(args.remove(false));
+	    if (spos)
+		pos = spos->toInteger(0);
+	    TelEngine::destruct(spos);
+	}
+	int index = -1;
+	for (int i = pos;i < length();i++) {
+	    ExpOperation* op2 = static_cast<ExpOperation*>(params().getParam(String(i)));
+	    if (!op2 || op2->opcode() != op1->opcode())
+		continue;
+	    ExpWrapper* w1 = YOBJECT(ExpWrapper,op1);
+	    ExpWrapper* w2 = YOBJECT(ExpWrapper,op2);
+	    if (w1 || w2) {
+		if (w1 && w2 && w1->object() == w2->object()) {
+		    index = i;
+		    break;
+		}
+	    } else if ((op1->number() == op2->number()) && (*op1 == *op2)) {
+		index = i;
+		break;
+	    }
+	}
+	TelEngine::destruct(op1);
+	ExpEvaluator::pushOne(stack,new ExpOperation(String(index)));
+	return true;
     }
     else
 	return JsObject::runNative(stack,oper,context);
@@ -700,11 +780,11 @@ bool JsArray::runNativeSlice(ObjList& stack, const ExpOperation& oper, GenObject
     //	       < 0 offset from the end of the array
     // end missing -> go to end of array  
     int begin = length(), end = length();
-    for (long int i = oper.number(); i; i--) {
+    for (int i = (int)oper.number(); i; i--) {
 	ExpOperation* op = popValue(stack,context);
 	if (op->isInteger()) {
 	    end = begin;
-	    begin = op->number();
+	    begin = (int)op->number();
 	}
 	TelEngine::destruct(op);
     }
@@ -717,7 +797,7 @@ bool JsArray::runNativeSlice(ObjList& stack, const ExpOperation& oper, GenObject
     // TODO
     //JsArray* slice = new JsArray(mutex());
     for (long int i = begin; i < end; i++) {
-//	slice->params().addParam(new NamedString(String((int)(i - begin),
+//	slice->params().addParam(new NamedString(String(i - begin),
     }
     return true;
 }
@@ -734,7 +814,7 @@ bool JsArray::runNativeSplice(ObjList& stack, const ExpOperation& oper, GenObjec
 	return false;
     // get start index
     ExpOperation* op = static_cast<ExpOperation*>(arguments[0]);
-    int begin = op->number();
+    int begin = (int)op->number();
     if (begin < 0)
 	begin = length() + begin;
     // get count to delete
@@ -742,7 +822,7 @@ bool JsArray::runNativeSplice(ObjList& stack, const ExpOperation& oper, GenObjec
     if (arguments.count() > 1) {
 	// get count
 	op = static_cast<ExpOperation*>(arguments[1]);
-	count = op->number();
+	count = (int)op->number();
     }
 
     // remove elements
@@ -780,10 +860,67 @@ bool JsArray::runNativeSplice(ObjList& stack, const ExpOperation& oper, GenObjec
     return true;
 }
 
+class JsComparator
+{
+public:
+    JsComparator(const char* funcName, ScriptRun* runner)
+	: m_name(funcName), m_runner(runner)
+	{ }
+    const char* m_name;
+    ScriptRun* m_runner;
+};
+
+int compare(GenObject* op1, GenObject* op2, void* data)
+{
+    JsComparator* cmp = static_cast<JsComparator*>(data);
+    if (!(cmp && cmp->m_runner))
+	return ::strcmp(*(static_cast<String*>(op1)),*(static_cast<String*>(op2)));
+    ScriptRun* runner = cmp->m_runner->code()->createRunner(cmp->m_runner->context());
+    if (!runner)
+	return 0;
+    ObjList stack;
+    stack.append((static_cast<ExpOperation*>(op1))->clone());
+    stack.append((static_cast<ExpOperation*>(op2))->clone());
+    ScriptRun::Status rval = runner->call(cmp->m_name,stack);
+    int ret = 0;
+    if (ScriptRun::Succeeded == rval) {
+	String* sret = static_cast<String*>(ExpEvaluator::popOne(runner->stack()));
+	ret = sret->toInteger();
+	TelEngine::destruct(sret);
+    }
+    TelEngine::destruct(runner);
+    return ret;
+}
+
 bool JsArray::runNativeSort(ObjList& stack, const ExpOperation& oper, GenObject* context)
 {
-    // TODO
-    return false;
+    ObjList arguments;
+    ExpOperation* op = 0;
+    if (extractArgs(this,stack,oper,context,arguments))
+	op = static_cast<ExpOperation*>(arguments[0]);
+    ObjList sorted;
+    // Copy the arguments in a ObjList for sorting
+    for (int i = 0;i < length();i++) {
+	NamedString* param = params().getParam(String(i));
+	sorted.append(param);
+	params().clearParam(param,false);
+    }
+    ScriptRun* runner = YOBJECT(ScriptRun,context);
+    if (op && !runner)
+	return false;
+    JsComparator* comp = op ? new JsComparator(op->name() ,runner) : 0;
+    sorted.sort(&compare,comp);
+    delete comp;
+    int i = 0;
+    for (ObjList* o = sorted.skipNull();o;o = o->skipNext()) {
+	NamedString* slice = static_cast<NamedString*>(o->get());
+	String* name = (String*)(&slice->name());
+	*name = String(i++);
+	params().addParam(slice);
+	o->setDelete(false);
+    }
+    setLength(i);
+    return true;
 }
 
 
@@ -827,8 +964,8 @@ bool JsMath::runNative(ObjList& stack, const ExpOperation& oper, GenObject* cont
     if (oper.name() == YSTRING("abs")) {
 	if (!oper.number())
 	    return false;
-	long int n = 0;
-	for (long int i = oper.number(); i; i--) {
+	int64_t n = 0;
+	for (int i = (int)oper.number(); i; i--) {
 	    ExpOperation* op = popValue(stack,context);
 	    if (op->isInteger())
 		n = op->number();
@@ -841,8 +978,8 @@ bool JsMath::runNative(ObjList& stack, const ExpOperation& oper, GenObject* cont
     else if (oper.name() == YSTRING("max")) {
 	if (!oper.number())
 	    return false;
-	long int n = LONG_MIN;
-	for (long int i = oper.number(); i; i--) {
+	int64_t n = LLONG_MIN;
+	for (int i = (int)oper.number(); i; i--) {
 	    ExpOperation* op = popValue(stack,context);
 	    if (op->isInteger() && op->number() > n)
 		n = op->number();
@@ -853,8 +990,8 @@ bool JsMath::runNative(ObjList& stack, const ExpOperation& oper, GenObject* cont
     else if (oper.name() == YSTRING("min")) {
 	if (!oper.number())
 	    return false;
-	long int n = LONG_MAX;
-	for (long int i = oper.number(); i; i--) {
+	int64_t n = LLONG_MAX;
+	for (int i = (int)oper.number(); i; i--) {
 	    ExpOperation* op = popValue(stack,context);
 	    if (op->isInteger() && op->number() < n)
 		n = op->number();
@@ -862,120 +999,233 @@ bool JsMath::runNative(ObjList& stack, const ExpOperation& oper, GenObject* cont
 	}
 	ExpEvaluator::pushOne(stack,new ExpOperation(n));
     }
+    else if (oper.name() == YSTRING("random")) {
+	long min = 0;
+	long max = LONG_MAX;
+	ObjList args;
+	if (extractArgs(stack,oper,context,args)) {
+	    if (args.skipNull()) {
+		const String* mins = static_cast<String*>(args[0]);
+		if (mins)
+		    min = mins->toLong(0);
+	    }
+	    if (args.count() >= 2) {
+		const String* maxs = static_cast<String*>(args[1]);
+		if (maxs)
+		    max = maxs->toLong(max);
+	    }
+	}
+	if (min < 0 || max < 0 || min >= max)
+	    return false;
+	unsigned long interval = max;
+	if (min != 0)
+	    interval -= min + 1;
+	int64_t rand = (Random::random() % interval) + min;
+	ExpEvaluator::pushOne(stack,new ExpOperation(rand));
+    }
     else
 	return JsObject::runNative(stack,oper,context);
     return true;
 }
 
 
+JsObject* JsDate::runConstructor(ObjList& stack, const ExpOperation& oper, GenObject* context)
+{
+    XDebug(DebugAll,"JsDate::runConstructor '%s'("FMT64")",oper.name().c_str(),oper.number());
+    ObjList args;
+    JsObject* obj = 0;
+    switch (extractArgs(stack,oper,context,args)) {
+	case 0:
+	    obj = new JsDate(mutex(),Time::msecNow());
+	    break;
+	case 1:
+	    {
+		ExpOperation* val = static_cast<ExpOperation*>(args[0]);
+		if (val && val->isInteger())
+		    obj = new JsDate(mutex(),val->number());
+	    }
+	    break;
+	case 3:
+	case 6:
+	case 7:
+	    {
+		unsigned int parts[7];
+		for (int i = 0; i < 7; i++) {
+		    parts[i] = 0;
+		    ExpOperation* val = static_cast<ExpOperation*>(args[i]);
+		    if (val) {
+			if (val->isInteger())
+			    parts[i] = (int)val->number();
+			else
+			    return 0;
+		    }
+		}
+		// Date components use local time, month starts from 0
+		if (parts[1] < 12)
+		    parts[1]++;
+		u_int64_t time = Time::toEpoch(parts[0],parts[1],parts[2],parts[3],parts[4],parts[5]);
+		obj = new JsDate(mutex(),1000 * time + parts[6],true);
+	    }
+	    break;
+	default:
+	    return 0;
+    }
+    if (obj && ref())
+	obj->params().addParam(new ExpWrapper(this,protoName()));
+    return obj;
+}
+
 bool JsDate::runNative(ObjList& stack, const ExpOperation& oper, GenObject* context)
 {
     XDebug(DebugAll,"JsDate::runNative() '%s' in '%s' [%p]",
 	oper.name().c_str(),toString().c_str(),this);
     if (oper.name() == YSTRING("now")) {
 	// Returns the number of milliseconds elapsed since 1 January 1970 00:00:00 UTC.
-	ExpEvaluator::pushOne(stack,new ExpOperation((long int)Time::msecNow()));  // should check conversion from u_int64_t 
+	ExpEvaluator::pushOne(stack,new ExpOperation((int64_t)Time::msecNow()));
     }
     else if (oper.name() == YSTRING("getDate")) {
 	// Returns the day of the month for the specified date according to local time.
 	// The value returned by getDate is an integer between 1 and 31.
-	// !NOTE : assuming that the date for this object is kept in params()
-	unsigned int time = params().getIntValue("time");
 	int year = 0;
 	unsigned int month = 0, day = 0, hour = 0, minute = 0, sec = 0;
-	if (Time::toDateTime(time,year,month,day,hour,minute,sec))
-	    ExpEvaluator::pushOne(stack,new ExpOperation((long int)day));
+	if (Time::toDateTime(m_time + m_offs,year,month,day,hour,minute,sec))
+	    ExpEvaluator::pushOne(stack,new ExpOperation((int64_t)day));
 	else
 	    return false;
     }
     else if (oper.name() == YSTRING("getDay")) {
 	// Get the day of the week for the date (0 is Sunday and returns values 0-6)
-	// TODO
+	int year = 0;
+	unsigned int month = 0, day = 0, hour = 0, minute = 0, sec = 0, wday = 0;
+	if (Time::toDateTime(m_time + m_offs,year,month,day,hour,minute,sec,&wday))
+	    ExpEvaluator::pushOne(stack,new ExpOperation((int64_t)wday));
+	else
+	    return false;
     }
     else if (oper.name() == YSTRING("getFullYear")) {
 	// Returns the year of the specified date according to local time.
-	// !NOTE : assuming that the date for this object is kept in params()
-	unsigned int time = params().getIntValue("time");
 	int year = 0;
 	unsigned int month = 0, day = 0, hour = 0, minute = 0, sec = 0;
-	if (Time::toDateTime(time,year,month,day,hour,minute,sec))
-	    ExpEvaluator::pushOne(stack,new ExpOperation((long int)year));
+	if (Time::toDateTime(m_time + m_offs,year,month,day,hour,minute,sec))
+	    ExpEvaluator::pushOne(stack,new ExpOperation((int64_t)year));
 	else
 	    return false;
     }
     else if (oper.name() == YSTRING("getHours")) {
 	// Returns the hour ( 0 - 23) of the specified date according to local time.
-	// !NOTE : assuming that the date for this object is kept in params()
-	unsigned int time = params().getIntValue("time");
 	int year = 0;
 	unsigned int month = 0, day = 0, hour = 0, minute = 0, sec = 0;
-	if (Time::toDateTime(time,year,month,day,hour,minute,sec))
-	    ExpEvaluator::pushOne(stack,new ExpOperation((long int)hour));
+	if (Time::toDateTime(m_time + m_offs,year,month,day,hour,minute,sec))
+	    ExpEvaluator::pushOne(stack,new ExpOperation((int64_t)hour));
 	else
 	    return false;
     }
     else if (oper.name() == YSTRING("getMilliseconds")) {
-	// TODO
+	// Returns just the milliseconds part ( 0 - 999 )
+	ExpEvaluator::pushOne(stack,new ExpOperation((int64_t)m_msec));
     }
     else if (oper.name() == YSTRING("getMinutes")) {
 	// Returns the minute ( 0 - 59 ) of the specified date according to local time.
-	// !NOTE : assuming that the date for this object is kept in params()
-	unsigned int time = params().getIntValue("time");
 	int year = 0;
 	unsigned int month = 0, day = 0, hour = 0, minute = 0, sec = 0;
-	if (Time::toDateTime(time,year,month,day,hour,minute,sec))
-	    ExpEvaluator::pushOne(stack,new ExpOperation((long int)minute));
+	if (Time::toDateTime(m_time + m_offs,year,month,day,hour,minute,sec))
+	    ExpEvaluator::pushOne(stack,new ExpOperation((int64_t)minute));
 	else
 	    return false;
     }
     else if (oper.name() == YSTRING("getMonth")) {
-	// Returns the minute ( 0 - 11 ) of the specified date according to local time.
-	// !NOTE : assuming that the date for this object is kept in params()
-	unsigned int time = params().getIntValue("time");
+	// Returns the month ( 0 - 11 ) of the specified date according to local time.
 	int year = 0;
 	unsigned int month = 0, day = 0, hour = 0, minute = 0, sec = 0;
-	if (Time::toDateTime(time,year,month,day,hour,minute,sec))
-	    ExpEvaluator::pushOne(stack,new ExpOperation((long int)month - 1));
+	if (Time::toDateTime(m_time + m_offs,year,month,day,hour,minute,sec))
+	    ExpEvaluator::pushOne(stack,new ExpOperation((int64_t)month - 1));
 	else
 	    return false;
     }
     else if (oper.name() == YSTRING("getSeconds")) {
 	// Returns the second ( 0 - 59 ) of the specified date according to local time.
-	// !NOTE : assuming that the date for this object is kept in params()
-	unsigned int time = params().getIntValue("time");
 	int year = 0;
 	unsigned int month = 0, day = 0, hour = 0, minute = 0, sec = 0;
-	if (Time::toDateTime(time,year,month,day,hour,minute,sec))
-	    ExpEvaluator::pushOne(stack,new ExpOperation((long int)sec));
+	if (Time::toDateTime(m_time + m_offs,year,month,day,hour,minute,sec))
+	    ExpEvaluator::pushOne(stack,new ExpOperation((int64_t)sec));
 	else
 	    return false;
     }
     else if (oper.name() == YSTRING("getTime")) {
-	// TODO
+	// Returns the time in milliseconds since UNIX Epoch
+	ExpEvaluator::pushOne(stack,new ExpOperation(1000 * ((int64_t)m_time) + (int64_t)m_msec));
+    }
+    else if (oper.name() == YSTRING("getTimezoneOffset")) {
+	// Returns the UTC to local difference in minutes, positive goes west
+	ExpEvaluator::pushOne(stack,new ExpOperation((int64_t)(m_offs / -60)));
     }
     else if (oper.name() == YSTRING("getUTCDate")) {
-	// TODO
+	// Returns the day of the month for the specified date according to local time.
+	// The value returned by getDate is an integer between 1 and 31.
+	int year = 0;
+	unsigned int month = 0, day = 0, hour = 0, minute = 0, sec = 0;
+	if (Time::toDateTime(m_time,year,month,day,hour,minute,sec))
+	    ExpEvaluator::pushOne(stack,new ExpOperation((int64_t)day));
+	else
+	    return false;
     }
     else if (oper.name() == YSTRING("getUTCDay")) {
-	// TODO
+	// Get the day of the week for the date (0 is Sunday and returns values 0-6)
+	int year = 0;
+	unsigned int month = 0, day = 0, hour = 0, minute = 0, sec = 0, wday = 0;
+	if (Time::toDateTime(m_time,year,month,day,hour,minute,sec,&wday))
+	    ExpEvaluator::pushOne(stack,new ExpOperation((int64_t)wday));
+	else
+	    return false;
     }
     else if (oper.name() == YSTRING("getUTCFullYear")) {
-	// TODO
+	// Returns the year of the specified date according to local time.
+	int year = 0;
+	unsigned int month = 0, day = 0, hour = 0, minute = 0, sec = 0;
+	if (Time::toDateTime(m_time,year,month,day,hour,minute,sec))
+	    ExpEvaluator::pushOne(stack,new ExpOperation((int64_t)year));
+	else
+	    return false;
     }
     else if (oper.name() == YSTRING("getUTCHours")) {
-	// TODO
+	// Returns the hour ( 0 - 23) of the specified date according to local time.
+	int year = 0;
+	unsigned int month = 0, day = 0, hour = 0, minute = 0, sec = 0;
+	if (Time::toDateTime(m_time,year,month,day,hour,minute,sec))
+	    ExpEvaluator::pushOne(stack,new ExpOperation((int64_t)hour));
+	else
+	    return false;
     }
     else if (oper.name() == YSTRING("getUTCMilliseconds")) {
-	// TODO
+	// Returns just the milliseconds part ( 0 - 999 )
+	ExpEvaluator::pushOne(stack,new ExpOperation((int64_t)m_msec));
     }
     else if (oper.name() == YSTRING("getUTCMinutes")) {
-	// TODO
+	// Returns the minute ( 0 - 59 ) of the specified date according to local time.
+	int year = 0;
+	unsigned int month = 0, day = 0, hour = 0, minute = 0, sec = 0;
+	if (Time::toDateTime(m_time,year,month,day,hour,minute,sec))
+	    ExpEvaluator::pushOne(stack,new ExpOperation((int64_t)minute));
+	else
+	    return false;
     }
     else if (oper.name() == YSTRING("getUTCMonth")) {
-	// TODO
+	// Returns the month ( 0 - 11 ) of the specified date according to local time.
+	int year = 0;
+	unsigned int month = 0, day = 0, hour = 0, minute = 0, sec = 0;
+	if (Time::toDateTime(m_time,year,month,day,hour,minute,sec))
+	    ExpEvaluator::pushOne(stack,new ExpOperation((int64_t)month - 1));
+	else
+	    return false;
     }
     else if (oper.name() == YSTRING("getUTCSeconds")) {
-	// TODO
+	// Returns the second ( 0 - 59 ) of the specified date according to local time.
+	int year = 0;
+	unsigned int month = 0, day = 0, hour = 0, minute = 0, sec = 0;
+	if (Time::toDateTime(m_time,year,month,day,hour,minute,sec))
+	    ExpEvaluator::pushOne(stack,new ExpOperation((int64_t)sec));
+	else
+	    return false;
     }
     else
 	return JsObject::runNative(stack,oper,context);
