@@ -4,21 +4,18 @@
  * This file is part of the YATE Project http://YATE.null.ro
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2011 Null Team
+ * Copyright (C) 2011-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include "yatescript.h"
@@ -55,14 +52,14 @@ bool ScriptParser::parseFile(const char* name, bool fragment)
     if (!f.openPath(name))
 	return false;
     int64_t len = f.length();
-    if (len <= 0 || len > 65535)
+    if (len <= 0 || len > 262143)
 	return false;
     DataBlock data(0,(unsigned int)len+1);
     char* text = (char*)data.data();
     if (f.readData(text,(int)len) != len)
 	return false;
     text[len] = '\0';
-    return parse(text,fragment);
+    return parse(text,fragment,name,(int)len);
 }
 
 void ScriptParser::setCode(ScriptCode* code)
@@ -81,7 +78,7 @@ ScriptContext* ScriptParser::createContext() const
     return new BasicContext;
 }
 
-ScriptRun* ScriptParser::createRunner(ScriptCode* code, ScriptContext* context) const
+ScriptRun* ScriptParser::createRunner(ScriptCode* code, ScriptContext* context, const char* title) const
 {
     if (!code)
 	return 0;
@@ -102,9 +99,9 @@ bool ScriptParser::callable(const String& name)
 // RTTI Interface access
 void* ScriptContext::getObject(const String& name) const
 {
-    if (name == YSTRING("ScriptContext"))
+    if (name == YATOM("ScriptContext"))
 	return const_cast<ScriptContext*>(this);
-    if (name == YSTRING("ExpExtender"))
+    if (name == YATOM("ExpExtender"))
 	return const_cast<ExpExtender*>(static_cast<const ExpExtender*>(this));
     return RefObject::getObject(name);
 }
@@ -233,12 +230,13 @@ ScriptRun::ScriptRun(ScriptCode* code, ScriptContext* context)
     if (code)
 	code->ref();
     m_code = code;
-    if (context)
+    if (context) {
 	context->ref();
+	m_context = context;
+    }
     else
-	context = new BasicContext;
-    m_context = context;
-    reset();
+	m_context = new BasicContext;
+    reset(!context);
 }
 
 ScriptRun::~ScriptRun()
@@ -257,12 +255,12 @@ const char* ScriptRun::textState(Status state)
 }
 
 // Reset script (but not the context) to initial state
-ScriptRun::Status ScriptRun::reset()
+ScriptRun::Status ScriptRun::reset(bool init)
 {
     Lock mylock(this);
     // TODO
     m_stack.clear();
-    return (m_state = (m_code && m_code->initialize(m_context)) ? Incomplete : Invalid);
+    return (m_state = (m_code && (!init || m_code->initialize(m_context))) ? Incomplete : Invalid);
 }
 
 // Resume execution, run one or more instructions of code
@@ -305,9 +303,9 @@ ScriptRun::Status ScriptRun::execute()
 }
 
 // Execute instructions until succeeds or fails
-ScriptRun::Status ScriptRun::run()
+ScriptRun::Status ScriptRun::run(bool init)
 {
-    reset();
+    reset(init);
     ScriptRun::Status s = state();
     while (Incomplete == s)
 	s = execute();
