@@ -4,21 +4,18 @@
  * This file is part of the YATE Project http://YATE.null.ro
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2011 Null Team
+ * Copyright (C) 2011-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #ifndef __YATESCRIPT_H
@@ -116,12 +113,115 @@ public:
     virtual bool runAssign(ObjList& stack, const ExpOperation& oper, GenObject* context);
 };
 
+/**
+ * A class used to keep a parsing context
+ * @short The parsing context
+ */
+class YSCRIPT_API ParsePoint
+{
+public:
+    /**
+     * Constructor
+     * @param expr Expression to be parsed
+     * @param eval ExpEvaluator associated with this parsing context
+     * @param lineNo The line number that is currently parsed
+     * @param fileName File name associated with this context
+     * @param seps Searched separator during parsing
+     */
+    explicit inline ParsePoint(const char*& expr, ExpEvaluator* eval = 0, unsigned int lineNo = 0,
+			const char* fileName = 0, const char* seps = 0)
+	: m_expr(expr), m_searchedSeps(seps), m_count(0), m_foundSep(0), m_lineNo(lineNo), m_eval(eval), m_fileName(fileName)
+	{ }
+
+    /**
+     * Assignment operator
+     * @param parsePoint Parsing context which is to be assigned to this parsing context
+     */
+    ParsePoint& operator=(ParsePoint& parsePoint);
+
+    /**
+     * Cast operator to const char*&
+     */
+    inline operator const char*&()
+	{ return m_expr; }
+
+    /**
+     * Assignement from const char*
+     */
+    inline ParsePoint& operator=(const char* newExpr)
+    {
+	m_expr = newExpr;
+	return *this;
+    }
+
+    /**
+     * Prefix incrementation operator. Incrementes the internal expression
+     */
+    inline ParsePoint& operator++() // prefix
+    {
+	++m_expr;
+	return *this;
+    }
+
+    /**
+     * Postfix incrementation operator. Incrementes the internal expression
+     */
+    inline ParsePoint& operator++(int unused) // postfix
+    {
+	++m_expr;
+	return *this;
+    }
+
+    /**
+     * Get first char in the parsed expression
+     * @return First char in the expression to be parsed
+     */
+    inline char firstChar()
+	{ return *m_expr; }
+
+    /**
+     * Expression to be parsed
+     */
+    const char* m_expr;
+
+    /**
+     * Searched instruction separators
+     */
+    const char* m_searchedSeps;
+
+    /**
+     * Number of how many times the parser must encouter a separator
+     */
+    unsigned int m_count;
+
+    /**
+     * Separator that the parser encountered
+     */
+    char m_foundSep;
+
+    /**
+     * Line numbet at which parsing is taking place
+     */
+    unsigned int m_lineNo;
+
+    /**
+     * ExpEvaluator associated with this parsing context
+     */
+    ExpEvaluator* m_eval;
+
+    /**
+    * File name associated for this context
+    */
+    String m_fileName;
+};
+
 /**
  * A class used to build stack based (posifix) expression parsers and evaluators
  * @short An expression parser and evaluator
  */
 class YSCRIPT_API ExpEvaluator : public DebugEnabler
 {
+    friend class ParsePoint;
 public:
     /**
      * Parsing styles
@@ -229,11 +329,11 @@ public:
 
     /**
      * Parse and compile an expression
-     * @param expr Pointer to expression to compile
+     * @param expr Parsing context to compile
      * @param context Pointer to arbitrary object to be passed to called methods
      * @return Number of expressions compiled, zero on error
      */
-    int compile(const char* expr, GenObject* context = 0);
+    int compile(ParsePoint& expr, GenObject* context = 0);
 
     /**
      * Evaluate the expression, optionally return results
@@ -296,8 +396,7 @@ public:
      * Check if the expression is empty (no operands or operators)
      * @return True if the expression is completely empty
      */
-    inline bool null() const
-	{ return m_opcodes.count() == 0; }
+    virtual bool null() const;
 
     /**
      * Dump a list of operations according to current operators dictionary
@@ -310,8 +409,7 @@ public:
      * Dump the postfix expression according to current operators dictionary
      * @param res Result string representation of operations
      */
-    inline void dump(String& res) const
-	{ return dump(m_opcodes,res); }
+    virtual void dump(String& res) const;
 
     /**
      * Dump a list of operations according to current operators dictionary
@@ -414,10 +512,10 @@ public:
 protected:
     /**
      * Method to skip over whitespaces, count parsed lines too
-     * @param expr Pointer to expression cursor, gets advanced
+     * @param expr Current parsing context, advances on expression to be compiled
      * @return First character after whitespaces where expr points
      */
-    virtual char skipWhites(const char*& expr);
+    virtual char skipWhites(ParsePoint& expr);
 
     /**
      * Helper method to conditionally convert to lower case
@@ -437,6 +535,20 @@ protected:
      */
     Opcode getOperator(const char*& expr, const TokenDict* operators, bool caseInsensitive = false) const;
 
+    /**
+     * Check if a character can be a letter character in a keyword or identifier
+     * @param c Character to check
+     * @return True if the character can be part of a keyword or identifier
+     */
+    virtual bool keywordLetter(char c) const;
+
+    /**
+     * Check if a character can be can be a digit character in a keyword or identifier
+     * @param c Character to check
+     * @return True if the character can be part of a keyword or identifier
+     */
+    virtual bool keywordDigit(char c) const;
+
     /**
      * Check if a character can be part of a keyword or identifier
      * @param c Character to check
@@ -496,58 +608,59 @@ protected:
 
     /**
      * Runs the parser and compiler for one (sub)expression
-     * @param expr Pointer to text to parse, gets advanced
+     * @param expr Current parsing context, advances on expression to be compiled
      * @param stop Optional character expected after the expression
      * @param nested User defined object to pass for nested parsing
      * @return True if one expression was compiled and a separator follows
      */
-    bool runCompile(const char*& expr, char stop, GenObject* nested = 0);
+    bool runCompile(ParsePoint& expr, char stop, GenObject* nested = 0);
 
     /**
      * Runs the parser and compiler for one (sub)expression
-     * @param expr Pointer to text to parse, gets advanced
+     * @param expr Current parsing context, advances on expression to be compiled
      * @param stop Optional list of possible characters expected after the expression
      * @param nested User defined object to pass for nested parsing
      * @return True if one expression was compiled and a separator follows
      */
-    virtual bool runCompile(const char*& expr, const char* stop = 0, GenObject* nested = 0);
+    virtual bool runCompile(ParsePoint& expr, const char* stop = 0, GenObject* nested = 0);
 
     /**
      * Skip over comments and whitespaces
-     * @param expr Pointer to expression cursor, gets advanced
+     * @param expr Current parsing context, advances on expression to be compiled
      * @param context Pointer to arbitrary object to be passed to called methods
      * @return First character after comments or whitespaces where expr points
      */
-    virtual char skipComments(const char*& expr, GenObject* context = 0);
+    virtual char skipComments(ParsePoint& expr, GenObject* context = 0);
 
     /**
      * Process top-level preprocessor directives
-     * @param expr Pointer to expression cursor, gets advanced
+     * @param expr Current parsing context, advances on expression to be compiled
      * @param context Pointer to arbitrary object to be passed to called methods
      * @return Number of expressions compiled, negative if no more directives
      */
-    virtual int preProcess(const char*& expr, GenObject* context = 0);
+    virtual int preProcess(ParsePoint& expr, GenObject* context = 0);
 
     /**
      * Returns next operator in the parsed text
-     * @param expr Pointer to text to parse, gets advanced if succeeds
+     * @param expr Current parsing context, advances on expression to be compiled if it succeeds
      * @return Operator code, OpcNone on failure
      */
-    virtual Opcode getOperator(const char*& expr);
+    virtual Opcode getOperator(ParsePoint& expr);
 
     /**
      * Returns next unary operator in the parsed text
-     * @param expr Pointer to text to parse, gets advanced if succeeds
+     * @param expr Current parsing context, advances on expression to be compiled if it succeeds
      * @return Operator code, OpcNone on failure
      */
-    virtual Opcode getUnaryOperator(const char*& expr);
+    virtual Opcode getUnaryOperator(ParsePoint& expr);
 
     /**
      * Returns next unary postfix operator in the parsed text
-     * @param expr Pointer to text to parse, gets advanced if succeeds
+     * @param expr Current parsing context, advances on expression to be compiled if it succeeds
+     * @param precedence The precedence of the previous operator
      * @return Operator code, OpcNone on failure
      */
-    virtual Opcode getPostfixOperator(const char*& expr);
+    virtual Opcode getPostfixOperator(ParsePoint& expr, int precedence = 0);
 
     /**
      * Helper method to get the canonical name of an operator
@@ -572,57 +685,58 @@ protected:
 
     /**
      * Check if we are at an expression separator and optionally skip past it
-     * @param expr Pointer to text to check, gets advanced if asked to remove separator
+     * @param expr Current parsing context to check, advances on expression to be compiled if asked to remove separator
      * @param remove True to skip past the found separator
      * @return True if a separator was found
      */
-    virtual bool getSeparator(const char*& expr, bool remove);
+    virtual bool getSeparator(ParsePoint& expr, bool remove);
 
     /**
      * Get an instruction or block, advance parsing pointer past it
-     * @param expr Pointer to text to parse, gets advanced on success
+     * @param expr Current parsing context, advances on expression to be compiled if it succeeds
      * @param stop Optional character expected after the instruction
      * @param nested User defined object passed from nested parsing
      * @return True if succeeded, must add the operands internally
      */
-    virtual bool getInstruction(const char*& expr, char stop = 0, GenObject* nested = 0);
+    virtual bool getInstruction(ParsePoint& expr, char stop = 0, GenObject* nested = 0);
 
     /**
      * Get an operand, advance parsing pointer past it
-     * @param expr Pointer to text to parse, gets advanced on success
+     * @param expr Current parsing context, advances on expression to be compiled if it succeeds
      * @param endOk Consider reaching the end of string a success
+     * @param precedence The precedence of the previous operator
      * @return True if succeeded, must add the operand internally
      */
-    virtual bool getOperand(const char*& expr, bool endOk = true);
+    virtual bool getOperand(ParsePoint& expr, bool endOk = true, int precedence = 0);
 
     /**
      * Get an inline simple type, usually string or number
-     * @param expr Pointer to text to parse, gets advanced on success
+     * @param expr Current parsing context, advances on expression to be compiled if it succeeds
      * @param constOnly Return only inline constants
      * @return True if succeeded, must add the operand internally
      */
-    virtual bool getSimple(const char*& expr, bool constOnly = false);
+    virtual bool getSimple(ParsePoint& expr, bool constOnly = false);
 
     /**
      * Get a numerical operand, advance parsing pointer past it
-     * @param expr Pointer to text to parse, gets advanced on success
+     * @param expr Current parsing context, advances on expression to be compiled if it succeeds
      * @return True if succeeded, must add the operand internally
      */
-    virtual bool getNumber(const char*& expr);
+    virtual bool getNumber(ParsePoint& expr);
 
     /**
      * Get a string operand, advance parsing pointer past it
-     * @param expr Pointer to text to parse, gets advanced on success
+     * @param expr Current parsing context, advances on expression to be compiled if it succeeds
      * @return True if succeeded, must add the operand internally
      */
-    virtual bool getString(const char*& expr);
+    virtual bool getString(ParsePoint& expr);
 
     /**
      * Get a function call, advance parsing pointer past it
-     * @param expr Pointer to text to parse, gets advanced on success
+     * @param expr Current parsing context, advances on expression to be compiled if it succeeds
      * @return True if succeeded, must add the operand internally
      */
-    virtual bool getFunction(const char*& expr);
+    virtual bool getFunction(ParsePoint& expr);
 
     /**
      * Helper method - get a string, advance parsing pointer past it
@@ -643,10 +757,10 @@ protected:
 
     /**
      * Get a field keyword, advance parsing pointer past it
-     * @param expr Pointer to text to parse, gets advanced on success
+     * @param expr Current parsing context, advances on expression to be compiled if it succeeds
      * @return True if succeeded, must add the operand internally
      */
-    virtual bool getField(const char*& expr);
+    virtual bool getField(ParsePoint& expr);
 
     /**
      * Add an aready built operation to the expression and set its line number
@@ -666,11 +780,11 @@ protected:
     /**
      * Add a simple operator to the expression
      * @param oper Operator code to add
-     * @param value Integer value to add
+     * @param value 64 bit nteger value to add
      * @param barrier True to create an evaluator stack barrier
      * @return Newly added operation
      */
-    ExpOperation* addOpcode(Opcode oper, long int value, bool barrier = false);
+    ExpOperation* addOpcode(Opcode oper, int64_t value, bool barrier = false);
 
     /**
      * Add a string constant to the expression
@@ -684,7 +798,7 @@ protected:
      * @param value Integer value to add, will be pushed on execution
      * @return Newly added operation
      */
-    ExpOperation* addOpcode(long int value);
+    ExpOperation* addOpcode(int64_t value);
 
     /**
      * Add a boolean constant to the expression
@@ -701,7 +815,7 @@ protected:
      * @param barrier True to create an exavuator stack barrier
      * @return Newly added operation
      */
-    ExpOperation* addOpcode(Opcode oper, const String& name, long int value = 0, bool barrier = false);
+    ExpOperation* addOpcode(Opcode oper, const String& name, int64_t value = 0, bool barrier = false);
 
     /**
      * Remove from the code and return the last operation
@@ -792,6 +906,11 @@ protected:
      */
     ObjList m_opcodes;
 
+    /**
+     * Internally used for faster appending to the operator codes list
+     */
+    ObjList* m_lastOpcode;
+
     /**
      * Flag that we encountered a parse or compile error
      */
@@ -803,6 +922,7 @@ protected:
     unsigned int m_lineNo;
 
 private:
+    bool getOperandInternal(ParsePoint& expr, bool endOk, int precedence);
     ExpExtender* m_extender;
 };
 
@@ -819,8 +939,8 @@ public:
      * Special value that is not recognized as an integer value
      * @return A value that indicates a non-integer value
      */
-    inline static long int nonInteger()
-	{ return LONG_MIN; }
+    inline static int64_t nonInteger()
+	{ return LLONG_MIN; }
 
     /**
      * Copy constructor
@@ -852,7 +972,7 @@ public:
     inline explicit ExpOperation(const String& value, const char* name = 0, bool autoNum = false)
 	: NamedString(name,value),
 	  m_opcode(ExpEvaluator::OpcPush),
-	  m_number(autoNum ? value.toLong(nonInteger()) : nonInteger()),
+	  m_number(autoNum ? value.toInt64(nonInteger()) : nonInteger()),
 	  m_lineNo(0), m_barrier(false)
 	{ if (autoNum && value.isBoolean()) m_number = value.toBoolean() ? 1 : 0; }
 
@@ -867,15 +987,15 @@ public:
 	{ }
 
     /**
-     * Push Number constructor
+     * Push 64 bit Number constructor
      * @param value Integer constant to push on stack on execution
      * @param name Optional of the newly created constant
      */
-    inline explicit ExpOperation(long int value, const char* name = 0)
+    inline explicit ExpOperation(int64_t value, const char* name = 0)
 	: NamedString(name,"NaN"),
 	  m_opcode(ExpEvaluator::OpcPush),
 	  m_number(value), m_lineNo(0), m_barrier(false)
-	{ if (value != nonInteger()) String::operator=((int)value); }
+	{ if (value != nonInteger()) String::operator=(value); }
 
     /**
      * Push Boolean constructor
@@ -895,7 +1015,7 @@ public:
      * @param value Optional integer constant used as function parameter count
      * @param barrier True if the operation is an expression barrier on the stack
      */
-    inline ExpOperation(ExpEvaluator::Opcode oper, const char* name = 0, long int value = nonInteger(), bool barrier = false)
+    inline ExpOperation(ExpEvaluator::Opcode oper, const char* name = 0, int64_t value = nonInteger(), bool barrier = false)
 	: NamedString(name,""),
 	  m_opcode(oper), m_number(value), m_lineNo(0), m_barrier(barrier)
 	{ }
@@ -912,6 +1032,19 @@ public:
 	  m_opcode(oper), m_number(nonInteger()), m_lineNo(0), m_barrier(barrier)
 	{ }
 
+    /**
+     * Constructor from components
+     * @param oper Operation code
+     * @param name Optional name of the operation or result
+     * @param value String value of operation
+     * @param number Integer value
+     * @param barrier True if the operation is an expression barrier on the stack
+     */
+    inline ExpOperation(ExpEvaluator::Opcode oper, const char* name, const char* value, int64_t number, bool barrier)
+	: NamedString(name,value),
+	  m_opcode(oper), m_number(number), m_lineNo(0), m_barrier(barrier)
+	{ }
+
     /**
      * Retrieve the code of this operation
      * @return Operation code as declared in the expression evaluator
@@ -930,7 +1063,7 @@ public:
      * Retrieve the number stored in this operation
      * @return Stored number
      */
-    inline long int number() const
+    inline int64_t number() const
 	{ return m_number; }
 
     /**
@@ -959,14 +1092,14 @@ public:
      * @param num Numeric value to assign to the operation
      * @return Assigned number
      */
-    inline long int operator=(long int num)
-	{ m_number = num; String::operator=((int)num); return num; }
+    inline int64_t operator=(int64_t num)
+	{ m_number = num; String::operator=(num); return num; }
 
     /**
      * Retrieve the numeric value of the operation
      * @return Number contained in operation, zero if not a number
      */
-    virtual long int valInteger() const;
+    virtual int64_t valInteger() const;
 
     /**
      * Retrieve the boolean value of the operation
@@ -974,6 +1107,12 @@ public:
      */
     virtual bool valBoolean() const;
 
+    /**
+     * Retrieve the name of the type of the value of this operation
+     * @return Name of the type of the value
+     */
+    virtual const char* typeOf() const;
+
     /**
      * Clone and rename method
      * @param name Name of the cloned operation
@@ -998,7 +1137,7 @@ public:
 
 private:
     ExpEvaluator::Opcode m_opcode;
-    long int m_number;
+    int64_t m_number;
     unsigned int m_lineNo;
     bool m_barrier;
 };
@@ -1015,9 +1154,10 @@ public:
      * Constructor
      * @param name Name of the function
      * @param argc Number of arguments expected by function
+     * @param barrier True if the function is an expression barrier on the stack
      */
-    inline ExpFunction(const char* name, long int argc = 0)
-	: ExpOperation(ExpEvaluator::OpcFunc,name,argc)
+    inline ExpFunction(const char* name, long int argc = 0, bool barrier = false)
+	: ExpOperation(ExpEvaluator::OpcFunc,name,argc,barrier)
 	{ if (name) (*this) << "[function " << name << "()]"; }
 
     /**
@@ -1083,6 +1223,12 @@ public:
      */
     virtual bool valBoolean() const;
 
+    /**
+     * Retrieve the name of the type of the value of this operation
+     * @return Name of the type of the value
+     */
+    virtual const char* typeOf() const;
+
     /**
      * Clone and rename method
      * @param name Name of the cloned operation
@@ -1345,9 +1491,10 @@ public:
     /**
      * Create a runner adequate for this block of parsed code
      * @param context Script context, must not be NULL
+     * @param title An optional name for the runner
      * @return A new script runner, NULL if context is NULL or feature is not supported
      */
-    virtual ScriptRun* createRunner(ScriptContext* context)
+    virtual ScriptRun* createRunner(ScriptContext* context, const char* title = 0)
 	{ return 0; }
 };
 
@@ -1508,9 +1655,10 @@ public:
 
     /**
      * Resets code execution to the beginning, does not clear context
+     * @param init Initialize context
      * @return Status of the runtime after reset
      */
-    virtual Status reset();
+    virtual Status reset(bool init = false);
 
     /**
      * Execute script from where it was left, may stop and return Incomplete state
@@ -1520,9 +1668,10 @@ public:
 
     /**
      * Execute script from the beginning until it returns a final state
+     * @param init Initialize context
      * @return Final status of the runtime after code execution
      */
-    virtual Status run();
+    virtual Status run(bool init = true);
 
     /**
      * Pause the script, make it return Incomplete state
@@ -1602,9 +1751,11 @@ public:
      * Parse a string as script source code
      * @param text Source code text
      * @param fragment True if the code is just an included fragment
+     * @param file Name of the file that is being parsed
+     * @param len Length of text, negative if unknown
      * @return True if the text was successfully parsed
      */
-    virtual bool parse(const char* text, bool fragment = false) = 0;
+    virtual bool parse(const char* text, bool fragment = false, const char* file = 0, int len = -1) = 0;
 
     /**
      * Parse a file as script source code
@@ -1637,17 +1788,19 @@ public:
      * Create a runner adequate for a block of parsed code
      * @param code Parsed code block
      * @param context Script context, an empty one will be allocated if NULL
+     * @param title An optional name for the runner
      * @return A new script runner, NULL if code is NULL
      */
-    virtual ScriptRun* createRunner(ScriptCode* code, ScriptContext* context = 0) const;
+    virtual ScriptRun* createRunner(ScriptCode* code, ScriptContext* context = 0, const char* title = 0) const;
 
     /**
      * Create a runner adequate for the parsed code
      * @param context Script context, an empty one will be allocated if NULL
+     * @param title An optional name for the runner
      * @return A new script runner, NULL if code is not yet parsed
      */
-    inline ScriptRun* createRunner(ScriptContext* context = 0) const
-	{ return createRunner(code(),context); }
+    inline ScriptRun* createRunner(ScriptContext* context = 0, const char* title = 0) const
+	{ return createRunner(code(),context,title); }
 
     /**
      * Check if a script has a certain function or method
@@ -1693,6 +1846,14 @@ public:
      */
     JsObject(const char* name = "Object", Mutex* mtx = 0, bool frozen = false);
 
+    /**
+     * Constructor for an empty object
+     * @param mtx Pointer to the mutex that serializes this object
+     * @param name Full name of the object
+     * @param frozen True if the object is to be frozen from creation
+     */
+    JsObject(Mutex* mtx, const char* name, bool frozen = false);
+
     /**
      * Destructor
      */
@@ -1898,14 +2059,6 @@ public:
     static void printRecursive(const GenObject* obj);
 
 protected:
-    /**
-     * Constructor for an empty object
-     * @param mtx Pointer to the mutex that serializes this object
-     * @param name Full name of the object
-     * @param frozen True if the object is to be frozen from creation
-     */
-    JsObject(Mutex* mtx, const char* name, bool frozen = false);
-
     /**
      * Try to evaluate a single native method
      * @param stack Evaluation stack in use, parameters are popped off this stack
@@ -1972,6 +2125,20 @@ public:
     inline const ExpFunction* getFunc() const
 	{ return &m_func; }
 
+    /**
+     * Retrieve the first name assigned to this function
+     * @return The name of the property towhich this function was first assigned
+     */
+    inline const String& firstName() const
+	{ return m_name; }
+
+    /**
+     * Set the name of this function if still empty
+     * @param name Name to set as first assigned name
+     */
+    inline void firstName(const char* name)
+	{ if (m_name.null()) m_name = name; }
+
     /**
      * Retrieve the name of the N-th formal argument
      * @param index Index of the formal argument
@@ -2011,6 +2178,7 @@ private:
     long int m_label;
     ScriptCode* m_code;
     ExpFunction m_func;
+    String m_name;
 };
 
 /**
@@ -2027,13 +2195,30 @@ public:
      */
     JsArray(Mutex* mtx = 0);
 
+    /**
+     * Constructor for an empty array
+     * @param mtx Pointer to the mutex that serializes this object
+     * @param name Full name of the object
+     * @param frozen True if the object is to be frozen from creation
+     */
+    inline JsArray(Mutex* mtx, const char* name, bool frozen = false)
+	: JsObject(mtx,name,frozen), m_length(0)
+	{ }
+
     /**
      * Retrieve the length of the array
      * @return Number of numerically indexed objects in array
      */
-    inline long length() const
+    inline int32_t length() const
 	{ return m_length; }
 
+    /**
+     * Set the internal length to a specific value
+     * @param len Length of array to set
+     */
+    inline void setLength(int32_t len)
+	{ m_length = len; }
+
     /**
      * Add an item at the end of the array
      * @param item Item to add to array
@@ -2047,17 +2232,26 @@ public:
      */
     virtual JsObject* copy(Mutex* mtx) const;
 
-protected:
-    /*
-     * Constructor for an empty array
-     * @param mtx Pointer to the mutex that serializes this object
-     * @param name Full name of the object
-     * @param frozen True if the object is to be frozen from creation
+    /**
+     * Try to assign a value to a single field if object is not frozen and update array length.
+     * Reimplemented from JsObject
+     * @param stack Evaluation stack in use
+     * @param oper Field to assign to, contains the field name and new value
+     * @param context Pointer to arbitrary object passed from evaluation methods
+     * @return True if assignment succeeded
      */
-    inline JsArray(Mutex* mtx, const char* name, bool frozen = false)
-	: JsObject(mtx,name,frozen), m_length(0)
-	{ }
+    virtual bool runAssign(ObjList& stack, const ExpOperation& oper, GenObject* context);
 
+    /**
+     * Try to evaluate a single field
+     * @param stack Evaluation stack in use, field value must be pushed on it
+     * @param oper Field to evaluate
+     * @param context Pointer to arbitrary object passed from evaluation methods
+     * @return True if evaluation succeeded
+     */
+    virtual bool runField(ObjList& stack, const ExpOperation& oper, GenObject* context);
+
+protected:
     /**
      * Clone and rename method
      * @param name Name of the cloned object
@@ -2076,24 +2270,11 @@ protected:
      */
     bool runNative(ObjList& stack, const ExpOperation& oper, GenObject* context);
 
-    /**
-     * Synchronize the "length" parameter to the internally stored length
-     */
-    inline void setLength()
-	{ params().setParam("length",String((int)m_length)); }
-
-    /**
-     * Set the internal length and the "length" parameter to a specific value
-     * @param len Length of array to set
-     */
-    inline void setLength(long len)
-	{ m_length = len; params().setParam("length",String((int)len)); }
-
 private:
     bool runNativeSlice(ObjList& stack, const ExpOperation& oper, GenObject* context);
     bool runNativeSplice(ObjList& stack, const ExpOperation& oper, GenObject* context);
     bool runNativeSort(ObjList& stack, const ExpOperation& oper, GenObject* context);
-    long m_length;
+    int32_t m_length;
 };
 
 /**
@@ -2168,13 +2349,24 @@ class YSCRIPT_API JsParser : public ScriptParser
 {
     YCLASS(JsParser,ScriptParser)
 public:
+    /**
+     * Constructor
+     * @param allowLink True to allow linking of the code, false otherwise.
+     * @param allowTrace True to allow the script to enable performance tracing
+     */
+    inline JsParser(bool allowLink = true, bool allowTrace = false)
+	: m_allowLink(allowLink), m_allowTrace(allowTrace)
+	{ }
+
     /**
      * Parse a string as Javascript source code
      * @param text Source code text
      * @param fragment True if the code is just an included fragment
+     * @param file Name of the file that is being parsed
+     * @param len Length of text, negative if unknown
      * @return True if the text was successfully parsed
      */
-    virtual bool parse(const char* text, bool fragment = false);
+    virtual bool parse(const char* text, bool fragment = false, const char* file = 0, int len = -1);
 
     /**
      * Create a context adequate for Javascript code
@@ -2186,17 +2378,19 @@ public:
      * Create a runner adequate for a block of parsed Javascript code
      * @param code Parsed code block
      * @param context Javascript context, an empty one will be allocated if NULL
+     * @param title An optional name for the runner
      * @return A new Javascript runner, NULL if code is NULL
      */
-    virtual ScriptRun* createRunner(ScriptCode* code, ScriptContext* context = 0) const;
+    virtual ScriptRun* createRunner(ScriptCode* code, ScriptContext* context = 0, const char* title = 0) const;
 
     /**
      * Create a runner adequate for the parsed Javascript code
      * @param context Javascript context, an empty one will be allocated if NULL
+     * @param title An optional name for the runner
      * @return A new Javascript runner, NULL if code is not yet parsed
      */
-    inline ScriptRun* createRunner(ScriptContext* context = 0) const
-	{ return createRunner(code(),context); }
+    inline ScriptRun* createRunner(ScriptContext* context = 0, const char* title = 0) const
+	{ return createRunner(code(),context,title); }
 
     /**
      * Check if a script has a certain function or method
@@ -2219,12 +2413,49 @@ public:
 	{ return m_basePath; }
 
     /**
-     * Set the pase script path
+     * Set the base script path
      * @param path Base path to add to relative script paths
      */
     inline void basePath(const char* path)
 	{ m_basePath = path; }
 
+    /**
+     * Retrieve the last parsed file name
+     * @return Name of the successfully parsed file or an empty String
+     */
+    inline const String& parsedFile() const
+	{ return m_parsedFile; }
+
+    /**
+     * Check if the script or any includes have changed
+     * @param file Name of the file to check
+     * @return True if the script may have changed, false if not changed
+     */
+    bool scriptChanged(const char* file) const;
+
+    /**
+     * Check if the script or any includes have changed
+     * @param file Name of the file to check
+     * @param path New base path to check
+     * @return True if the script may have changed, false if not changed
+     */
+    inline bool scriptChanged(const char* file, const String& path) const
+	{ return (path != m_basePath) || scriptChanged(file); }
+
+    /**
+     * Set whether the Javascript code should be linked or not
+     * @param allowed True to allow linking, false otherwise
+     */
+    inline void link(bool allowed = true)
+	{ m_allowLink = allowed; }
+
+    /**
+     * Set whether the Javascript code can be traced or not
+     * @param allowed True to allow tracing, false otherwise
+     */
+    inline void trace(bool allowed = true)
+	{ m_allowTrace = allowed; }
+
     /**
      * Parse and run a piece of Javascript code
      * @param text Source code fragment to execute
@@ -2262,6 +2493,9 @@ public:
 
 private:
     String m_basePath;
+    String m_parsedFile;
+    bool m_allowLink;
+    bool m_allowTrace;
 };
 
 }; // namespace TelEngine
