@@ -5,21 +5,18 @@
  * SDP media handling
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2009 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include <yatesdp.h>
@@ -33,15 +30,19 @@ SDPSession::SDPSession(SDPParser* parser)
     : m_parser(parser), m_mediaStatus(MediaMissing),
       m_rtpForward(false), m_sdpForward(false), m_rtpMedia(0),
       m_sdpSession(0), m_sdpVersion(0),
-      m_secure(m_parser->m_secure), m_rfc2833(m_parser->m_rfc2833)
+      m_secure(m_parser->m_secure), m_rfc2833(m_parser->m_rfc2833),
+      m_ipv6(false), m_enabler(0), m_ptr(0)
 {
+    setSdpDebug();
 }
 
 SDPSession::SDPSession(SDPParser* parser, NamedList& params)
     : m_parser(parser), m_mediaStatus(MediaMissing),
       m_rtpForward(false), m_sdpForward(false), m_rtpMedia(0),
-      m_sdpSession(0), m_sdpVersion(0)
+      m_sdpSession(0), m_sdpVersion(0),
+      m_ipv6(false), m_enabler(0), m_ptr(0)
 {
+    setSdpDebug();
     m_rtpForward = params.getBoolValue("rtp_forward");
     m_secure = params.getBoolValue("secure",parser->m_secure);
     m_rfc2833 = parser->m_rfc2833;
@@ -58,7 +59,7 @@ bool SDPSession::setMedia(ObjList* media)
 {
     if (media == m_rtpMedia)
 	return false;
-    DDebug(m_parser,DebugAll,"SDPSession::setMedia(%p) [%p]",media,this);
+    DDebug(m_enabler,DebugAll,"SDPSession::setMedia(%p) [%p]",media,m_ptr);
     ObjList* tmp = m_rtpMedia;
     m_rtpMedia = media;
     bool chg = m_rtpMedia != 0;
@@ -73,6 +74,7 @@ bool SDPSession::setMedia(ObjList* media)
 	}
 	TelEngine::destruct(tmp);
     }
+    printRtpMedia("Set media");
     return chg;
 }
 
@@ -111,9 +113,11 @@ void SDPSession::setRfc2833(const String& value)
 bool SDPSession::dispatchRtp(SDPMedia* media, const char* addr, bool start,
     bool pick, RefObject* context)
 {
-    DDebug(m_parser,DebugAll,"SDPSession::dispatchRtp(%p,%s,%u,%u,%p) [%p]",
-	media,addr,start,pick,context,this);
+    DDebug(m_enabler,DebugAll,"SDPSession::dispatchRtp(%p,%s,%u,%u,%p) [%p]",
+	media,addr,start,pick,context,m_ptr);
     Message* m = buildChanRtp(media,addr,start,context);
+    if (m)
+	dispatchingRtp(m,media);
     if (!(m && Engine::dispatch(m))) {
 	TelEngine::destruct(m);
 	return false;
@@ -162,8 +166,8 @@ bool SDPSession::dispatchRtp(const char* addr, bool start, RefObject* context)
 {
     if (!m_rtpMedia)
 	return false;
-    DDebug(m_parser,DebugAll,"SDPSession::dispatchRtp(%s,%u,%p) [%p]",
-	addr,start,context,this);
+    DDebug(m_enabler,DebugAll,"SDPSession::dispatchRtp(%s,%u,%p) [%p]",
+	addr,start,context,m_ptr);
     bool ok = false;
     ObjList* o = m_rtpMedia->skipNull();
     while (o) {
@@ -173,9 +177,9 @@ bool SDPSession::dispatchRtp(const char* addr, bool start, RefObject* context)
 	    o = o->skipNext();
 	}
 	else {
-	    Debug(m_parser,DebugMild,
+	    Debug(m_enabler,DebugMild,
 		"Removing failed SDP media '%s' format '%s' from offer [%p]",
-		m->c_str(),m->format().safe(),this);
+		m->c_str(),m->format().safe(),m_ptr);
 	    o->remove();
 	    o = o->skipNull();
 	}
@@ -188,7 +192,7 @@ bool SDPSession::startRtp(RefObject* context)
 {
     if (m_rtpForward || !m_rtpMedia || (m_mediaStatus != MediaStarted))
 	return false;
-    DDebug(m_parser,DebugAll,"SDPSession::startRtp(%p) [%p]",context,this);
+    DDebug(m_enabler,DebugAll,"SDPSession::startRtp(%p) [%p]",context,m_ptr);
     bool ok = false;
     for (ObjList* o = m_rtpMedia->skipNull(); o; o = o->skipNext()) {
 	SDPMedia* m = static_cast<SDPMedia*>(o->get());
@@ -200,7 +204,7 @@ bool SDPSession::startRtp(RefObject* context)
 // Update from parameters. Build a default SDP if no media is found in params
 bool SDPSession::updateSDP(const NamedList& params)
 {
-    DDebug(m_parser,DebugAll,"SDPSession::updateSdp('%s') [%p]",params.c_str(),this);
+    DDebug(m_enabler,DebugAll,"SDPSession::updateSdp('%s') [%p]",params.c_str(),m_ptr);
     bool defaults = true;
     const char* sdpPrefix = params.getValue("osdp-prefix","osdp");
     ObjList* lst = 0;
@@ -278,7 +282,7 @@ bool SDPSession::updateSDP(const NamedList& params)
 // Return true if media changed
 bool SDPSession::updateRtpSDP(const NamedList& params)
 {
-    DDebug(m_parser,DebugAll,"SDPSession::updateRtpSDP(%s) [%p]",params.c_str(),this);
+    DDebug(m_enabler,DebugAll,"SDPSession::updateRtpSDP(%s) [%p]",params.c_str(),m_ptr);
     String addr;
     ObjList* tmp = updateRtpSDP(params,addr,m_rtpMedia);
     if (tmp) {
@@ -289,11 +293,36 @@ bool SDPSession::updateRtpSDP(const NamedList& params)
     return false;
 }
 
+// Utility used in createSDP
+static int addIP(String& buf, const char* addr, int family = SocketAddr::Unknown)
+{
+    if (family != SocketAddr::IPv4 && family != SocketAddr::IPv6) {
+	if (addr) {
+	    family = SocketAddr::family(addr);
+	    if (family != SocketAddr::IPv4 && family != SocketAddr::IPv6)
+		family = SocketAddr::IPv4;
+	}
+	else
+	    family = SocketAddr::IPv4;
+    }
+    if (family == SocketAddr::IPv4)
+	buf << "IN IP4 ";
+    else
+	buf << "IN IP6 ";
+    if (!TelEngine::null(addr))
+	buf << addr;
+    else if (family == SocketAddr::IPv4)
+	buf << SocketAddr::ipv4NullAddr();
+    else
+	buf << SocketAddr::ipv6NullAddr();
+    return family;
+}
+
 // Creates a SDP body from transport address and list of media descriptors
 // Use own list if given media list is 0
 MimeSdpBody* SDPSession::createSDP(const char* addr, ObjList* mediaList)
 {
-    DDebug(m_parser,DebugAll,"SDPSession::createSDP('%s',%p) [%p]",addr,mediaList,this);
+    DDebug(m_enabler,DebugAll,"SDPSession::createSDP('%s',%p) [%p]",addr,mediaList,m_ptr);
     if (!mediaList)
 	mediaList = m_rtpMedia;
     // if we got no media descriptors we simply create no SDP
@@ -307,12 +336,15 @@ MimeSdpBody* SDPSession::createSDP(const char* addr, ObjList* mediaList)
     // override the address with the externally advertised if needed
     if (addr && m_rtpNatAddr)
 	addr = m_rtpNatAddr;
+    if (!m_originAddr)
+	m_originAddr = addr ? addr : m_host.safe();
     // no address means on hold or muted
     String origin;
     origin << "yate " << m_sdpSession << " " << m_sdpVersion;
-    origin << " IN IP4 " << (addr ? addr : m_host.safe());
+    origin << " ";
+    int f = addIP(origin,m_originAddr);
     String conn;
-    conn << "IN IP4 " << (addr ? addr : "0.0.0.0");
+    addIP(conn,addr,f);
 
     MimeSdpBody* sdp = new MimeSdpBody;
     sdp->addLine("v","0");
@@ -352,11 +384,7 @@ MimeSdpBody* SDPSession::createSDP(const char* addr, ObjList* mediaList)
 	    const String* s = static_cast<const String*>(f->get());
 	    if (s) {
 		int mode = 0;
-		if (*s == "ilbc20")
-		    ptime = mode = 20;
-		else if (*s == "ilbc30")
-		    ptime = mode = 30;
-		else if (*s == "g729b")
+		if (*s == "g729b")
 		    continue;
 		int payload = s->toInteger(SDPParser::s_payloads,-1);
 		int defcode = payload;
@@ -372,16 +400,16 @@ MimeSdpBody* SDPSession::createSDP(const char* addr, ObjList* mediaList)
 			tmp = *mapping;
 			tmp >> "=" >> payload;
 			found = true;
-			XDebug(m_parser,DebugAll,"RTP mapped payload %d for '%s' [%p]",
-			    payload,s->c_str(),this);
+			XDebug(m_enabler,DebugAll,"RTP mapped payload %d for '%s' [%p]",
+			    payload,s->c_str(),m_ptr);
 			break;
 		    }
 		    String tmp2 = *mapping;
 		    int pload;
 		    tmp2 >> "=" >> pload;
 		    if (payload == pload) {
-			XDebug(m_parser,DebugAll,"RTP conflict for payload %d, allocating new [%p]",
-			    payload,this);
+			XDebug(m_enabler,DebugAll,"RTP conflict for payload %d, allocating new [%p]",
+			    payload,m_ptr);
 			payload = -1;
 			u_int32_t bmap = 0;
 			for (ObjList* sl = map; sl; sl = sl->next()) {
@@ -429,6 +457,10 @@ MimeSdpBody* SDPSession::createSDP(const char* addr, ObjList* mediaList)
 			defcode = payload;
 		    const char* map = lookup(defcode,SDPParser::s_rtpmap);
 		    if (map && m_parser->m_codecs.getBoolValue(*s,defcodecs && DataTranslator::canConvert(*s))) {
+			if (*s == "ilbc20")
+			    ptime = mode = 20;
+			else if (*s == "ilbc30")
+			    ptime = mode = 30;
 			frm << " " << payload;
 			String* temp = new String("rtpmap:");
 			*temp << payload << " " << map;
@@ -471,12 +503,12 @@ MimeSdpBody* SDPSession::createSDP(const char* addr, ObjList* mediaList)
 
 	if (frm.null()) {
 	    if (m->isAudio() || !m->fmtList()) {
-		Debug(m_parser,DebugMild,"No formats for '%s', excluding from SDP [%p]",
-		    m->c_str(),this);
+		Debug(m_enabler,DebugMild,"No formats for '%s', excluding from SDP [%p]",
+		    m->c_str(),m_ptr);
 		continue;
 	    }
-	    Debug(m_parser,DebugInfo,"Assuming formats '%s' for media '%s' [%p]",
-		m->fmtList(),m->c_str(),this);
+	    Debug(m_enabler,DebugInfo,"Assuming formats '%s' for media '%s' [%p]",
+		m->fmtList(),m->c_str(),m_ptr);
 	    frm << " " << m->fmtList();
 	    // brutal but effective
 	    for (char* p = const_cast<char*>(frm.c_str()); *p; p++) {
@@ -535,8 +567,11 @@ MimeSdpBody* SDPSession::createSDP()
 }
 
 // Creates a SDP from RTP address data present in message
-MimeSdpBody* SDPSession::createPasstroughSDP(NamedList& msg, bool update)
+MimeSdpBody* SDPSession::createPasstroughSDP(NamedList& msg, bool update,
+    bool allowEmptyAddr)
 {
+    XDebug(m_enabler,DebugAll,"createPasstroughSDP(%s,%u,%u) [%p]",
+	msg.c_str(),update,allowEmptyAddr,m_ptr);
     String tmp = msg.getValue("rtp_forward");
     msg.clearParam("rtp_forward");
     if (!(m_rtpForward && tmp.toBoolean()))
@@ -550,7 +585,7 @@ MimeSdpBody* SDPSession::createPasstroughSDP(NamedList& msg, bool update)
 	}
     }
     String addr;
-    ObjList* lst = updateRtpSDP(msg,addr,update ? m_rtpMedia : 0);
+    ObjList* lst = updateRtpSDP(msg,addr,update ? m_rtpMedia : 0,allowEmptyAddr);
     if (!lst)
 	return 0;
     MimeSdpBody* sdp = createSDP(addr,lst);
@@ -594,8 +629,8 @@ void SDPSession::updateFormats(const NamedList& msg, bool changeMedia)
 	    SDPMedia* rtp = static_cast<SDPMedia*>(m_rtpMedia->operator[](tmp));
 	    if (!rtp)
 		continue;
-	    Debug(m_parser,DebugNote,"Removing disabled media '%s' [%p]",
-		tmp.c_str(),this);
+	    Debug(m_enabler,DebugNote,"Removing disabled media '%s' [%p]",
+		tmp.c_str(),m_ptr);
 	    m_rtpMedia->remove(rtp,false);
 	    mediaChanged(*rtp);
 	    TelEngine::destruct(rtp);
@@ -622,12 +657,12 @@ void SDPSession::updateFormats(const NamedList& msg, bool changeMedia)
 	SDPMedia* rtp = static_cast<SDPMedia*>(m_rtpMedia->operator[](tmp));
 	if (rtp) {
 	    if (rtp->update(*p))
-		Debug(m_parser,DebugNote,"Formats for '%s' changed to '%s' [%p]",
-		    tmp.c_str(),rtp->formats().c_str(),this);
+		Debug(m_enabler,DebugNote,"Formats for '%s' changed to '%s' [%p]",
+		    tmp.c_str(),rtp->formats().c_str(),m_ptr);
 	}
 	else if (*p) {
-	    Debug(m_parser,DebugNote,"Got formats '%s' for absent media '%s' [%p]",
-		p->c_str(),tmp.c_str(),this);
+	    Debug(m_enabler,DebugNote,"Got formats '%s' for absent media '%s' [%p]",
+		p->c_str(),tmp.c_str(),m_ptr);
 	    if (trans) {
 		rtp = new SDPMedia(tmp,trans,p->c_str());
 		m_rtpMedia->append(rtp);
@@ -663,9 +698,12 @@ bool SDPSession::addSdpParams(NamedList& msg, const String& rawSdp)
 
 // Add RTP forwarding parameters to a message
 bool SDPSession::addRtpParams(NamedList& msg, const String& natAddr,
-    const MimeBody* body, bool force)
+    const MimeBody* body, bool force, bool allowEmptyAddr)
 {
-    if (!(m_rtpMedia && m_rtpAddr))
+    XDebug(m_enabler,DebugAll,"addRtpParams(%s,%s,%p,%u,%u) media=%p rtpaddr=%s [%p]",
+	msg.c_str(),natAddr.c_str(),body,force,allowEmptyAddr,m_rtpMedia,
+	m_rtpAddr.c_str(),m_ptr);
+    if (!(m_rtpMedia && (m_rtpAddr || allowEmptyAddr)))
 	return false;
     putMedia(msg,false);
     if (force || (!startRtp() && m_rtpForward)) {
@@ -719,6 +757,7 @@ Message* SDPSession::buildChanRtp(SDPMedia* media, const char* addr, bool start,
     m->addParam("direction","bidir");
     if (media->format())
 	m->addParam("format",media->format());
+    m->addParam("ipv6_support",String::boolText(m_ipv6));
     if (m_rtpLocalAddr)
 	m->addParam("localip",m_rtpLocalAddr);
     m->addParam("remoteip",addr);
@@ -754,7 +793,7 @@ Message* SDPSession::buildChanRtp(SDPMedia* media, const char* addr, bool start,
 		    m->addParam("crypto_params",sdes.matchString(4));
 	    }
 	    else
-		Debug(m_parser,DebugWarn,"Invalid SDES: '%s' [%p]",sdes.c_str(),this);
+		Debug(m_enabler,DebugWarn,"Invalid SDES: '%s' [%p]",sdes.c_str(),m_ptr);
 	}
 	else if (media->securable())
 	    m->addParam("secure",String::boolText(true));
@@ -794,11 +833,13 @@ void SDPSession::setLocalRtpChanged(bool chg)
 }
 
 // Update RTP/SDP data from parameters
-ObjList* SDPSession::updateRtpSDP(const NamedList& params, String& rtpAddr, ObjList* oldList)
+ObjList* SDPSession::updateRtpSDP(const NamedList& params, String& rtpAddr, ObjList* oldList,
+    bool allowEmptyAddr)
 {
-    DDebug(DebugAll,"SDPSession::updateRtpSDP(%s,%s,%p)",params.c_str(),rtpAddr.c_str(),oldList);
+    XDebug(DebugAll,"SDPSession::updateRtpSDP(%s,%s,%p,%u)",
+	params.c_str(),rtpAddr.c_str(),oldList,allowEmptyAddr);
     rtpAddr = params.getValue("rtp_addr");
-    if (!rtpAddr)
+    if (!(rtpAddr || allowEmptyAddr))
 	return 0;
     const char* sdpPrefix = params.getValue("osdp-prefix","osdp");
     ObjList* lst = 0;
@@ -819,7 +860,7 @@ ObjList* SDPSession::updateRtpSDP(const NamedList& params, String& rtpAddr, ObjL
 	if (!params.getBoolValue("media" + tmp,audio))
 	    continue;
 	int port = p->toInteger();
-	if (!port)
+	if (!(port || allowEmptyAddr))
 	    continue;
 	const char* fmts = params.getValue("formats" + tmp);
 	if (!fmts)
@@ -867,8 +908,36 @@ ObjList* SDPSession::updateRtpSDP(const NamedList& params, String& rtpAddr, ObjL
 // Media changed notification.
 void SDPSession::mediaChanged(const SDPMedia& media)
 {
-    XDebug(m_parser,DebugAll,"SDPSession::mediaChanged('%s' %p)%s%s [%p]",
-	media.c_str(),&media,(media.id() ? " id=" : ""),media.id().safe(),this);
+    XDebug(m_enabler,DebugAll,"SDPSession::mediaChanged('%s' %p)%s%s [%p]",
+	media.c_str(),&media,(media.id() ? " id=" : ""),media.id().safe(),m_ptr);
+}
+
+// Dispatch rtp notification
+void SDPSession::dispatchingRtp(Message*& msg, SDPMedia* media)
+{
+    XDebug(m_enabler,DebugAll,"SDPSession::dispatchingRtp(%p,%p) [%p]",msg,media,m_ptr);
+}
+
+// Set data used in debug 
+void SDPSession::setSdpDebug(DebugEnabler* enabler, void* ptr)
+{
+    m_enabler = enabler ? enabler : static_cast<DebugEnabler*>(m_parser);
+    m_ptr = ptr ? ptr : (void*)this;
+}
+
+// Print current media to output
+void SDPSession::printRtpMedia(const char* reason)
+{
+    if (!(m_rtpMedia && m_enabler->debugAt(DebugAll)))
+	return;
+    String tmp;
+    for (ObjList* o = m_rtpMedia->skipNull(); o; o = o->skipNext()) {
+    	SDPMedia* m = static_cast<SDPMedia*>(o->get());
+	if (tmp)
+	    tmp << " ";
+	tmp << m->c_str() << "=" << m->formats();
+    }
+    Debug(m_enabler,DebugAll,"%s: %s [%p]",reason,tmp.c_str(),m_ptr);
 }
 
 };   // namespace TelEngine
