@@ -5,21 +5,18 @@
  * Yet Another Signalling Stack - implements the support for SS7, ISDN and PSTN
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include "yatesig.h"
@@ -254,7 +251,7 @@ bool SS7Layer2::control(NamedList& params)
     }
     if (!(cmp && toString() == cmp))
 	return false;
-    return (cmd >= 0) && control((Operation)cmd,&params);
+    return TelEngine::controlReturn(&params,(cmd >= 0) && control((Operation)cmd,&params));
 }
 
 bool SS7Layer2::getEmergency(NamedList* params, bool emg) const
@@ -433,16 +430,16 @@ bool SS7MTP2::control(Operation oper, NamedList* params)
     switch (oper) {
 	case Pause:
 	    abortAlignment(false);
-	    return true;
+	    return TelEngine::controlReturn(params,true);
 	case Resume:
 	    if (aligned() || !m_autostart)
-		return true;
+		return TelEngine::controlReturn(params,true);
 	    // fall-through
 	case Align:
 	    startAlignment(getEmergency(params));
-	    return true;
+	    return TelEngine::controlReturn(params,true);
 	case Status:
-	    return operational();
+	    return TelEngine::controlReturn(params,operational());
 	default:
 	    return SignallingReceiver::control((SignallingInterface::Operation)oper,params);
     }
