@@ -5,21 +5,18 @@
  * Yet Another Signalling Stack - implements the support for SS7, ISDN and PSTN
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include "yatesig.h"
@@ -850,7 +847,7 @@ bool SS7MTP3::control(Operation oper, NamedList* params)
 		if (ok)
 		    SS7Layer3::notify(-1);
 	    }
-	    return true;
+	    return TelEngine::controlReturn(params,true);
 	case Restart:
 	    if (ok) {
 		ok = false;
@@ -877,12 +874,12 @@ bool SS7MTP3::control(Operation oper, NamedList* params)
 		}
 		Debug(this,DebugNote,"Emergency resume attempt on %u links [%p]",cnt,this);
 	    }
-	    return true;
+	    return TelEngine::controlReturn(params,true);
 	case Status:
 	    printRoutes();
-	    return ok;
+	    return TelEngine::controlReturn(params,ok);;
     }
-    return false;
+    return TelEngine::controlReturn(params,false);
 }
 
 bool SS7MTP3::control(NamedList& params)
