@@ -5,21 +5,18 @@
  * Yet Another Signalling Stack - implements the support for SS7, ISDN and PSTN
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include "yatesig.h"
@@ -339,16 +336,17 @@ HandledMSU SS7Management::receivedMSU(const SS7MSU& msu, const SS7Label& label,
     const unsigned char* buf = msu.getData(label.length()+1,1);
     if (!buf)
 	return false;
-    SS7MsgSNM* msg = SS7MsgSNM::parse(this,buf[0],label.type(),buf+1,len-1);
+    RefPointer<SS7MsgSNM> msg = SS7MsgSNM::parse(this,buf[0],label.type(),buf+1,len-1);
     if (!msg)
 	return false;
+    msg->deref();
 
     if (debugAt(DebugInfo)) {
 	String tmp;
 	msg->toString(tmp,label,debugAt(DebugAll));
 	const char* name = network ? network->toString().c_str() : 0;
 	Debug(this,DebugInfo,"Received %u bytes message (%p) on %s:%d%s",
-	    len,msg,name,sls,tmp.c_str());
+	    len,static_cast<void*>(msg),name,sls,tmp.c_str());
     }
 
     String addr;
@@ -747,7 +745,6 @@ HandledMSU SS7Management::receivedMSU(const SS7MSU& msu, const SS7Label& label,
 	    addr.c_str(),params.c_str(),len,tmp.c_str());
     }
 
-    TelEngine::destruct(msg);
     return true;
 }
 
@@ -841,13 +838,13 @@ bool SS7Management::control(NamedList& params)
 			    unsigned char data[5];
 			    int len = SS7PointCode::length(t)+1;
 			    data[0] = cmd;
-			    return dest.store(t,data+1,spare) &&
+			    return TelEngine::controlReturn(&params,dest.store(t,data+1,spare) &&
 				((cmd == SS7MsgSNM::TFP) ?
 				    postpone(new SS7MSU(txSio,lbl,data,len),lbl,txSls,1000) :
-				    (transmitMSU(SS7MSU(txSio,lbl,data,len),lbl,txSls) >= 0));
+				    (transmitMSU(SS7MSU(txSio,lbl,data,len),lbl,txSls) >= 0)));
 			}
 		    }
-		    return false;
+		    return TelEngine::controlReturn(&params,false);
 		// Messages sent with just the code
 		case SS7MsgSNM::ECO:
 		case SS7MsgSNM::TRA:
@@ -862,19 +859,19 @@ bool SS7Management::control(NamedList& params)
 		case SS7MsgSNM::CNP:
 		    {
 			unsigned char data = cmd;
-			return transmitMSU(SS7MSU(txSio,lbl,&data,1),lbl,txSls) >= 0;
+			return TelEngine::controlReturn(&params,transmitMSU(SS7MSU(txSio,lbl,&data,1),lbl,txSls) >= 0);
 		    }
 		// Messages postponed with just the code
 		case SS7MsgSNM::LIN:
 		    {
 			unsigned char data = cmd;
-			return postpone(new SS7MSU(txSio,lbl,&data,1),lbl,txSls,2500,5000);
+			return TelEngine::controlReturn(&params,postpone(new SS7MSU(txSio,lbl,&data,1),lbl,txSls,2500,5000));
 		    }
 		case SS7MsgSNM::LUN:
 		case SS7MsgSNM::LFU:
 		    {
 			unsigned char data = cmd;
-			return postpone(new SS7MSU(txSio,lbl,&data,1),lbl,txSls,1200,2400);
+			return TelEngine::controlReturn(&params,postpone(new SS7MSU(txSio,lbl,&data,1),lbl,txSls,1200,2400));
 		    }
 		// Changeover messages
 		case SS7MsgSNM::COO:
@@ -883,7 +880,7 @@ bool SS7Management::control(NamedList& params)
 		case SS7MsgSNM::XCA:
 		    if (params.getBoolValue(YSTRING("emergency"),false)) {
 			unsigned char data = (SS7MsgSNM::COO == cmd) ? SS7MsgSNM::ECO : SS7MsgSNM::ECA;
-			return transmitMSU(SS7MSU(txSio,lbl,&data,1),lbl,txSls) >= 0;
+			return TelEngine::controlReturn(&params,transmitMSU(SS7MSU(txSio,lbl,&data,1),lbl,txSls) >= 0);
 		    }
 		    else {
 			int seq = params.getIntValue(YSTRING("sequence"),0) & 0x00ffffff;
@@ -913,11 +910,11 @@ bool SS7Management::control(NamedList& params)
 				break;
 			    default:
 				Debug(DebugStub,"Please implement COO for type %u",t);
-				return false;
+				return TelEngine::controlReturn(&params,false);
 			}
-			return (cmd == SS7MsgSNM::COA)
+			return TelEngine::controlReturn(&params,(cmd == SS7MsgSNM::COA)
 			    ? transmitMSU(SS7MSU(txSio,lbl,&data,len),lbl,txSls) >= 0
-			    : postpone(new SS7MSU(txSio,lbl,&data,len),lbl,txSls,1800,0,true);
+			    : postpone(new SS7MSU(txSio,lbl,&data,len),lbl,txSls,1800,0,true));
 		    }
 		// Changeback messages
 		case SS7MsgSNM::CBD:
@@ -938,11 +935,11 @@ bool SS7Management::control(NamedList& params)
 				break;
 			    default:
 				Debug(DebugStub,"Please implement CBD for type %u",t);
-				return false;
+				return TelEngine::controlReturn(&params,false);
 			}
-			return (cmd == SS7MsgSNM::CBA)
+			return TelEngine::controlReturn(&params,(cmd == SS7MsgSNM::CBA)
 			    ? transmitMSU(SS7MSU(txSio,lbl,&data,len),lbl,txSls) >= 0
-			    : postpone(new SS7MSU(txSio,lbl,&data,len),lbl,txSls,1000,2000,true);
+			    : postpone(new SS7MSU(txSio,lbl,&data,len),lbl,txSls,1000,2000,true));
 		    }
 		default:
 		    if (cmd >= 0)
@@ -952,7 +949,7 @@ bool SS7Management::control(NamedList& params)
 	}
     }
     TelEngine::destruct(l);
-    return false;
+    return TelEngine::controlReturn(&params,false);
 }
 
 void SS7Management::notify(SS7Layer3* network, int sls)
