@@ -5,21 +5,18 @@
  * Yet Another Signalling Stack - implements the support for SS7, ISDN and PSTN
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include "yatesig.h"
@@ -183,6 +180,9 @@ void SIGTRAN::attach(SIGTransport* trans)
     }
     if (trans) {
 	trans->attach(this);
+	SignallingEngine* engine = SignallingEngine::self();
+	if (engine)
+	    engine->insert(trans);
 	trans->deref();
     }
 }
@@ -325,11 +325,13 @@ SIGAdaptation::SIGAdaptation(const char* name, const NamedList* params,
     DDebug(this,DebugAll,"Creating SIGTRAN UA [%p]",this);
     for (int i = 0; i < 32;i++)
 	m_streamsHB[i] = HeartbeatDisabled;
-    m_waitHeartbeatAck.interval(*params,"wait_hb_ack",500,2000,false);
-    m_sendHeartbeat.interval(*params,"send_hb",15000,30000,true);
-    // The maximum interval in miliseconds allowed for SCTP to retransmit
-    // a lost package
-    m_maxRetransmit = params->getIntValue("max_interval_retrans",1000);
+    if (params) {
+	m_waitHeartbeatAck.interval(*params,"wait_hb_ack",500,2000,false);
+	m_sendHeartbeat.interval(*params,"send_hb",15000,30000,true);
+	// The maximum interval in miliseconds allowed for SCTP to retransmit
+	// a lost package
+	m_maxRetransmit = params->getIntValue("max_interval_retrans",1000);
+    }
 }
 
 SIGAdaptation::~SIGAdaptation()
@@ -763,6 +765,7 @@ bool SIGAdaptClient::activate()
 		DataBlock data;
 		if (m_aspId != -1)
 		    addTag(data,0x0011,m_aspId);
+		mylock.drop();
 		transmitMSG(ASPSM,AspsmUP,data);
 		return true;
 	    }
@@ -772,6 +775,7 @@ bool SIGAdaptClient::activate()
 		DataBlock data;
 		if (m_traffic != TrafficUnused)
 		    addTag(data,0x000b,m_traffic);
+		mylock.drop();
 		return transmitMSG(ASPTM,AsptmACTIVE,data,1);
 	    }
 	default:
@@ -1414,7 +1418,7 @@ bool SS7M2PA::control(NamedList& params)
     }
     if (!(cmp && toString() == cmp))
 	return false;
-    return (cmd >= 0) && control((M2PAOperations)cmd,&params);
+    return TelEngine::controlReturn(&params,(cmd >= 0) && control((M2PAOperations)cmd,&params));
 }
 
 bool SS7M2PA::control(M2PAOperations oper, NamedList* params)
@@ -1431,22 +1435,22 @@ bool SS7M2PA::control(M2PAOperations oper, NamedList* params)
 	    m_state = OutOfService;
 	    abortAlignment("Control request pause.");
 	    transmitLS();
-	    return true;
+	    return TelEngine::controlReturn(params,true);
 	case Resume:
 	    if (aligned() || !m_autostart)
-		return true;
+		return TelEngine::controlReturn(params,true);
 	case Align:
 	{
 	    m_state = getEmergency(params) ? ProvingEmergency : ProvingNormal;
 	    abortAlignment("Control request align.");
-	    return true;
+	    return TelEngine::controlReturn(params,true);
 	}
 	case Status:
-	    return operational();
+	    return TelEngine::controlReturn(params,operational());
 	case TransRestart:
-	    return restart(true);
+	    return TelEngine::controlReturn(params,restart(true));
 	default:
-	    return false;
+	    return TelEngine::controlReturn(params,false);
     }
 }
 
@@ -1947,22 +1951,22 @@ bool SS7M2UA::control(Operation oper, NamedList* params)
 		    SIGAdaptation::addTag(buf,0x0001,(u_int32_t)m_iid);
 		// Release Request
 		if (!adaptation()->transmitMSG(SIGTRAN::MAUP,4,buf,getStreamId()))
-		    return false;
+		    return TelEngine::controlReturn(params,false);
 		getSequence();
 	    }
 	    m_linkState = LinkDown;
 	    if (!m_retrieve.started())
 		SS7Layer2::notify();
-	    return true;
+	    return TelEngine::controlReturn(params,true);
 	case Resume:
 	    if (operational())
-		return true;
+		return TelEngine::controlReturn(params,true);
 	    if (!m_autostart)
-		return activate();
+		return TelEngine::controlReturn(params,activate());
 	    if (m_retrieve.started()) {
 		if (LinkDown == m_linkState)
 		    m_linkState = getEmergency(params,false) ? LinkReqEmg : LinkReq;
-		return activate();
+		return TelEngine::controlReturn(params,activate());
 	    }
 	    // fall through
 	case Align:
@@ -1980,18 +1984,19 @@ bool SS7M2UA::control(Operation oper, NamedList* params)
 		SIGAdaptation::addTag(buf,0x0302,(emg ? 2 : 3));
 		// State Request
 		if (!adaptation()->transmitMSG(SIGTRAN::MAUP,7,buf,getStreamId()))
-		    return false;
+		    return TelEngine::controlReturn(params,false);
 		buf.clear();
 		if (m_iid >= 0)
 		    SIGAdaptation::addTag(buf,0x0001,(u_int32_t)m_iid);
 		// Establish Request
-		return adaptation()->transmitMSG(SIGTRAN::MAUP,2,buf,getStreamId());
+		return TelEngine::controlReturn(params,
+			adaptation()->transmitMSG(SIGTRAN::MAUP,2,buf,getStreamId()));
 	    }
-	    return activate();
+	    return TelEngine::controlReturn(params,activate());
 	case Status:
-	    return operational();
+	    return TelEngine::controlReturn(params,operational());
 	default:
-	    return false;
+	    return TelEngine::controlReturn(params,false);
     }
 }
 
