@@ -4,21 +4,18 @@
  * This file is part of the YATE Project http://YATE.null.ro 
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include <yatesip.h>
@@ -68,6 +65,7 @@ static TokenDict sip_responses[] = {
     { "Extension Required", 421 },
     { "Session Timer Too Small", 422 },
     { "Interval Too Brief", 423 },
+    { "Bad Location Information", 424 },
     { "Use Identity Header", 428 },
     { "Provide Referrer Identity", 429 },
     { "Flow Failed", 430 },                                // RFC5626
@@ -76,6 +74,8 @@ static TokenDict sip_responses[] = {
     { "Unsupported Certificate", 437 },
     { "Invalid Identity Header", 438 },
     { "First Hop Lacks Outbound Support", 439 },           // RFC5626
+    { "Max-Breadth Exceeded", 440 },
+    { "Bad Info Package", 469 },
     { "Consent Needed", 470 },
     { "Temporarily Unavailable", 480 },
     { "Call/Transaction Does Not Exist", 481 },
@@ -135,7 +135,7 @@ void SIPParty::setAddr(const String& addr, int port, bool local)
     a = addr;
     p = port;
     DDebug(DebugAll,"SIPParty updated %s address '%s:%d' [%p]",
-	local ? "local" : "remote",a.c_str(),p,this);
+	local ? "local" : "remote",SocketAddr::appendTo(a,p).c_str(),this);
 }
 
 void SIPParty::getAddr(String& addr, int& port, bool local)
@@ -328,8 +328,9 @@ void SIPEngine::processEvent(SIPEvent *event)
 		    if (!event->getMessage()->isAnswer())
 			break;
 		default:
-		    if (event->getParty())
-			event->getParty()->transmit(event);
+		    if (event->getParty() && !event->getParty()->transmit(event) &&
+			event->getTransaction())
+			event->getTransaction()->msgTransmitFailed(event->getMessage());
 	    }
 	}
 	if (event->isIncoming()) {
@@ -519,7 +520,7 @@ int SIPEngine::authUser(const SIPMessage* message, String& user, bool proxy, Gen
 	const GenObject* o = l->get();
 	if (!o)
 	    continue;
-	const MimeHeaderLine* t = static_cast<const MimeHeaderLine*>(o->getObject("MimeHeaderLine"));
+	const MimeHeaderLine* t = static_cast<const MimeHeaderLine*>(o->getObject(YATOM("MimeHeaderLine")));
 	if (!t || (t->name() |= hdr))
 	    continue;
 	// remember this line for foreign authentication
