@@ -4,21 +4,18 @@
  * This file is part of the YATE Project http://YATE.null.ro 
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include <yatesip.h>
@@ -139,8 +136,8 @@ SIPMessage::SIPMessage(const SIPMessage* original, const SIPMessage* answer)
 	String tmp;
 	tmp << version << "/" << getParty()->getProtoName();
 	if (getParty()) {
-	    Lock lock(getParty()->mutex());
-	    tmp << " " << getParty()->getLocalAddr() << ":" << getParty()->getLocalPort();
+	    tmp << " ";
+	    getParty()->appendAddr(tmp,true);
 	}
 	hl = new MimeHeaderLine("Via",tmp);
 	header.append(hl);
@@ -232,7 +229,8 @@ void SIPMessage::complete(SIPEngine* engine, const char* user, const char* domai
     if (!hl) {
 	String tmp;
 	tmp << version << "/" << getParty()->getProtoName();
-	tmp << " " << partyLAddr << ":" << partyLPort;
+	tmp << " ";
+	SocketAddr::appendTo(tmp,partyLAddr,partyLPort);
 	hl = new MimeHeaderLine("Via",tmp);
 	if (isReliable() && 0 == (flags & NoConnReuse))
 	    hl->setParam("alias");
@@ -263,7 +261,7 @@ void SIPMessage::complete(SIPEngine* engine, const char* user, const char* domai
 	    String tmp = "<sip:";
 	    if (user)
 		tmp << String::uriEscape(user,'@',"+?&") << "@";
-	    tmp << domain << ">";
+	    SocketAddr::appendAddr(tmp,domain) << ">";
 	    hl = new MimeHeaderLine("From",tmp);
 	    header.append(hl);
 	}
@@ -319,8 +317,7 @@ void SIPMessage::complete(SIPEngine* engine, const char* user, const char* domai
 	if (tmp)
 	    tmp = tmp.uriEscape('@',"+?&") + "@";
 	tmp = "<sip:" + tmp;
-	tmp << partyLAddr << ":";
-	tmp << partyLPort << ">";
+	SocketAddr::appendTo(tmp,partyLAddr,partyLPort) << ">";
 	addHeader("Contact",tmp);
     }
 
