@@ -4,21 +4,18 @@
  * This file is part of the YATE Project http://YATE.null.ro 
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include <yatesip.h>
@@ -507,10 +504,14 @@ SIPTransaction::Processed SIPTransaction::processMessage(SIPMessage* message, co
 	const NamedString* ns = message->getParam("To","tag");
 	if (m_tag.null()) {
 	    if (ns) {
-		// establish the dialog
-		m_tag = *ns;
-		DDebug(getEngine(),DebugInfo,"SIPTransaction found dialog tag '%s' [%p]",
-		    m_tag.c_str(),this);
+		if (message->code > 100) {
+		    // establish the dialog
+		    m_tag = *ns;
+		    DDebug(getEngine(),DebugInfo,"SIPTransaction found dialog tag '%s' [%p]",
+			m_tag.c_str(),this);
+		}
+		else
+		    Debug(getEngine(),DebugMild,"Received To tag in 100 answer! (sender bug)");
 	    }
 	}
 	else if (!ns) {
@@ -688,6 +689,49 @@ SIPEvent* SIPTransaction::getServerEvent(int state, int timeout)
     return e;
 }
 
+// Event transmission failed notification
+void SIPTransaction::msgTransmitFailed(SIPMessage* msg)
+{
+    if (!msg)
+	return;
+    Lock lock(getEngine());
+    DDebug(getEngine(),DebugNote,
+	"SIPTransaction send failed state=%s msg=%p first=%p last=%p [%p]",
+	stateName(m_state),msg,m_firstMessage,m_lastMessage,this);
+    // Do nothing in termination states
+    if (m_state == Invalid || m_state == Finish || m_state == Cleared)
+	return;
+    if (isOutgoing()) {
+	if (m_state == Trying) {
+	    if (msg != m_firstMessage)
+		return;
+	    // Reliable transport: terminate now
+	    // Non reliable: terminate if this is the first attempt
+	    if ((msg->getParty() && msg->getParty()->isReliable()) ||
+		m_engine->getReqTransCount() == m_timeouts) {
+		Debug(getEngine(),DebugInfo,
+		    "SIPTransaction send failed state=%s: clearing [%p]",
+		    stateName(m_state),this);
+		m_response = 500;
+		changeState(Cleared);
+		return;
+	    }
+	}
+	else if (m_state == Initial || msg != m_lastMessage)
+	    return;
+    }
+    else {
+	// Incoming
+	if (msg != m_lastMessage)
+	    return;
+    }
+    // Avoid party retry
+    Debug(getEngine(),DebugAll,
+	"SIPTransaction send failed state=%s resetting msg %p party [%p]",
+	stateName(m_state),msg,this);
+    msg->setParty();
+}
+
 bool SIPTransaction::tryAutoAuth(SIPMessage* answer)
 {
     if ((answer->code != 401) && (answer->code != 407))
