@@ -3,21 +3,18 @@
  * This file is part of the YATE Project http://YATE.null.ro 
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include <yatexml.h>
@@ -925,35 +922,6 @@ inline unsigned char getDec(String& dec)
     return 0;
 }
 
-// Obtain a hexa representation of the given hexa char
-inline signed char hexDecode(char c)
-{
-    if (('0' <= c) && (c <= '9'))
-	return c - '0';
-    if (('A' <= c) && (c <= 'F'))
-	return c - 'A' + 10;
-    if (('a' <= c) && (c <= 'f'))
-	return c - 'a' + 10;
-    return -1;
-}
-
-// Obtain a char from the given hexa number
-inline unsigned char getHex(String& hex)
-{
-    if (hex.length() > 6) {
-	DDebug(DebugNote,"Hex number '%s' too long",hex.c_str());
-	return 0;
-    }
-    signed char c1 = hexDecode(hex.at(3));
-    signed char c2 = hexDecode(hex.at(4));
-    if (c1 == -1 || c2 == -1) {
-	DDebug(DebugNote,"Invalid hex number '%s'",hex.c_str());
-	return 0;
-    }
-    unsigned char c = (c1 << 4) | c2;
-    return c;
-}
-
 // Unescape the given text
 void XmlSaxParser::unEscape(String& text)
 {
@@ -985,9 +953,19 @@ void XmlSaxParser::unEscape(String& text)
 	    String aux(str + found,len - found);
 	    char re = 0;
 	    if (aux.startsWith("&#")) {
-		if (aux.at(2) == 'x')
-		    re = getHex(aux);
-		else
+		if (aux.at(2) == 'x') {
+		    if (aux.length() > 4 && aux.length() <= 12) {
+			int esc = aux.substr(3,aux.length() - 4).toInteger(-1,16);
+			if (esc != -1) {
+			    UChar uc(esc);
+			    buf.append(str,found) << uc.c_str();
+			    str += len;
+			    len = 0;
+			    found = -1;
+			    continue;
+			}
+		    }
+		} else
 		    re = getDec(aux);
 	    }
 	    if (re == '&') {
@@ -1533,10 +1511,8 @@ XmlSaxParser::Error XmlDocument::addChild(XmlChild* child)
 	// Text outside root: ignore empty, raise error otherwise
 	XmlText* text = child->xmlText();
 	if (text) {
-	    String tmp(text->getText());
-	    tmp.trimSpaces();
-	    if (!tmp) {
-		TelEngine::destruct(child);
+	    if (text->onlySpaces()) {
+		m_beforeRoot.addChild(text);
 		return XmlSaxParser::NoError;
 	    }
 	    Debug(DebugNote,"XmlDocument. Got text outside element [%p]",this);
@@ -1551,6 +1527,8 @@ XmlSaxParser::Error XmlDocument::addChild(XmlChild* child)
 	DDebug(DebugStub,"XmlDocument. Request to add xml element child to incomplete root [%p]",this);
 	return XmlSaxParser::NotWellFormed;
     }
+    if ((child->xmlText() && child->xmlText()->onlySpaces()) || child->xmlComment())
+	return m_afterRoot.addChild(child);
     // TODO: check what xml we can add after the root or if we can add
     //  anything after an incomplete root
     Debug(DebugStub,"XmlDocument. Request to add non element while having a root [%p]",this);
@@ -1560,7 +1538,7 @@ XmlSaxParser::Error XmlDocument::addChild(XmlChild* child)
 // Retrieve the document declaration
 XmlDeclaration* XmlDocument::declaration() const
 {
-    for (ObjList* o = m_beforeRoot.getChildren().skipNull(); o; o = o->skipNull()) {
+    for (ObjList* o = m_beforeRoot.getChildren().skipNull(); o; o = o->skipNext()) {
 	XmlDeclaration* d = (static_cast<XmlChild*>(o->get()))->xmlDeclaration();
 	if (d)
 	    return d;
@@ -1581,6 +1559,7 @@ void XmlDocument::toString(String& dump, bool escape, const String& indent, cons
 	dump << origIndent;
 	m_root->toString(dump,escape,indent,origIndent);
     }
+    m_afterRoot.toString(dump,escape,indent,origIndent);
 }
 
 // Reset this XmlDocument. Destroys root and clear the others xml objects
@@ -1588,6 +1567,7 @@ void XmlDocument::reset()
 {
     TelEngine::destruct(m_root);
     m_beforeRoot.clearChildren();
+    m_afterRoot.clearChildren();
     m_file.clear();
 }
 
@@ -1596,16 +1576,23 @@ XmlSaxParser::Error XmlDocument::read(Stream& in, int* error)
 {
     XmlDomParser parser(static_cast<XmlParent*>(this),false);
     char buf[8096];
+    bool start = true;
     while (true) {
 	int rd = in.readData(buf,sizeof(buf) - 1);
 	if (rd > 0) {
 	    buf[rd] = 0;
-	    if (parser.parse(buf) || parser.error() == XmlSaxParser::Incomplete)
+	    const char* text = buf;
+	    if (start) {
+		String::stripBOM(text);
+		start = false;
+	    }
+	    if (parser.parse(text) || parser.error() == XmlSaxParser::Incomplete)
 		continue;
 	    break;
 	}
 	break;
     }
+    parser.completeText();
     if (parser.error() != XmlSaxParser::NoError) {
 	DDebug(DebugNote,"XmlDocument error loading stream. Parser error %d '%s' [%p]",
 	    parser.error(),parser.getError(),this);
@@ -1633,6 +1620,7 @@ int XmlDocument::write(Stream& out, bool escape, const String& indent,
     m_beforeRoot.toString(dump,escape,indent,origIndent);
     if (m_root)
 	m_root->toString(dump,escape,indent,origIndent,completeOnly);
+    m_afterRoot.toString(dump,escape,indent,origIndent);
     return out.writeData(dump);
 }
 
@@ -1756,6 +1744,18 @@ XmlElement::~XmlElement()
 	m_element.c_str(),this);
 }
 
+// Set element's unprefixed tag, don't change namespace prefix
+void XmlElement::setUnprefixedTag(const String& s)
+{
+    if (!s || s == unprefixedTag())
+	return;
+    if (TelEngine::null(m_prefixed))
+	m_element.assign(s);
+    else
+	m_element.assign(*m_prefixed + ":" + s);
+    setPrefixed();
+}
+
 // Set inherited namespaces from a given element. Reset them anyway
 void XmlElement::setInheritedNs(const XmlElement* xml, bool inherit)
 {
@@ -1805,14 +1805,41 @@ void XmlElement::addInheritedNs(const NamedList& list)
 }
 
 // Obtain the first text of this xml element
-const String& XmlElement::getText()
+const String& XmlElement::getText() const
 {
-    XmlText* txt = 0;
+    const XmlText* txt = 0;
     for (ObjList* ob = getChildren().skipNull(); ob && !txt; ob = ob->skipNext())
 	txt = (static_cast<XmlChild*>(ob->get()))->xmlText();
     return txt ? txt->getText() : String::empty();
 }
 
+XmlChild* XmlElement::getFirstChild()
+{
+    if (!m_children.getChildren().skipNull())
+	return 0;
+    return static_cast<XmlChild*>(m_children.getChildren().skipNull()->get());
+}
+
+XmlText* XmlElement::setText(const char* text)
+{
+    XmlText* txt = 0;
+    for (ObjList* o = getChildren().skipNull(); o; o = o->skipNext()) {
+	txt = (static_cast<XmlChild*>(o->get()))->xmlText();
+	if (txt)
+	    break;
+    }
+    if (txt) {
+	if (!text)
+	    return static_cast<XmlText*>(removeChild(txt));
+	txt->setText(text);
+    }
+    else if (text) {
+	txt = new XmlText(text);
+	addChild(txt);
+    }
+    return txt;
+}
+
 // Add a text child
 void XmlElement::addText(const char* text)
 {
@@ -1922,6 +1949,14 @@ unsigned int XmlElement::copyAttributes(NamedList& list, const String& prefix) c
     return copy;
 }
 
+void XmlElement::setAttributes(NamedList& list, const String& prefix, bool skipPrefix)
+{
+    if (prefix)
+	m_element.copySubParams(list,prefix,skipPrefix);
+    else
+	m_element.copyParams(list);
+}
+
 // Retrieve a namespace attribute. Search in parent or inherited for it
 String* XmlElement::xmlnsAttribute(const String& name) const
 {
@@ -2169,6 +2204,19 @@ void XmlText::toString(String& dump, bool esc, const String& indent,
 	dump << m_text;
 }
 
+bool XmlText::onlySpaces()
+{
+    if (!m_text)
+	return true;
+    const char *s = m_text;
+    unsigned int i = 0;
+    for (;i < m_text.length();i++) {
+	if (s[i] == ' ' || s[i] == '\t' || s[i] == '\v' || s[i] == '\f' || s[i] == '\r' || s[i] == '\n')
+	    continue;
+	return false;
+    }
+    return true;
+}
 
 /*
  * XmlDoctype
