@@ -5,21 +5,18 @@
  * XML Parser and support classes
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #ifndef __YATEXML_H
@@ -1085,6 +1082,7 @@ private:
     XmlElement* m_root;                  // The root element
     XmlFragment m_beforeRoot;            // XML children before root (declaration ...)
     String m_file;                       // The file name used on load
+    XmlFragment m_afterRoot;             // XML children after root (comments, empty text)
 };
 
 
@@ -1152,6 +1150,12 @@ public:
     inline const String& unprefixedTag() const
 	{ return m_prefixed ? m_prefixed->name() : static_cast<const String&>(m_element); }
 
+    /**
+     * Set element's unprefixed tag, don't change namespace prefix
+     * @param s New element's tag
+     */
+    void setUnprefixedTag(const String& s);
+
     /**
      * Retrieve the element's tag (without prefix)
      * @return Element tag
@@ -1312,7 +1316,7 @@ public:
 	const String* auth = 0) const;
 
     /**
-     * Find the first child of this XmlElement
+     * Find the first XmlElement child of this XmlElement
      * @param name Optional name of the child
      * @param ns Optional child namespace
      * @param noPrefix True to compare the tag without namespace prefix, false to
@@ -1325,7 +1329,7 @@ public:
 	{ return XmlFragment::findElement(getChildren().skipNull(),name,ns,noPrefix); }
 
     /**
-     * Finds next child of this XmlElement
+     * Finds next XmlElement child of this XmlElement
      * @param prev Previous child
      * @param name Optional name of the child
      * @param ns Optional child namespace
@@ -1357,10 +1361,25 @@ public:
 	    return c ? &(c->getText()) : 0;
 	}
 
+    /**
+     * Get first XmlChild of this XmlElement
+     * @return The first XmlChild found.
+     */
+     XmlChild* getFirstChild();
+
     /**
      * @return The first XmlText found in this XmlElement children
      */
-    const String& getText();
+    const String& getText() const;
+
+    /**
+     * Set text for first XmlText element found in this XmlElement's children
+     * If child text element does not exist, create it and append it to the element's children.
+     * @param text Text to set to the XmlElement. If null, the first found XmlText element
+     *  will be deleted.
+     * @return The set XmlText if text was set, null if an XmlText was deleted
+     */
+    XmlText* setText(const char* text);
 
     /**
      * Add a text child
@@ -1383,6 +1402,15 @@ public:
      */
     unsigned int copyAttributes(NamedList& list, const String& prefix) const;
 
+    /**
+     * Set element attributes from a list of parameters
+     * @param list List of attributes
+     * @param prefix Add only the attributes that start with this prefix. =
+     *  If NULL, it will set as attributes the whole parameter list
+     * @param skipPrefix Skip over the prefix when building attribute name
+     */
+    void setAttributes(NamedList& list, const String& prefix, bool skipPrefix = true);
+
     /**
      * Add or replace an attribute
      * @param name Attribute name
@@ -1687,6 +1715,13 @@ public:
     inline const String& getText() const
 	{ return m_text; }
 
+    /**
+     * Set the text
+     * @param text Text to set in this XmlText
+     */
+    inline void setText(const char* text)
+	{ m_text = text; }
+
     /**
      * Build a String from this XmlText
      * @param dump The string where to append representation
@@ -1707,6 +1742,11 @@ public:
     virtual XmlText* xmlText()
 	{ return this; }
 
+    /**
+     * Helper method to check if the text held by this XmlText contains only spaces
+     * @return False if the text contains non space characters.
+     */
+    bool onlySpaces();
 private:
     String m_text;                        // The text
 };
