@@ -5,21 +5,18 @@
  * Call Forker
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include <yatephone.h>
@@ -524,7 +521,7 @@ bool ForkMaster::msgProgress(Message& msg, const String& dest)
     if (m_fake || !dataEp) {
 	const CallEndpoint* call = slave->getPeer();
 	if (!call)
-	    call = static_cast<const CallEndpoint*>(msg.userObject("CallEndpoint"));
+	    call = static_cast<const CallEndpoint*>(msg.userObject(YATOM("CallEndpoint")));
 	if (call) {
 	    dataEp = call->getEndpoint();
 	    if (dataEp) {
@@ -694,7 +691,7 @@ void ForkModule::initialize()
 		    case Text:
 		    case Update:
 		    case Control:
-		    case ImExecute:
+		    case MsgExecute:
 			break;
 		    default:
 			Debug(this,DebugWarn,"Refusing to fork message '%s'",item->name().c_str());
