@@ -5,21 +5,18 @@
  * Cdr builder
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include <yatengine.h>
@@ -199,6 +196,8 @@ static int s_seq = 0;
 static String s_runId;
 static bool s_cdrUpdates = true;
 static bool s_cdrStatus = false;
+static bool s_statusAnswer = true;
+static bool s_ringOnProgress = false;
 static unsigned int s_statusUpdate = 60000;
 static StatusThread* s_updaterThread = 0;
 
@@ -420,7 +419,7 @@ String CdrBuilder::getStatus() const
 	"|" << getValue(YSTRING("billid"));
     unsigned int sec = 0;
     if (m_start)
-	sec = (Time::now() - m_start + 500000) / 1000000;
+	sec = (int)((Time::now() - m_start + 500000) / 1000000);
     s << "|" << sec;
     return s;
 }
@@ -534,7 +533,9 @@ bool CdrHandler::received(Message &msg)
 	    s_updaterThread->exit();
 	return false;
     }
-    if ((m_type == CdrProgress) && !msg.getBoolValue(YSTRING("earlymedia"),false))
+    if ((m_type == CdrProgress) &&
+	!(msg.getBoolValue(YSTRING("earlymedia"),false) ||
+	msg.getBoolValue(YSTRING("ringing"),s_ringOnProgress)))
 	return false;
     bool track = true;
     if (m_type == CdrUpdate) {
@@ -591,8 +592,8 @@ bool CdrHandler::received(Message &msg)
     }
     if (b) {
 	rval = b->update(msg,type,msg.msgTime().usec());
-	if (type == CdrAnswer)
-	    b->m_statusTime = Time::msecNow() + s_statusUpdate;
+	if (type == CdrAnswer && !b->m_statusTime)
+	    b->m_statusTime = Time::msecNow() + (s_statusAnswer ? 0 : s_statusUpdate);
     } else
 	Debug("cdrbuild",level,"Got message '%s' for untracked id '%s'",
 	    msg.c_str(),id.c_str());
@@ -603,6 +604,8 @@ bool CdrHandler::received(Message &msg)
 	if (id && (b = CdrBuilder::find(id))) {
 	    b->update(type,msg.msgTime().usec(),msg.getValue("status"));
 	    b->emit();
+	    if (type == CdrAnswer && !b->m_statusTime)
+		b->m_statusTime = Time::msecNow() + (s_statusAnswer ? 0 : s_statusUpdate);
 	}
     }
     return rval;
@@ -660,9 +663,9 @@ void StatusThread::run()
 	CdrBuilder* cdr = static_cast<CdrBuilder*>(o->get());
 	if (cdr->getStatus() != YSTRING("answered"))
 	    continue;
-	if (cdr->m_statusTime < now.msec()) {
+	if (cdr->m_statusTime && (cdr->m_statusTime < now.msec())) {
 	    cdr->emit("status");
-	    cdr->m_statusTime = now.msec() + s_statusUpdate;
+	    cdr->m_statusTime = s_statusUpdate ? (now.msec() + s_statusUpdate) : (u_int64_t)-1;
 	}
     }
     s_mutex.unlock();
@@ -674,9 +677,9 @@ void StatusThread::run()
 	Time t;
 	for (ObjList* o = s_cdrs.skipNull();o;o = o->skipNext()) {
 	    CdrBuilder* cdr = static_cast<CdrBuilder*>(o->get());
-	    if (cdr->m_statusTime && cdr->m_statusTime < t.msec()) {
+	    if (cdr->m_statusTime && (cdr->m_statusTime < t.msec())) {
 		cdr->emit("status");
-		cdr->m_statusTime = t.msec() + s_statusUpdate;
+		cdr->m_statusTime = s_statusUpdate ? (t.msec() + s_statusUpdate) : (u_int64_t)-1;
 	    }
 	}
     }
@@ -793,7 +796,7 @@ void CustomTimer::getRelativeTime(String& ret, u_int64_t time)
     String tmp = ret;
     int index = tmp.find(YSTRING("HH"));
     if (index >= 0) {
-	int h = timeLeft / 3600;
+	int h = (int)(timeLeft / 3600);
 	timeLeft = timeLeft % 3600;
 	String aux = "";
 	if (h <= 9)
@@ -803,7 +806,7 @@ void CustomTimer::getRelativeTime(String& ret, u_int64_t time)
 
     index = tmp.find(YSTRING("mm"));
     if (index >= 0) {
-	int m = timeLeft / 60;
+	int m = (int)(timeLeft / 60);
 	timeLeft = timeLeft % 60;
 	String aux = "";
 	if (m <= 9)
@@ -873,13 +876,17 @@ void CdrBuildPlugin::initialize()
     }
     s_cdrUpdates = cfg.getBoolValue("general","updates",true);
     s_cdrStatus = cfg.getBoolValue("general","status",false);
+    s_statusAnswer = cfg.getBoolValue("general","status_answer",true);
     int sUpdate = cfg.getIntValue("general","status_interval",60);
-    if (sUpdate < 60)
+    if (sUpdate <= 0)
+	s_statusUpdate = 0;
+    else if (sUpdate < 60)
 	s_statusUpdate = 60000;
     else if (sUpdate > 600)
 	s_statusUpdate = 600000;
     else
 	s_statusUpdate = sUpdate * 1000;
+    s_ringOnProgress = cfg.getBoolValue("general","ring_on_progress",false);;
 
     if (s_cdrStatus && !s_updaterThread) {
 	s_updaterThread = new StatusThread();
