@@ -5,21 +5,18 @@
  * CoreAudio sound channel driver for Mac OS X.
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2010 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 
@@ -614,12 +611,12 @@ bool CoreAudioSource::control(NamedList& params)
 {
     DDebug(DebugAll,"CoreAudioSource::control() [%p]",this);
     if (!m_volSettable)
-	return false;
+	return TelEngine::controlReturn(&params,false);
     int vol = params.getIntValue("in_volume",-1);
     if (vol == -1) {
 	Debug(DebugAll,"CoreAudioSource::control() [%p] - invalid value to set for volume",this);
-	return false;
-    }   
+	return TelEngine::controlReturn(&params,false);
+    }
     Float32 volValue = vol / 100.0;
 
     bool setVolStatus = false;
@@ -650,8 +647,8 @@ bool CoreAudioSource::control(NamedList& params)
         Debug(DebugAll,"CoreAudioSource::control() [%p] - set volume failed on all channels",this);
 
     if(params.getParam("out_volume"))
-	return false;
-    return setVolStatus;
+	return TelEngine::controlReturn(&params,false);
+    return TelEngine::controlReturn(&params,setVolStatus);
 }
 	
 	
@@ -836,11 +833,11 @@ bool CoreAudioConsumer::control(NamedList& params)
 {
     DDebug(DebugAll,"CoreAudioConsumer::control() [%p]",this);
     if (!m_volSettable)
-	return false;
+	return TelEngine::controlReturn(&params,false);
     int vol = params.getIntValue("out_volume",-1);
     if (vol == -1) {
-    	Debug(DebugAll,"CoreAudioConsumer::control() [%p] invalid value to set for volume",this);
-    	return false;
+	Debug(DebugAll,"CoreAudioConsumer::control() [%p] invalid value to set for volume",this);
+	return TelEngine::controlReturn(&params,false);
     }
     Float32 volValue = vol / 100.0;
 
@@ -872,7 +869,7 @@ bool CoreAudioConsumer::control(NamedList& params)
     if (!setVolStatus)
         Debug(DebugAll,"CoreAudioConsumer::control() [%p] - set volume failed on all channels",this);
 
-    return setVolStatus;
+    return TelEngine::controlReturn(&params,setVolStatus);
 }
 	
 CoreAudioChan::CoreAudioChan(const String& dev, unsigned int rate)
@@ -1010,7 +1007,7 @@ bool CoreAudioHandler::received(Message &msg)
 
     return true;
 }
-	
+
 bool StatusHandler::received(Message &msg)
 {
     const String* sel = msg.getParam("module");
@@ -1033,7 +1030,7 @@ bool MasqHandler::received(Message &msg)
     }
     return false;
 }
-	
+
 bool DropHandler::received(Message &msg)
 {
     String id(msg.getValue("id"));
@@ -1046,7 +1043,7 @@ bool DropHandler::received(Message &msg)
     }
     return false;
 }
-	
+
 bool AttachHandler::received(Message& msg)
 {
     int more = 2;
@@ -1057,7 +1054,7 @@ bool AttachHandler::received(Message& msg)
     	if (!src.startSkip("coreaudio/",false))
 	    src = "";
     }
-	
+
     String cons(msg.getValue("consumer"));
     if (cons.null())
 	more--;
@@ -1072,10 +1069,10 @@ bool AttachHandler::received(Message& msg)
 	Debug(DebugWarn,"CoreAudio asked to attach source '%s' and consumer '%s'",src.c_str(),cons.c_str());
 	return false;
     }
-	
-    RefPointer<DataEndpoint> dd = static_cast<DataEndpoint*>(msg.userObject("DataEndpoint"));
+
+    RefPointer<DataEndpoint> dd = static_cast<DataEndpoint*>(msg.userObject(YATOM("DataEndpoint")));
     if (!dd) {
-	CallEndpoint *ch = static_cast<CallEndpoint*>(msg.userObject("CallEndpoint"));
+	CallEndpoint *ch = static_cast<CallEndpoint*>(msg.userObject(YATOM("CallEndpoint")));
 	if (ch) {
 	    DataEndpoint::commonMutex().lock();
 	    dd = ch->setEndpoint();
@@ -1086,21 +1083,21 @@ bool AttachHandler::received(Message& msg)
 	Debug(DebugWarn,"CoreAudio attach request with no control or data channel!");
 	return false;
     }
-	
+
     if (src) {
 	CoreAudioSource* s = new CoreAudioSource(msg.getIntValue("rate",DEFAULT_SAMPLE_RATE));
 	if (s->init())
 	    dd->setSource(s);
 	s->deref();
     }
-	
+
     if (cons) {
 	CoreAudioConsumer* c = new CoreAudioConsumer(msg.getIntValue("rate",DEFAULT_SAMPLE_RATE));
 	if (c->init())
 	    dd->setConsumer(c);
 	c->deref();
     }
-	
+
     // Stop dispatching if we handled all requested
     return !more;
 }
@@ -1111,7 +1108,7 @@ CoreAudioPlugin::CoreAudioPlugin()
 {
     Output("Loaded module CoreAudio");
 }
-	
+
 void CoreAudioPlugin::initialize()
 {
     Output("Initializing module CoreAudio");
@@ -1129,5 +1126,5 @@ bool CoreAudioPlugin::isBusy() const
 {
     return (s_audioChan != 0);
 }
-	
+
 }; // anonymous namespace
