@@ -5,21 +5,18 @@
  * DirectSound channel driver for Windows.
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 // define DCOM before including windows.h so advanced COM functions can be compiled
@@ -394,7 +391,7 @@ bool DSoundPlay::control(NamedList& msg)
 	    outValue = 0;
 	msg.setParam("out_volume", String(outValue));
     }
-    return ok;
+    return TelEngine::controlReturn(&msg,ok);
 }
 
 void DSoundPlay::cleanup()
@@ -592,7 +589,7 @@ bool DSoundRec::control(TelEngine::NamedList &msg)
 
     inValue = (10 - m_rshift) * 10;
     msg.setParam("in_volume", String(inValue));
-    return ok;
+    return TelEngine::controlReturn(&msg,ok);
 }
 
 
@@ -615,7 +612,7 @@ bool DSoundSource::control(NamedList& msg)
 {
     if (m_dsound)
 	return m_dsound->control(msg);
-    return false;
+    return TelEngine::controlReturn(&msg,false);
 }
 
 
@@ -690,9 +687,9 @@ bool AttachHandler::received(Message &msg)
     if (src.null() && cons.null())
 	return false;
 
-    DataEndpoint *dd = static_cast<DataEndpoint*>(msg.userObject("DataEndpoint"));
+    DataEndpoint *dd = static_cast<DataEndpoint*>(msg.userObject(YATOM("DataEndpoint")));
     if (!dd) {
-	CallEndpoint *ch = static_cast<CallEndpoint*>(msg.userObject("CallEndpoint"));
+	CallEndpoint *ch = static_cast<CallEndpoint*>(msg.userObject(YATOM("CallEndpoint")));
 	if (ch)
 	    dd = ch->setEndpoint();
     }
