@@ -5,23 +5,20 @@
  * Conference room data mixer
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
  * N-way mixing with self echo suppresion idea by Andrew McDonald
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include <yatephone.h>
@@ -924,7 +921,7 @@ unsigned long ConfConsumer::Consume(const DataBlock& data, unsigned long tStamp,
 	maxLock = 500000;
     // make sure looping back conferences is not fatal
     if (!m_room->lock(maxLock)) {
-	Debug(&__plugin,DebugWarn,"Failed to lock room '%s' - data loopback?%s [%p]",
+	Alarm(&__plugin,"bug",DebugWarn,"Failed to lock room '%s' - data loopback?%s [%p]",
 	    m_room->toString().c_str(),(autoMute ? " Channel muted!" : ""),this);
 	// mute the channel to avoid getting back here
 	if (autoMute)
@@ -1019,7 +1016,7 @@ bool ConfConsumer::control(NamedList& msg)
 	m_smart = param->toBoolean();
 	ok = true;
     }
-    return DataConsumer::control(msg) || ok;
+    return TelEngine::controlReturn(&msg,DataConsumer::control(msg) || ok);
 }
 
 
@@ -1210,7 +1207,7 @@ bool ConfHandler::received(Message& msg)
 	    Debug(&__plugin,DebugNote,"Conference request with no channel!");
 	return ok;
     }
-    if (chan->getObject("ConfChan")) {
+    if (chan->getObject(YATOM("ConfChan"))) {
 	Debug(&__plugin,DebugWarn,"Conference request from a conference leg!");
 	return false;
     }
