@@ -5,22 +5,19 @@
  * External module handler
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  * Portions copyright (C) 2005 Maciek Kaminski
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include <yatephone.h>
@@ -59,6 +56,7 @@ static Configuration s_cfg;
 static ObjList s_chans;
 static ObjList s_modules;
 static Mutex s_mutex(true,"ExtModule");
+static Mutex s_watchMutex(false,"ExtWatchers");
 static int s_waitFlush = WAIT_FLUSH;
 static int s_timeout = MSG_TIMEOUT;
 static bool s_timebomb = false;
@@ -190,11 +188,14 @@ public:
     inline MsgWatcher(ExtModReceiver* receiver)
 	: m_receiver(receiver)
 	{ }
-    virtual ~MsgWatcher();
+    virtual void destruct();
     virtual void dispatched(const Message& msg, bool handled);
     bool addWatched(const String& name);
     bool delWatched(const String& name);
+protected:
+    virtual void destroyed();
 private:
+    void clear();
     ExtModReceiver* m_receiver;
     ObjList m_watched;
 };
@@ -225,7 +226,7 @@ public:
     void cleanup();
     bool flush();
     void die(bool clearChan = true);
-    void use();
+    bool use();
     bool unuse();
     inline const String& scriptFile() const
 	{ return m_script; }
@@ -235,6 +236,8 @@ public:
 	{ return m_selfWatch; }
     inline void setRestart(bool restart)
 	{ m_restart = restart; }
+    inline bool dead() const
+	{ return m_dead; }
     void describe(String& rval) const;
 
 private:
@@ -594,10 +597,8 @@ ExtMessage::~ExtMessage()
 
 void ExtMessage::startup(ExtModReceiver* recv)
 {
-    if (recv && m_id) {
+    if (recv && m_id && recv->use())
 	m_receiver = recv;
-	m_receiver->use();
-    }
     Engine::enqueue(this);
 }
 
@@ -606,36 +607,49 @@ void ExtMessage::dispatched(bool accepted)
     m_accepted = accepted;
     Message::dispatched(accepted);
 }
-  
 
-MsgWatcher::~MsgWatcher()
+
+void MsgWatcher::destruct()
 {
-    Engine::self()->setHook(this,true);
-    if (m_receiver && (m_receiver->m_watcher == this))
-	m_receiver->m_watcher = 0;
-    m_receiver = 0;
+    clear();
+    MessagePostHook::destruct();
+}
+
+void MsgWatcher::destroyed()
+{
+    clear();
+    MessagePostHook::destroyed();
 }
 
 void MsgWatcher::dispatched(const Message& msg, bool handled)
 {
-    if (!m_receiver)
+    Lock mylock(s_watchMutex);
+    ExtModReceiver* recv = m_receiver;
+    Lock lock(recv);
+    if (!(recv && recv->use()))
 	return;
+    mylock.drop();
 
-    if (!m_receiver->selfWatch()) {
+    if (!recv->selfWatch()) {
 	// check if the message was generated by ourselves - avoid reentrance
 	ExtMessage* m = YOBJECT(ExtMessage,&msg);
-	if (m && m->belongsTo(m_receiver))
+	if (m && m->belongsTo(recv)) {
+	    recv->unuse();
 	    return;
+	}
     }
 
     ObjList* l = m_watched.skipNull();
     for (; l; l = l->skipNext()) {
 	const String* s = static_cast<const String*>(l->get());
-	if (s->null() || (*s == msg)) {
-	    m_receiver->returnMsg(&msg,"",handled);
+	if (s->null() || (*s == msg))
 	    break;
-	}
     }
+    if (l && m_receiver) {
+	lock.drop();
+	recv->returnMsg(&msg,"",handled);
+    }
+    recv->unuse();
 }
 
 bool MsgWatcher::addWatched(const String& name)
@@ -660,13 +674,23 @@ bool MsgWatcher::delWatched(const String& name)
     return false;
 }
 
+void MsgWatcher::clear()
+{
+    Engine::self()->setHook(this,true);
+    ExtModReceiver* recv = m_receiver;
+    m_receiver = 0;
+    if (recv && (recv->m_watcher == this))
+	recv->m_watcher = 0;
+}
+
 
 ExtModReceiver* ExtModReceiver::build(const char* script, const char* args, bool ref,
 				      File* ain, File* aout, ExtModChan* chan)
 {
     ExtModReceiver* recv = new ExtModReceiver(script,args,ain,aout,chan);
     if (ref) {
-	recv->use();
+	if (!recv->use())
+	    return 0;
 	if (recv->start())
 	    return recv;
 	recv->unuse();
@@ -693,12 +717,14 @@ ExtModReceiver* ExtModReceiver::find(const String& script)
     return 0;
 }
 
-void ExtModReceiver::use()
+bool ExtModReceiver::use()
 {
     lock();
-    if (m_use > 0)
+    bool ok = (m_use > 0);
+    if (ok)
 	++m_use;
     unlock();
+    return ok;
 }
 
 bool ExtModReceiver::unuse()
@@ -818,9 +844,11 @@ bool ExtModReceiver::start()
 
 bool ExtModReceiver::flush()
 {
+    s_watchMutex.lock();
     lock();
     bool needWait = (0 != m_watcher);
     TelEngine::destruct(m_watcher);
+    s_watchMutex.unlock();
     // Make sure we release all pending messages and not accept new ones
     needWait = needWait || (m_relays.count() != 0);
     if (s_pluginSafe)
@@ -944,7 +972,7 @@ bool ExtModReceiver::received(Message &msg, int id)
 	lock();
 	ok = (m_waiting.find(&h) != 0);
 	if (ok && tout && (Time::now() > tout)) {
-	    Debug(DebugWarn,"Message %p '%s' did not return in %d msec [%p]",
+	    Alarm("extmodule","performance",DebugWarn,"Message %p '%s' did not return in %d msec [%p]",
 		&msg,msg.c_str(),m_timeout,this);
 	    m_waiting.remove(&h,false);
 	    ok = false;
@@ -1164,7 +1192,7 @@ bool ExtModReceiver::outputLine(const char* line)
 	}
 	if (len > 0) {
 	    unlock();
-	    Thread::yield();
+	    Thread::idle();
 	    lock();
 	}
     }
@@ -1182,7 +1210,7 @@ bool ExtModReceiver::outputLine(const char* line)
 	    return true;
 	if (w < 0)
 	    return false;
-	Thread::yield();
+	Thread::idle();
 	lock();
     }
 }
@@ -1730,7 +1758,7 @@ bool ExtListener::init(const NamedList& sect)
     else if (role == "channel")
 	m_role = ExtModReceiver::RoleChannel;
     else if (role) {
-	Debug(DebugWarn,"Unknown role '%s' of listener '%s'",role.c_str(),m_name.c_str());
+	Debug(DebugConf,"Unknown role '%s' of listener '%s'",role.c_str(),m_name.c_str());
 	return false;
     }
     String type(sect.getValue("type"));
@@ -1751,7 +1779,7 @@ bool ExtListener::init(const NamedList& sect)
 	    return false;
     }
     else {
-	Debug(DebugWarn,"Unknown type '%s' of listener '%s'",type.c_str(),m_name.c_str());
+	Debug(DebugConf,"Unknown type '%s' of listener '%s'",type.c_str(),m_name.c_str());
 	return false;
     }
     if (!m_socket.create(addr.family(),SOCK_STREAM)) {
@@ -1779,7 +1807,7 @@ void ExtListener::run()
 	if (!skt) {
 	    if (m_socket.canRetry())
 		continue;
-	    Debug(DebugWarn,"Error on accept(), shutting down ExtListener '%s'",m_name.c_str());
+	    Alarm("extmodule","socket",DebugWarn,"Error on accept(), shutting down ExtListener '%s'",m_name.c_str());
 	    break;
 	}
 	String tmp = addr.host();
@@ -1805,7 +1833,7 @@ ExtListener* ExtListener::build(const char* name, const NamedList& sect)
 	return 0;
     ExtListener* ext = new ExtListener(name);
     if (!ext->init(sect)) {
-	Debug(DebugGoOn,"Could not start listener '%s'",name);
+	Alarm("extmodule","config",DebugWarn,"Could not start listener '%s'",name);
 	delete ext;
 	ext = 0;
     }
