@@ -10,21 +10,18 @@
  * Fax driver (transmission+receiving)
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 // For SpanDSP we have to ask for various C99 stuff
@@ -176,6 +173,7 @@ public:
     enum Type {
 	Unknown,
 	Detect,
+	Switch,
 	Analog,
 	Digital,
     };
@@ -194,7 +192,7 @@ public:
 	{ return m_sender; }
     inline bool isCaller() const
 	{ return m_caller; }
-    static void setParams(Message& msg, Type type, int t38version = -1);
+    void setParams(Message& msg, Type type, int t38version = -1);
     Type guessType(const Message& msg);
     static int guessT38(const Message& msg, int version = 0);
 private:
@@ -229,6 +227,8 @@ static bool s_debug = false;
 static const TokenDict s_types[] = {
     { "autodetect",  FaxChan::Detect },
     { "detect",      FaxChan::Detect },
+    { "autoswitch",  FaxChan::Switch },
+    { "switch",      FaxChan::Switch },
     { "analog",      FaxChan::Analog },
     { "digital",     FaxChan::Digital },
     { 0, 0 }
@@ -658,6 +658,7 @@ bool FaxChan::startup(Message& msg)
     Type t = guessType(msg);
     switch (t) {
 	case Detect:
+	case Switch:
 	    m_t38version = guessT38(msg,m_t38version);
 	    // fall through
 	case Analog:
@@ -682,7 +683,15 @@ bool FaxChan::startup(Message& msg)
 bool FaxChan::msgAnswered(Message& msg)
 {
     if (Channel::msgAnswered(msg)) {
+	bool chg = (Switch == m_type);
 	startup(msg);
+	if (chg && (Analog == m_type)) {
+	    Message* m = message("call.update");
+	    m->addParam("operation","notify");
+	    m->addParam("audio_changed",String::boolText(true));
+	    setParams(*m,Digital,m_t38version);
+	    Engine::enqueue(m);
+	}
 	return true;
     }
     return false;
@@ -751,6 +760,10 @@ void FaxChan::setParams(Message& msg, Type type, int t38version)
 		msg.setParam("t38version",ver);
 		msg.setParam("osdp_image_T38FaxVersion",ver);
 	    }
+	case Switch:
+	    if (Unknown == m_type)
+		m_type = type;
+	    break;
 	default:
 	    break;
     }
@@ -804,6 +817,11 @@ bool FaxDriver::msgExecute(Message& msg, String& dest)
 	return false;
     }
     dest = dest.matchString(2);
+    if (transmit && !File::exists(dest)) {
+	msg.setParam("error","noroute");
+	msg.setParam("reason","File not found");
+	return false;
+    }
 
     RefPointer<FaxChan> fc;
     CallEndpoint* ce = YOBJECT(CallEndpoint,msg.userData());
