@@ -5,21 +5,18 @@
  * File transfer Driver
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include <yatephone.h>
@@ -48,9 +45,9 @@ class FileDriver;                        // The driver
 class FileHolder
 {
 public:
-    inline FileHolder(const String& name)
+    inline FileHolder(const String& name, const String& dropChan)
 	: m_fileName(name), m_fileTime(0), m_fileSize(-1), m_transferred(0),
-	m_params("")
+	m_params(""), m_dropChan(dropChan), m_waitOnDropMs(0)
 	{}
     // Get file name
     inline const String& fileName() const
@@ -71,6 +68,18 @@ public:
 	    }
 	    return m_fileTime;
 	}
+    // Set drop chan id
+    inline void setDropChan(const String& id)
+	{ m_dropChan = id; }
+    // Build drop message. Reset drop chan
+    inline Message* dropMessage() {
+	    if (!m_dropChan)
+		return 0;
+	    Message* m = new Message("call.drop");
+	    m->addParam("id",m_dropChan);
+	    m_dropChan = "";
+	    return m;
+	}
     // Add MD5 and/or file info parameters
     void addFileInfo(NamedList& params, bool md5, bool extra);
     // Add saved params to another list
@@ -83,6 +92,9 @@ protected:
     int64_t m_transferred;               // Transferred bytes
     String m_md5HexDigest;               // MD5 digest of the file
     NamedList m_params;                  // Parameters to copy in notifications
+    String m_dropChan;                   // Channel to drop on termination
+    unsigned int m_waitOnDropMs;         // Time to wait to drop channel
+
 };
 
 // A file data source
@@ -94,9 +106,6 @@ public:
     // Create the data source, and init it
     FileSource(const String& file, NamedList* params = 0, const char* chan = 0,
 	const char* format = 0);
-    // Set drop chan id
-    inline void setDropChan(const String& id)
-	{ m_dropChan = id; }
     // Check if this data source is connected
     inline bool connected() {
 	    Lock mylock(this);
@@ -112,7 +121,6 @@ private:
     virtual void destroyed();
 
     String m_notify;                     // Target id to notify
-    String m_dropChan;                   // Channel to drop on termination
     bool m_notifyProgress;               // Notify file transfer progress
     bool m_notifyPercent;                // Notify percent changes only
     int m_percent;                       // Notify current percent
@@ -130,9 +138,9 @@ class FileConsumer : public DataConsumer, public FileHolder
 public:
     FileConsumer(const String& file, NamedList* params = 0, const char* chan = 0,
 	const char* format = "data");
-    // Set drop chan id
-    inline void setDropChan(const String& id)
-	{ m_dropChan = id; }
+    // Check if file should be overwritten
+    inline bool overWrite() const
+	{ return m_overWrite; }
     // Check if this data consumer is connected
     inline bool connected() const
 	{ return 0 != getConnSource(); }
@@ -147,10 +155,11 @@ protected:
     virtual void destroyed();
     // Terminate: close file, notify, check MD5 (if used)
     void terminate(const char* error = 0);
+    // Make sure a file path exists
+    bool createPath(String* error);
 private:
     String m_notify;                     // Target id to notify
     String m_tmpFileName;
-    String m_dropChan;                   // Channel to drop on termination
     bool m_notifyProgress;               // Notify file transfer progress
     bool m_notifyPercent;                // Notify percent changes only
     int m_percent;                       // Notify current percent
@@ -158,6 +167,8 @@ private:
     u_int64_t m_startTime;
     bool m_terminated;
     bool m_delTemp;                      // Delete temporary file
+    bool m_createPath;                   // Create file path
+    bool m_overWrite;                    // Overwright existing file
 };
 
 // File source worker
@@ -203,9 +214,11 @@ public:
 	const char* error = 0, const NamedList* params = 0,
 	const char* chan = 0);
     // Copy params
-    inline void copyParams(NamedList& dest, const NamedList& src) {
+    inline void copyParams(NamedList& dest, const NamedList& src, bool exec = false) {
 	    Lock lock(this);
-	    dest.copyParams(src,m_copyParams);
+	    const String& list = !exec ? m_copyParams : m_copyExecParams;
+	    if (list)
+		dest.copyParams(src,list);
 	}
     // Attach default path to a file if file path is missing
     void getPath(String& file);
@@ -313,6 +326,16 @@ inline String& dirStr(bool outgoing)
     return outgoing ? s_dirSend : s_dirRecv;
 }
 
+// Make sure a path contains only current system path separators
+static void toNativeSeparators(String& path)
+{
+    char repl = (*Engine::pathSeparator() == '/') ? '\\' : '/';
+    char* s = (char*)path.c_str();
+    for (unsigned int i = 0; i < path.length(); i++, s++)
+	if (*s == repl)
+	    *s = *Engine::pathSeparator();
+}
+
 
 /*
  * FileHolder
@@ -339,8 +362,7 @@ void FileHolder::addFileInfo(NamedList& params, bool md5, bool extra)
 FileSource::FileSource(const String& file, NamedList* params, const char* chan,
     const char* format)
     : DataSource(!null(format) ? format : "data"),
-    FileHolder(file),
-    m_dropChan(chan),
+    FileHolder(file,chan),
     m_notifyProgress(s_notifyProgress),
     m_notifyPercent(s_notifyPercent),
     m_percent(0),
@@ -352,8 +374,11 @@ FileSource::FileSource(const String& file, NamedList* params, const char* chan,
 	m_notifyProgress = params->getBoolValue("notify_progress",m_notifyProgress);
 	m_buflen = getIntValue(*params,"send_chunk_size",s_sendChunk,SEND_CHUNK_MIN,true);
 	m_sleepMs = getIntValue(*params,"send_interval",s_sendIntervalMs,SEND_SLEEP_MIN,false);
+	m_waitOnDropMs = params->getIntValue("wait_on_drop",0,0);
 	__plugin.copyParams(m_params,*params);
     }
+    if (!m_sleepMs)
+	m_sleepMs = SEND_SLEEP_DEF;
     Debug(&__plugin,DebugAll,"FileSource('%s') [%p]",file.c_str(),this);
 }
 
@@ -388,6 +413,8 @@ bool FileSource::init(bool buildMd5, String& error)
 // Wait for a consumer to be attached. Send the file
 void FileSource::run()
 {
+    DDebug(&__plugin,DebugAll,"FileSource(%s) start running [%p]",
+	m_fileName.c_str(),this);
     m_transferred = 0;
     FileDriver::notifyStatus(true,m_notify,"pending",m_fileName,0,m_fileSize,0,&m_params);
 
@@ -396,18 +423,16 @@ void FileSource::run()
     // Use a while() to break to the end to cleanup properly
     while (true) {
 	// Wait until at least one consumer is attached
-    	while (true) {
+	while (true) {
 	    if (Thread::check(false)) {
 		error = "cancelled";
 		break;
 	    }
-	    if (!lock(5000000)) {
-		Thread::msleep(1);
+	    if (!lock(100000))
 		continue;
-	    }
-	    int cons = (0 != m_consumers.skipNull());
+	    bool cons = (0 != m_consumers.skipNull());
 	    unlock();
-	    Thread::yield();
+	    Thread::idle();
 	    if (cons)
 		break;
 	}
@@ -420,8 +445,11 @@ void FileSource::run()
 
 	FileDriver::notifyStatus(true,m_notify,"start",m_fileName,0,m_fileSize,0,
 	    &m_params,m_dropChan);
+
 	unsigned long tStamp = 0;
 	start = Time::msecNow();
+	if (!m_fileSize)
+	    break;
 	// Set file pos at start
 	if (-1 == m_file.Stream::seek(0)) {
 	    Thread::errorString(error,m_file.error());
@@ -479,10 +507,7 @@ void FileSource::run()
 		    break;
 	    }
 	    tStamp += m_sleepMs;
-	    if (m_sleepMs)
-		Thread::msleep(m_sleepMs,false);
-	    else
-		Thread::yield(false);
+	    Thread::msleep(m_sleepMs,false);
 	}
 	break;
     }
@@ -503,14 +528,25 @@ void FileSource::run()
     FileDriver::notifyStatus(true,m_notify,"terminated",m_fileName,
 	m_transferred,m_fileSize,error,&m_params);
 
-    if (m_dropChan) {
-	// Wait a while to give some time to the remote party to receive the data
-	unsigned int n = !error ? s_srcLingerIntervals : 0;
+    Message* m = dropMessage();
+    if (m) {
+	// Wait for a while to give some time to the remote party to receive the data
+	unsigned int n = 0;
+	if (!error) {
+	    if (m_waitOnDropMs) {
+		n = m_waitOnDropMs / m_sleepMs;
+		if (!n)
+		    n = 1;
+	    }
+	    else
+		n = s_srcLingerIntervals;
+	}
+	XDebug(&__plugin,DebugAll,
+	    "FileSource(%s) dropping chan '%s' waiting %u intervals of %ums [%p]",
+	    m_fileName.c_str(),m->getValue("id"),n,m_sleepMs,this);
 	for (; n && !Thread::check(false); n--)
-	    Thread::msleep(m_sleepMs ? m_sleepMs : SEND_SLEEP_DEF,false);
+	    Thread::msleep(m_sleepMs,false);
 	// Drop channel
-	Message* m = new Message("call.drop");
-	m->addParam("id",m_dropChan);
 	if (error) {
 	    if (error == "cancelled")
 		m->addParam("reason","cancelled");
@@ -553,13 +589,14 @@ void FileSource::destroyed()
 FileConsumer::FileConsumer(const String& file, NamedList* params, const char* chan,
     const char* format)
     : DataConsumer(!null(format) ? format : "data"),
-    FileHolder(file),
-    m_dropChan(chan),
+    FileHolder(file,chan),
     m_notifyProgress(s_notifyProgress),
     m_notifyPercent(s_notifyPercent),
     m_percent(0),
-    m_startTime(0), m_terminated(false), m_delTemp(true)
+    m_startTime(0), m_terminated(false), m_delTemp(true),
+    m_createPath(false), m_overWrite(false)
 {
+    toNativeSeparators(m_fileName);
     __plugin.getPath(m_fileName);
     if (params) {
 	m_notify = params->getValue("notify");
@@ -567,10 +604,12 @@ FileConsumer::FileConsumer(const String& file, NamedList* params, const char* ch
 	m_fileSize = params->getIntValue("file_size",0);
 	m_md5HexDigest = params->getValue("file_md5");
 	m_fileTime = params->getIntValue("file_time");
+	m_createPath = params->getBoolValue(YSTRING("create_path"));
+	m_overWrite = params->getBoolValue(YSTRING("overwrite"));
 	__plugin.copyParams(m_params,*params);
     }
     Debug(&__plugin,DebugAll,"FileConsumer('%s') [%p]",m_fileName.c_str(),this);
-    if (m_fileName && !(m_fileName.endsWith("/") || m_fileName.endsWith("\\"))) {
+    if (m_fileName && m_fileName[m_fileName.length() - 1] != *Engine::pathSeparator()) {
 	m_tmpFileName << m_fileName << ".tmp";
 	m_delTemp = !File::exists(m_tmpFileName);
     }
@@ -580,17 +619,39 @@ FileConsumer::FileConsumer(const String& file, NamedList* params, const char* ch
 
 unsigned long FileConsumer::Consume(const DataBlock& data, unsigned long tStamp, unsigned long flags)
 {
+    if (m_terminated)
+	return 0;
+
     if (!m_startTime) {
 	m_startTime = Time::now();
 	FileDriver::notifyStatus(false,m_notify,"start",m_fileName,0,m_fileSize,0,
 	    &m_params,m_dropChan);
 	// Check file existence
 	if (fileExists(true,false)) {
-	    terminate("File exists");
-	    Debug(&__plugin,DebugNote,
-		"FileConsumer(%s) failed to start: temporary file already exists! [%p]",
-		m_fileName.c_str(),this);
-	    return 0;
+	    if (!m_overWrite) {
+		terminate("File exists");
+		Debug(&__plugin,DebugNote,
+		    "FileConsumer(%s) failed to start: temporary file already exists! [%p]",
+		    m_fileName.c_str(),this);
+		return 0;
+	    }
+	    int code = 0;
+	    if (!File::remove(m_tmpFileName,&code)) {
+		String error;
+		Thread::errorString(error,code);
+		terminate(error);
+		Debug(&__plugin,DebugNote,
+		    "FileConsumer(%s) failed to delete temporary file. %d: '%s' [%p]",
+		    m_fileName.c_str(),code,error.c_str(),this);
+		return 0;
+	    }
+	}
+	else if (m_createPath) {
+	    String error;
+	    if (!createPath(&error)) {
+		terminate(error);
+		return 0;
+	    }
 	}
 	m_delTemp = true;
 	if (!m_file.openPath(m_tmpFileName,true,false,true,true,true)) {
@@ -604,13 +665,10 @@ unsigned long FileConsumer::Consume(const DataBlock& data, unsigned long tStamp,
 	}
     }
 
-    if (data.null())
-	return 0;
-
     XDebug(&__plugin,DebugAll,"FileConsumer(%s) consuming %u bytes [%p]",
 	m_fileName.c_str(),data.length(),this);
 
-    if (m_file.valid()) {
+    if (data.length() && m_file.valid()) {
 	if (m_file.writeData(data.data(),data.length())) {
 	    if (m_md5HexDigest)
 		m_md5 << data;
@@ -635,7 +693,7 @@ unsigned long FileConsumer::Consume(const DataBlock& data, unsigned long tStamp,
     }
 
     m_transferred += data.length();
-    if (m_transferred && (m_transferred >= m_fileSize))
+    if (m_transferred >= m_fileSize)
 	terminate();
     return data.length();
 }
@@ -668,7 +726,7 @@ void FileConsumer::terminate(const char* error)
 	    break;
 	}
 	// Check file existence
-	if (fileExists(false,true)) {
+	if (!m_overWrite && fileExists(false,true)) {
 	    err = "File exists";
 	    break;
 	}
@@ -694,9 +752,8 @@ void FileConsumer::terminate(const char* error)
     // Notify and terminate drop the channel
     FileDriver::notifyStatus(false,m_notify,"terminated",m_fileName,
 	m_transferred,m_fileSize,err,&m_params);
-    if (m_dropChan) {
-	Message* m = new Message("call.drop");
-	m->addParam("id",m_dropChan);
+    Message* m = dropMessage();
+    if (m) {
 	if (err) {
 	    m->addParam("reason","failure");
 	    m->addParam("error",err);
@@ -705,6 +762,57 @@ void FileConsumer::terminate(const char* error)
     }
 }
 
+// Make sure a file path exists
+bool FileConsumer::createPath(String* error)
+{
+    const String& orig = m_tmpFileName;
+    if (!orig)
+	return true;
+    char sep = *Engine::pathSeparator();
+    int pos = orig.rfind(sep);
+    if (pos <= 0)
+	return true;
+    String path = orig.substr(0,pos);
+    ObjList list;
+    bool exists = false;
+    while (path) {
+	exists = File::exists(path);
+	if (exists)
+	    break;
+	int pos = path.rfind(sep);
+	if (pos < 0)
+	    break;
+	String* s = new String(path.substr(pos + 1));
+	if (!TelEngine::null(s))
+	    list.insert(s);
+	else
+	    TelEngine::destruct(s);
+	path = path.substr(0,pos);
+    }
+    int code = 0;
+    bool ok = true;
+    if (path && !exists)
+	ok = File::mkDir(path,&code);
+    while (ok) {
+	ObjList* o = list.skipNull();
+	if (!o)
+	    break;
+	path.append(*static_cast<String*>(o->get()),Engine::pathSeparator());
+	o->remove();
+	ok = File::mkDir(path,&code);
+    }
+    if (ok)
+	return true;
+    String tmp;
+    if (!error)
+	error = &tmp;
+    Thread::errorString(*error,code);
+    Debug(&__plugin,DebugNote,
+	"FileConsumer(%s) failed to create path for '%s'. %d: '%s' [%p]",
+	m_fileName.c_str(),orig.c_str(),code,error->c_str(),this);
+    return false;
+}
+
 
 /*
  * FileSourceWorker
@@ -813,7 +921,7 @@ bool FileDriver::msgExecute(Message& msg, String& dest)
 	}
 	else {
 	    cons = new FileConsumer(dest.matchString(2),&msg,0,format);
-	    ok = !cons->fileExists();
+	    ok = cons->overWrite() || !cons->fileExists();
 	    if (ok)
 		addConsumer(cons);
 	    else
@@ -843,7 +951,10 @@ bool FileDriver::msgExecute(Message& msg, String& dest)
     // Init call from here
     Message m("call.route");
     m.addParam("module",name());
-    m.copyParams(msg,m_copyExecParams);
+    copyParams(m,msg,true);
+    const String& cp = msg[YSTRING("copyparams")];
+    if (cp)
+	m.copyParams(msg,cp);
     String callto(msg.getValue("direct"));
     if (callto.null()) {
 	const char* targ = msg.getValue("target");
@@ -881,7 +992,7 @@ bool FileDriver::msgExecute(Message& msg, String& dest)
     }
     else {
 	cons = new FileConsumer(dest.matchString(2),&msg,0,format);
-	if (!cons->fileExists()) {
+	if (cons->overWrite() || !cons->fileExists()) {
 	    addConsumer(cons);
 	    fileHolder = static_cast<FileHolder*>(cons);
 	}
@@ -905,6 +1016,9 @@ bool FileDriver::msgExecute(Message& msg, String& dest)
     m.addParam("format",format);
     m.addParam("operation",dirStr(outgoing));
     fileHolder->addFileInfo(m,copyMD5,msg.getBoolValue("getfileinfo",s_srcFileInfo));
+    const String& remoteFile = msg[YSTRING("remote_file")];
+    if (remoteFile)
+	m.setParam(YSTRING("file_name"),remoteFile);
     m.addParam("cdrtrack","false");
     bool ok = Engine::dispatch(m);
     if (ok)
@@ -1143,7 +1257,7 @@ bool FileDriver::notifyStatus(bool send, const String& id, const char* status,
 void FileDriver::getPath(String& file)
 {
     // Check if the file already have a path separator
-    if (-1 != file.find('/') || -1 != file.find('\\'))
+    if (-1 != file.find(*Engine::pathSeparator()))
 	return;
     Lock lock(this);
     if (s_path)
