@@ -5,21 +5,18 @@
  * Jabber features module
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include <yatephone.h>
@@ -280,7 +277,7 @@ void JBFeaturesModule::initialize()
     s_groupSeparator = general->getValue("groups_separator");
     if (s_groupSeparator.length() == 2) {
 	DataBlock d;
-	d.unHexify(s_groupSeparator.c_str(),2);
+	d.unHexify(s_groupSeparator);
 	s_groupSeparator.clear();
 	if (d.length() && d[0])
 	    s_groupSeparator = (char)d[0];
@@ -460,7 +457,7 @@ bool JBFeaturesModule::handleFeaturePrivateData(JabberID& from, Message& msg)
 	    XMPPNamespace::IqPrivate);
 	XmlElement* pdata = 0;
 	if (m) {
-	    Array* a = static_cast<Array*>(m->userObject("Array"));
+	    Array* a = static_cast<Array*>(m->userObject(YATOM("Array")));
 	    String* data = a ? YOBJECT(String,a->get(0,1)) : 0;
 	    pdata = data ? XMPPUtils::getXml(*data) : 0;
 	    if (pdata) {
@@ -550,7 +547,7 @@ bool JBFeaturesModule::handleFeatureVCard(JabberID& from, Message& msg)
 	return buildError(msg,xml);
     XmlElement* vcard = 0;
     if (t == XMPPUtils::IqGet && m) {
-	Array* a = static_cast<Array*>(m->userObject("Array"));
+	Array* a = static_cast<Array*>(m->userObject(YATOM("Array")));
 	if (a) {
 	    String* vc = YOBJECT(String,a->get(0,1));
 	    XDebug(this,DebugInfo,"Got vcard for '%s': '%s'",p.getValue("username"),
@@ -607,7 +604,7 @@ bool JBFeaturesModule::handleFeatureMsgOffline(JabberID& from, Message& msg)
 	    p.addParam("maxcount",String(m_maxChatCount));
 	    Message* m = queryDb(p,m_chatAccount,m_chatQueryAdd);
 	    if (m) {
-		Array* a = static_cast<Array*>(m->userObject("Array"));
+		Array* a = static_cast<Array*>(m->userObject(YATOM("Array")));
 		String* res = a ? YOBJECT(String,a->get(0,1)) : 0;
 		if (res) {
 		    DDebug(this,DebugAll,"Got result %s to add chat",res->c_str());
@@ -633,7 +630,7 @@ bool JBFeaturesModule::handleFeatureMsgOffline(JabberID& from, Message& msg)
 	Message* m = queryDb(p,m_chatAccount,m_chatQueryGet);
 	if (!m)
 	    return false;
-	Array* a = static_cast<Array*>(m->userObject("Array"));
+	Array* a = static_cast<Array*>(m->userObject(YATOM("Array")));
 	int rows = a ? a->getRows() : 0;
 	int cols = a ? a->getColumns() : 0;
 	DDebug(this,DebugAll,"Got %d offline messages for user =%s",
