@@ -5,21 +5,18 @@
  * A sample message sniffer that inserts a wildcard message handler
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include <yatengine.h>
@@ -40,6 +37,7 @@ static const char* s_debugs[] =
     "yes",
     "no",
     "filter",
+    "timer",
     0
 };
 
@@ -67,6 +65,7 @@ public:
 };
 
 static bool s_active = true;
+static bool s_timer = false;
 static Regexp s_filter;
 static Mutex s_mutex(false,"FilterSniff");
 
@@ -95,7 +94,7 @@ static void dumpParams(const Message &msg, String& par)
 
 bool SniffHandler::received(Message &msg)
 {
-    if (msg == YSTRING("engine.timer"))
+    if (!s_timer && (msg == YSTRING("engine.timer")))
 	return false;
     if (msg == YSTRING("engine.command")) {
 	static const String name("sniffer");
@@ -103,14 +102,18 @@ bool SniffHandler::received(Message &msg)
 	if (line.startSkip(name)) {
 	    line >> s_active;
 	    line.trimSpaces();
+	    if (line.startSkip("timer"))
+		(line >> s_timer).trimSpaces();
 	    if (line.startSkip("filter")) {
 		s_mutex.lock();
 		s_filter = line;
 		s_mutex.unlock();
 	    }
-	    msg.retValue() << "Message sniffer is " << (s_active ? "on" : "off");
+	    msg.retValue() << "Message sniffer: " << (s_active ? "on" : "off");
+	    if (s_active)
+		msg.retValue() << ", timer: " << (s_timer ? "on" : "off");
 	    if (s_active && s_filter)
-		msg.retValue() << " filter " << s_filter;
+		msg.retValue() << ", filter: " << s_filter;
 	    msg.retValue() << "\r\n";
 	    return true;
 	}
@@ -150,7 +153,7 @@ bool SniffHandler::received(Message &msg)
 
 void HookHandler::dispatched(const Message& msg, bool handled)
 {
-    if ((!s_active) || (msg == YSTRING("engine.timer")))
+    if (!s_active || (!s_timer && (msg == YSTRING("engine.timer"))))
 	return;
     Lock lock(s_mutex);
     if (s_filter && !s_filter.matches(msg))
