@@ -5,21 +5,18 @@
  * OpenSSL based SSL/TLS socket support
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include <yatephone.h>
@@ -534,7 +531,7 @@ bool SslHandler::received(Message& msg)
 	return 0 != __plugin.findContext(msg);
     }
     addRand(msg.msgTime());
-    Socket** ppSock = static_cast<Socket**>(msg.userObject("Socket*"));
+    Socket** ppSock = static_cast<Socket**>(msg.userObject(YATOM("Socket*")));
     if (!ppSock) {
 	Debug(&__plugin,DebugGoOn,"SslHandler: No pointer to Socket");
 	return false;
@@ -767,7 +764,7 @@ bool CipherHandler::received(Message& msg)
     const String* name = msg.getParam("cipher");
     if (!name)
 	return false;
-    Cipher** ppCipher = static_cast<Cipher**>(msg.userObject("Cipher*"));
+    Cipher** ppCipher = static_cast<Cipher**>(msg.userObject(YATOM("Cipher*")));
 #ifndef OPENSSL_NO_AES
     if (*name == "aes_ctr") {
 	if (ppCipher)
