@@ -6,21 +6,18 @@
  * administrating interface.
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include <yatengine.h>
@@ -156,7 +153,7 @@ public:
 	{ }
     void* getObject(const String& name) const
     {
-	if (name == "Socket*")
+	if (name == YATOM("Socket*"))
 	    return m_sock;
 	return RefObject::getObject(name);
     }
@@ -212,6 +209,10 @@ public:
     bool processChar(unsigned char c);
     bool processLine(const char *line, bool saveLine = true);
     bool autoComplete();
+    void putConnInfo(NamedList& msg) const;
+    void execCommand(const String& str, bool saveOffset);
+    bool pagedCommand(bool up);
+    void endSubnegotiation();
     void errorBeep();
     void clearLine();
     void writeBuffer();
@@ -234,8 +235,12 @@ private:
     bool m_output;
     bool m_colorize;
     bool m_machine;
+    int m_offset;
     int m_threshold;
     Socket* m_socket;
+    unsigned char m_subBuf[64];
+    unsigned char m_subOpt;
+    unsigned char m_subLen;
     unsigned char m_lastch;
     unsigned char m_escmode;
     bool m_echoing;
@@ -247,6 +252,8 @@ private:
     RefPointer<RManagerListener> m_listener;
     unsigned int m_cursorPos;
     unsigned int m_histLen;
+    unsigned int m_width;
+    unsigned int m_height;
 };
 
 class RManager : public Plugin
@@ -309,13 +316,13 @@ bool RManagerListener::initSocket()
 
     m_socket.create(AF_INET, SOCK_STREAM);
     if (!m_socket.valid()) {
-	Debug("RManager",DebugGoOn,"Unable to create the listening socket: %s",
+	Alarm("RManager","socket",DebugGoOn,"Unable to create the listening socket: %s",
 	    strerror(m_socket.error()));
 	return false;
     }
 
     if (!m_socket.setBlocking(false)) {
-	Debug("RManager",DebugGoOn, "Failed to set listener to nonblocking mode: %s",
+	Alarm("RManager","socket",DebugGoOn, "Failed to set listener to nonblocking mode: %s",
 	    strerror(m_socket.error()));
 	return false;
     }
@@ -326,12 +333,12 @@ bool RManagerListener::initSocket()
     m_address << sa.host() << ":" << sa.port();
     m_socket.setReuse();
     if (!m_socket.bind(sa)) {
-	Debug("RManager",DebugGoOn,"Failed to bind to %s : %s",
+	Alarm("RManager","socket",DebugGoOn,"Failed to bind to %s : %s",
 	    m_address.c_str(),strerror(m_socket.error()));
 	return false;
     }
     if (!m_socket.listen(2)) {
-	Debug("RManager",DebugGoOn,"Unable to listen on socket: %s",
+	Alarm("RManager","socket",DebugGoOn,"Unable to listen on socket: %s",
 	    strerror(m_socket.error()));
 	return false;
     }
@@ -413,9 +420,11 @@ Connection::Connection(Socket* sock, const char* addr, RManagerListener* listene
     : Thread("RManager Connection"),
       m_aliases(""),
       m_auth(None), m_debug(false), m_output(false), m_colorize(false), m_machine(false),
-      m_threshold(DebugAll),
-      m_socket(sock), m_lastch(0), m_escmode(0), m_echoing(false), m_beeping(false),
-      m_timeout(0), m_address(addr), m_listener(listener), m_cursorPos(0), m_histLen(DEF_HISTORY)
+      m_offset(-1), m_threshold(DebugAll),
+      m_socket(sock), m_subOpt(0), m_subLen(0),
+      m_lastch(0), m_escmode(0), m_echoing(false), m_beeping(false),
+      m_timeout(0), m_address(addr), m_listener(listener),
+      m_cursorPos(0), m_histLen(DEF_HISTORY), m_width(0), m_height(24)
 {
     s_mutex.lock();
     s_connList.append(this);
@@ -462,12 +471,12 @@ void Connection::run()
     m_histLen = cfg().getIntValue("maxhistory",DEF_HISTORY);
     if (m_histLen > MAX_HISTORY)
 	m_histLen = MAX_HISTORY;
-    String hdr = cfg().getValue("header","YATE ${version}-${release} (http://YATE.null.ro) ready on ${nodename}.");
+    String hdr = cfg().getValue("header","YATE ${version}-${release} r${revision} (http://YATE.null.ro) ready on ${nodename}.");
     Engine::runParams().replaceParams(hdr);
     if (cfg().getBoolValue("telnet",true)) {
 	m_colorize = cfg().getBoolValue("color",false);
-	// WILL SUPPRESS GO AHEAD, WILL ECHO - and enough BS and blanks to hide them
-	writeStr("\377\373\003\377\373\001\r      \b\b\b\b\b\b");
+	// WILL SUPPRESS GO AHEAD, WILL ECHO, DO NAWS - and enough BS and blanks to hide them
+	writeStr("\377\373\003\377\373\001\377\375\037\r         \b\b\b\b\b\b\b\b\b");
     }
     if (hdr) {
 	writeStr("\r" + hdr + "\r\n");
@@ -563,6 +572,20 @@ bool Connection::processTelnetChar(unsigned char c)
 	c,(c >= ' ') ? "" : "^", (c >= ' ') ? c : c+0x40);
     if (m_lastch == 255) {
 	m_lastch = 0;
+	if (m_subOpt) {
+	    switch (c) {
+		case 240: // SE
+		    endSubnegotiation();
+		    m_subOpt = 0;
+		    m_subLen = 0;
+		    return false;
+		case 255:
+		    break;
+		default:
+		    Debug("RManager",DebugMild,"Unsupported telnet octet %u (0x%02X) after IAC in SB",c,c);
+		    return false;
+	    }
+	}
 	switch (c) {
 	    case 241: // NOP
 		return false;
@@ -578,6 +601,7 @@ bool Connection::processTelnetChar(unsigned char c)
 	    case 248: // EL
 		c = 0x15;
 		break;
+	    case 250: // SB
 	    case 251: // WILL
 	    case 252: // WON'T
 	    case 253: // DO
@@ -594,6 +618,10 @@ bool Connection::processTelnetChar(unsigned char c)
     else if (m_lastch) {
 	DDebug("RManager",DebugMild,"Command %u param %u",m_lastch,c);
 	switch (m_lastch) {
+	    case 250: // SB
+		m_subOpt = c;
+		m_subLen = 0;
+		break;
 	    case 251: // WILL
 		switch (c) {
 		    case 1: // ECHO
@@ -601,6 +629,7 @@ bool Connection::processTelnetChar(unsigned char c)
 			writeStr("\377\374\001"); // WON'T ECHO
 			break;
 		}
+		break;
 	    case 252: // WON'T
 		break;
 	    case 253: // DO
@@ -637,9 +666,32 @@ bool Connection::processTelnetChar(unsigned char c)
 	m_lastch = c;
 	return false;
     }
+    if (m_subOpt) {
+	if (m_subLen < sizeof(m_subBuf))
+	    m_subBuf[m_subLen++] = c;
+	return false;
+    }
     return processChar(c);
 }
 
+// process Telnet subnegotiation
+void Connection::endSubnegotiation()
+{
+    switch (m_subOpt) {
+	case 31: // NAWS
+	    if (m_subLen != 4)
+		break;
+	    m_width = (((unsigned int)m_subBuf[0]) << 8) | m_subBuf[1];
+	    m_height = (((unsigned int)m_subBuf[2]) << 8) | m_subBuf[3];
+	    DDebug("RManager",DebugAll,"New screen size is %u x %u on connection %s",m_width,m_height,m_address.c_str());
+	    return;
+	default:
+	    Debug("RManager",DebugMild,"Unsupported telnet suboption %u (0x%02X)",m_subOpt,m_subOpt);
+	    return;
+    }
+    Debug("RManager",DebugMild,"Invalid content for telnet suboption %u (0x%02X)",m_subOpt,m_subOpt);
+}
+
 // process incoming terminal characters
 bool Connection::processChar(unsigned char c)
 {
@@ -886,8 +938,14 @@ bool Connection::processChar(unsigned char c)
 			writeBufferTail(true);
 			return false;
 		    //case '2': // Insert
-		    //case '5': // Page up
-		    //case '6': // Page down
+		    case '5': // Page up
+			if (pagedCommand(true))
+			    return false;
+			break;
+		    case '6': // Page down
+			if (pagedCommand(false))
+			    return false;
+			break;
 		}
 		break;
 	}
@@ -917,12 +975,46 @@ bool Connection::processChar(unsigned char c)
 	m_cursorPos++;
 	if (m_echoing) {
 	    writeStr(tmp);
-	    writeBufferTail(m_cursorPos);
+	    writeBufferTail();
 	}
     }
     return false;
 }
 
+bool Connection::pagedCommand(bool up)
+{
+    if (m_buffer || m_offset < 0)
+	return false;
+    const String* s = static_cast<const String*>(m_history.get());
+    if (!s)
+	return false;
+    if (up) {
+	if (!m_height)
+	    return false;
+	int offs = m_offset - m_height;
+	if (offs < 0)
+	    return false;
+	offs -= m_height;
+	if (offs < 0)
+	    offs = 0;
+	m_offset = offs;
+    }
+    execCommand(*s,true);
+    return true;
+}
+
+// put window size parameters in message
+void Connection::putConnInfo(NamedList& msg) const
+{
+    if (m_address)
+	msg.setParam("cmd_address",m_address);
+    msg.setParam("cmd_machine",String::boolText(m_machine));
+    if (m_width && m_height && !m_machine) {
+	msg.setParam("cmd_width",String(m_width));
+	msg.setParam("cmd_heigth",String(m_height));
+    }
+}
+
 // perform auto-completion of partial line
 bool Connection::autoComplete()
 {
@@ -1028,8 +1120,10 @@ bool Connection::autoComplete()
 	completeWords(m.retValue(),s_level,partWord);
 	break;
     }
-    if (m_auth >= Admin)
+    if (m_auth >= Admin) {
+	putConnInfo(m);
 	Engine::dispatch(m);
+    }
     if (m.retValue().null())
 	return false;
     if (m.retValue().find('\t') < 0) {
@@ -1085,6 +1179,7 @@ bool Connection::processLine(const char *line, bool saveLine)
 
     line = 0;
     m_buffer.clear();
+    m_offset = -1;
 
     if (str.startSkip("quit"))
     {
@@ -1117,6 +1212,7 @@ bool Connection::processLine(const char *line, bool saveLine)
 		}
 	    }
 	    m.addParam("line",str);
+	    putConnInfo(m);
 	    if ((m_auth >= Admin) && Engine::dispatch(m))
 		writeStr(m.retValue());
 	    else
@@ -1133,8 +1229,10 @@ bool Connection::processLine(const char *line, bool saveLine)
 		    m.retValue() << " " << info->args;
 		m.retValue() << "\r\n";
 	    }
-	    if (m_auth >= Admin)
+	    if (m_auth >= Admin) {
+		putConnInfo(m);
 		Engine::dispatch(m);
+	    }
 	    writeStr(m.retValue());
 	}
 	return false;
@@ -1186,6 +1284,7 @@ bool Connection::processLine(const char *line, bool saveLine)
 	    m.addParam("module",str);
 	    str = ":" + str;
 	}
+	putConnInfo(m);
 	Engine::dispatch(m);
 	str = "%%+status" + str + "\r\n";
 	str << m.retValue() << "%%-status\r\n";
@@ -1263,6 +1362,7 @@ bool Connection::processLine(const char *line, bool saveLine)
 	    m.addParam("id",str); 
 	if (reason)
 	    m.addParam("reason",reason);
+	putConnInfo(m);
 	if (Engine::dispatch(m))
 	    str = (m_machine ? "%%=drop:success:" : "Dropped ") + str + "\r\n";
 	else if (all)
@@ -1282,6 +1382,7 @@ bool Connection::processLine(const char *line, bool saveLine)
 	Message m("call.execute");
 	m.addParam("callto",str.substr(0,pos));
 	m.addParam((target.find('/') > 0) ? "direct" : "target",target);
+	putConnInfo(m);
 
 	if (Engine::dispatch(m)) {
 	    String id(m.getValue("id"));
@@ -1331,6 +1432,7 @@ bool Connection::processLine(const char *line, bool saveLine)
 	    m.addParam("module",str);
 	    if (l)
 		m.addParam("line",l);
+	    putConnInfo(m);
 	    if (Engine::dispatch(m))
 		writeStr(m.retValue());
 	    else
@@ -1361,6 +1463,7 @@ bool Connection::processLine(const char *line, bool saveLine)
 	Message m("chan.control");
 	m.addParam("targetid",id);
 	m.addParam("component",id);
+	m.addParam("module","rmanager");
 	static const Regexp r("^\\(.* \\)\\?\\([^= ]\\+\\)=\\([^=]*\\)$");
 	while (ctrl) {
 	    if (!ctrl.matches(r)) {
@@ -1370,11 +1473,22 @@ bool Connection::processLine(const char *line, bool saveLine)
 	    m.setParam(ctrl.matchString(2),ctrl.matchString(3).trimBlanks());
 	    ctrl = ctrl.matchString(1).trimBlanks();
 	}
+	putConnInfo(m);
 	if (Engine::dispatch(m)) {
-	    if (m_machine)
-		str = "%%=control:success:" + id + ":" + m.retValue() + "\r\n";
-	    else
-		str = "Control '" + id + "' " + m.retValue().safe("OK") + "\r\n";
+	    NamedString* opStatus = m.getParam(YSTRING("operation-status"));
+	    String* stringRet = m.getParam(YSTRING("retVal"));
+	    const String& retVal = stringRet ? *stringRet : m.retValue();
+	    if (!opStatus || opStatus->toBoolean()) {
+		if (m_machine)
+		    str = "%%=control:success:" + id + ":" + retVal + "\r\n";
+		else
+		    str = "Control '" + id + "' " + retVal.safe("OK") + "\r\n";
+	    } else {
+		if (m_machine)
+		    str = "%%=control:error:" + id + ":" + retVal + "\r\n";
+		else
+		    str = "Control '" + id + "' " + retVal.safe("FAILED") + "\r\n";
+	    }
 	}
 	else
 	    str = (m_machine ? "%%=control:fail:" : "Could not control ") + str + "\r\n";
@@ -1487,28 +1601,41 @@ bool Connection::processLine(const char *line, bool saveLine)
 		str = str.substr(sep+3);
 	    }
 	}
-	Message m("engine.command");
-	m.addParam("line",str);
-	if (Engine::dispatch(m)) {
-	    writeStr(m.retValue());
-	    const ObjList* l = YOBJECT(ObjList,m.userData());
-	    if (l)
-		l = l->skipNull();
-	    for (; l; l = l->skipNext()) {
-		const GenObject* o = l->get();
-		const CapturedEvent* ev = YOBJECT(CapturedEvent,o);
-		if (ev)
-		    writeEvent(ev->c_str(),ev->level());
-		else
-		    writeEvent(o->toString(),-1);
-	    }
-	}
-	else
-	    writeStr((m_machine ? "%%=syntax:" : "Cannot understand: ") + str + "\r\n");
+	execCommand(str,saveLine);
     }
     return false;
 }
 
+// execute a command, display output and remember any offset
+void Connection::execCommand(const String& str, bool saveOffset)
+{
+    if (str.null())
+	return;
+    Message m("engine.command");
+    m.addParam("line",str);
+    putConnInfo(m);
+    if (saveOffset && m_offset >= 0)
+	m.setParam("cmd_offset",String(m_offset));
+    if (Engine::dispatch(m)) {
+	writeStr(m.retValue());
+	const ObjList* l = YOBJECT(ObjList,m.userData());
+	if (l)
+	    l = l->skipNull();
+	for (; l; l = l->skipNext()) {
+	    const GenObject* o = l->get();
+	    const CapturedEvent* ev = YOBJECT(CapturedEvent,o);
+	    if (ev)
+		writeEvent(ev->c_str(),ev->level());
+	    else
+		writeEvent(o->toString(),-1);
+	}
+	if (saveOffset)
+	    m_offset = m.getIntValue("cmd_offset",-1);
+    }
+    else
+	writeStr((m_machine ? "%%=syntax:" : "Cannot understand: ") + str + "\r\n");
+}
+
 // dump encoded messages after processing, only in machine mode
 void Connection::writeStr(const Message &msg, bool received)
 {
