@@ -5,21 +5,18 @@
  * Cache implementation
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2011 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include <yatephone.h>
@@ -512,7 +509,7 @@ bool Cache::copyParams(const String& id, NamedList& list, const String* cpParams
 	lock();
 	const char* error = m.getValue("error");
 	if (ok && !error) {
-	    Array* a = static_cast<Array*>(m.userObject("Array"));
+	    Array* a = static_cast<Array*>(m.userObject(YATOM("Array")));
 	    int rows = a ? a->getRows() : 0;
 	    if (rows > 0)
 		item = addUnsafe(*a,1,a->getColumns());
@@ -1309,7 +1306,7 @@ void CacheModule::loadCache(const String& name, bool async, ObjList* items)
 	    Debug(this,DebugInfo,"Cache '%s' vanished while loading",name.c_str());
 	    break;
 	}
-	Array* a = static_cast<Array*>(m.userObject("Array"));
+	Array* a = static_cast<Array*>(m.userObject(YATOM("Array")));
 	int rows = a ? a->getRows() : 0;
 	unsigned int loadedRows = (rows > 0) ? rows - 1 : 0;
 	if (!items)
