@@ -5,21 +5,18 @@
  * Yet Another Signalling Stack - implements the support for SS7
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2009 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include <yatephone.h>
@@ -1708,10 +1705,10 @@ bool SLT::control(Operation oper, NamedList* params)
 	case Pause:
 	    setReqStatus(OutOfService);
 	    sendManagement(Disconnect_R);
-	    return true;
+	    return TelEngine::controlReturn(params,true);
 	case Resume:
 	    if (aligned() || !m_autostart)
-		return true;
+		return TelEngine::controlReturn(params,true);
 	    // fall through
 	case Align:
 	    {
@@ -1727,9 +1724,9 @@ bool SLT::control(Operation oper, NamedList* params)
 			configure(true);
 		}
 	    }
-	    return true;
+	    return TelEngine::controlReturn(params,true);
 	case Status:
-	    return aligned() && m_status == Configured;
+	    return TelEngine::controlReturn(params,aligned() && m_status == Configured);
 	default:
 	    return false;//SignallingReceiver::control((SignallingInterface::Operation)oper,params);
     }
