@@ -5,21 +5,18 @@
  * Monitor CPU load and inform YATE about it
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2010 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include <yatengine.h>
@@ -170,7 +167,7 @@ public:
     void initialize(const Configuration& params);
     inline void requestExit()
 	{ m_exit = true; }
-    bool update(const Message& msg);
+    bool update(Message& msg);
     inline Cpu* getCpu()
 	{ return m_cpu; }
 private:
@@ -279,35 +276,41 @@ void CpuUpdater::setCpu(Cpu* cpu)
 }
 
 // This method appends a target from chan.control message
-bool CpuUpdater::update(const Message& msg)
+bool CpuUpdater::update(Message& msg)
 {
-    String mon = msg.getValue("operation","");
+    const String& mon = msg[YSTRING("operation")];
     NamedString* inc = 0;
     for (unsigned int i = 0;i < msg.count();i++) {
 	NamedString* ns = msg.getParam(i);
-	if (ns->name() == "operation" || ns->name() == "component" ||
-		ns->name() == "targetid")
+	if (!ns->name().startsWith("cpu.") || ns->name().length() <= 4)
 	    continue;
-	inc = ns;
+	inc = new NamedString(ns->name().substr(4),*ns);
+	break;
     }
     if (!inc) {
 	DDebug(&s_module,DebugNote,"No target parameter for monitor %s",mon.c_str());
-	return false;
+	return TelEngine::controlReturn(&msg,true);
     }
     Lock lock(this);
+    bool ret = false;
     switch (lookup(mon,s_monitors,Unknown)) {
 	case YateUser:
-	    return m_yateUser.addTarget(*inc,m_oscillationTimer);
+	    ret = m_yateUser.addTarget(*inc,m_oscillationTimer);
+	    break;
 	case YateKernel:
-	    return m_yateSys.addTarget(*inc,m_oscillationTimer);
+	    ret = m_yateSys.addTarget(*inc,m_oscillationTimer);
+	    break;
 	case YateTotal:
-	    return m_yateTotal.addTarget(*inc,m_oscillationTimer);
+	    ret = m_yateTotal.addTarget(*inc,m_oscillationTimer);
+	    break;
 	case System:
-	    return m_system.addTarget(*inc,m_oscillationTimer);
+	    ret = m_system.addTarget(*inc,m_oscillationTimer);
+	    break;
 	default:
 	    Debug(&s_module,DebugNote,"Unknown cpu monitor %s",mon.c_str());
     }
-    return false;
+    TelEngine::destruct(inc);
+    return TelEngine::controlReturn(&msg,ret);
 }
 
 void CpuUpdater::initialize(const Configuration& params)
