@@ -5,21 +5,18 @@
  * Wave record+playback helper for database storage, uses wavefile
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include <yatephone.h>
@@ -90,9 +87,9 @@ static void alterSource(Message& msg, const String& name)
     Message m("database");
     m.addParam("account",account);
     m.addParam("query",query);
-    if (!Engine::dispatch(m) || (m.getIntValue("rows") != 1) || (m.getIntValue("columns") != 1))
+    if (!Engine::dispatch(m) || (m.getIntValue(YSTRING("rows")) != 1) || (m.getIntValue(YSTRING("columns")) != 1))
 	return;
-    Array* a = static_cast<Array*>(m.userObject("Array"));
+    Array* a = static_cast<Array*>(m.userObject(YATOM("Array")));
     if (!a)
 	return;
     GenObject* obj = a->take(0,1);
@@ -179,7 +176,7 @@ DbWriter::~DbWriter()
 
 void* DbWriter::getObject(const String& name) const
 {
-    if (name == "Stream")
+    if (name == YATOM("Stream"))
 	return static_cast<Stream*>(const_cast<DbWriter*>(this));
     return GenObject::getObject(name);
 }
