@@ -5,21 +5,18 @@
  * Last chance routing in call.execute and msg.execute messages.
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include <yatengine.h>
@@ -38,10 +35,10 @@ public:
 class LateHandler : public MessageHandler
 {
 public:
-    LateHandler(const char* name, unsigned priority);
+    LateHandler(const char* name, unsigned priority, const char* routeType);
     virtual bool received(Message& msg);
 protected:
-    String m_routeMsg;
+    String m_routeType;
 };
 
 static Regexp s_regexp;
@@ -49,6 +46,7 @@ static String s_called;
 static Mutex s_mutex(false,"LateRoute");
 static LateHandler* s_callHandler = 0;
 static LateHandler* s_msgHandler = 0;
+static const String s_routeTypeParam = "route_type";
 
 INIT_PLUGIN(LateRouter);
 
@@ -65,12 +63,10 @@ UNLOAD_PLUGIN(unloadNow)
 }
 
 
-LateHandler::LateHandler(const char* name, unsigned priority)
-    : MessageHandler(name,priority,__plugin.name())
+LateHandler::LateHandler(const char* name, unsigned priority, const char* routeType)
+    : MessageHandler(name,priority,__plugin.name()),
+    m_routeType(routeType)
 {
-    // build a proper routing message name
-    int sep = find('.');
-    m_routeMsg = substr(0,sep) + ".route";
 }
 
 bool LateHandler::received(Message& msg)
@@ -83,6 +79,7 @@ bool LateHandler::received(Message& msg)
 	return false;
     String callto = dest;
     dest = dest.replaceMatches(s_called);
+    lock.drop();
     msg.replaceParams(dest);
     if (dest.trimBlanks().null())
 	return false;
@@ -91,7 +88,9 @@ bool LateHandler::received(Message& msg)
     msg.clearParam("callto");
     msg.setParam("called",dest);
     // change the message name to the routing one
-    msg = m_routeMsg;
+    msg = "call.route";
+    if (!msg.getParam(s_routeTypeParam))
+	msg.addParam(s_routeTypeParam,m_routeType);
     bool ok = Engine::dispatch(msg);
     dest = msg.retValue();
     msg.retValue().clear();
@@ -99,7 +98,8 @@ bool LateHandler::received(Message& msg)
     msg = toString();
     ok = ok && dest && (dest != "-") && (dest != "error");
     if (ok && (dest == callto)) {
-	Debug(DebugMild,"Call to '%s' late routed back to itself!",callto.c_str());
+	Debug(DebugMild,"%s to '%s' late routed back to itself!",
+	   msg.getValue(s_routeTypeParam,m_routeType),callto.c_str());
 	ok = false;
     }
     if (!ok) {
@@ -108,7 +108,8 @@ bool LateHandler::received(Message& msg)
 	msg.setParam("callto",callto);
 	return false;
     }
-    Debug(DebugInfo,"Late routing call to '%s' via '%s'",callto.c_str(),dest.c_str());
+    Debug(DebugInfo,"Late routing %s to '%s' via '%s'",
+	msg.getValue(s_routeTypeParam,m_routeType),callto.c_str(),dest.c_str());
     // let it pass through to the new target
     msg.setParam("callto",dest);
     return false;
@@ -162,8 +163,10 @@ void LateRouter::initialize()
     DDebug("lateroute",DebugInfo,"regexp='%s' called='%s'",s_regexp.c_str(),s_called.c_str());
     if (!s_callHandler && cfg.getBoolValue("general","enabled",(s_regexp && s_called))) {
 	int priority = cfg.getIntValue("general","priority",75);
-	s_callHandler = new LateHandler("call.execute",priority);
-	s_msgHandler = new LateHandler("msg.execute",priority);
+	s_callHandler = new LateHandler("call.execute",priority,
+	    cfg.getValue("route type","call.execute","call"));
+	s_msgHandler = new LateHandler("msg.execute",priority,
+	    cfg.getValue("route type","msg.execute","msg"));
 	Engine::install(s_callHandler);
 	Engine::install(s_msgHandler);
     }
