@@ -6,21 +6,18 @@
  *  for other protocols
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 
@@ -899,7 +896,7 @@ MGCPSpan* MGCPSpan::findNotify(const String& id, const String& epId)
 
 MGCPSpan::MGCPSpan(const NamedList& params, const char* name, const MGCPEpInfo& ep)
     : SignallingCircuitSpan(params.getValue("debugname",name),
-	static_cast<SignallingCircuitGroup*>(params.getObject("SignallingCircuitGroup"))),
+	static_cast<SignallingCircuitGroup*>(params.getObject(YATOM("SignallingCircuitGroup")))),
       m_circuits(0), m_count(0), m_epId(ep), m_operational(false),
       m_rtpForward(false), m_sdpForward(false), m_rtpForcedFwd(false), m_fxo(false), m_fxs(false),
       m_ntfyMatch(true), m_rqntEmbed(true), m_rqntCheck(true), m_rqntType(RqntOnce)
@@ -1384,12 +1381,12 @@ MGCPCircuit::~MGCPCircuit()
 void* MGCPCircuit::getObject(const String& name) const
 {
     if (connected()) {
-	if (name == YSTRING("DataSource"))
+	if (name == YATOM("DataSource"))
 	    return m_source;
-	if (name == YSTRING("DataConsumer"))
+	if (name == YATOM("DataConsumer"))
 	    return m_consumer;
     }
-    if (name == YSTRING("MGCPCircuit"))
+    if (name == YATOM("MGCPCircuit"))
 	return (void*)this;
     return SignallingCircuit::getObject(name);
 }
