@@ -5,21 +5,18 @@
  * This is the MySQL support from Yate.
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include <yatephone.h>
@@ -552,13 +549,13 @@ bool MyAcct::initDb()
 
     int initCons = initConns();
     if (!initCons) {
-	Debug(&module,DebugWarn,"Could not initiate any connections for account '%s', trying again in %d seconds",
+	Alarm(&module,"database",DebugWarn,"Could not initiate any connections for account '%s', trying again in %d seconds",
 	    c_str(),m_retryTime);
 	module.startInitThread();
 	return true;
     }
     if (initCons != m_poolSize) {
-	Debug(&module,DebugMild,"Could initiate only %d of %d connections for account '%s', trying again in %d seconds",
+	Alarm(&module,"database",DebugMild,"Could initiate only %d of %d connections for account '%s', trying again in %d seconds",
 	    initCons,m_poolSize,c_str(),m_retryTime);
 	module.startInitThread();
     }
