@@ -5,22 +5,19 @@
  * PBX assist module
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  * Author: Monica Tepelus
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include <yatepbx.h>
@@ -195,7 +192,7 @@ PBXList::~PBXList()
 
 ChanAssist* PBXList::create(Message& msg, const String& id)
 {
-    if (msg == "chan.startup" || msg.userObject("Channel")) {
+    if (msg == "chan.startup" || msg.userObject(YATOM("Channel"))) {
 	// if a filter is set try to match it
 	if (s_filter && (s_filterFail == s_filter.matches(id)))
 	    return 0;
@@ -424,7 +421,7 @@ bool PBXAssist::msgDisconnect(Message& msg, const String& reason)
     if ((reason == "hold") || (reason == "park") || (reason == "intrusion")) {
 	String onhold = m_keep.getValue("onhold",s_onhold);
 	if (onhold) {
-	    Channel* c = static_cast<Channel*>(msg.userObject("Channel"));
+	    Channel* c = static_cast<Channel*>(msg.userObject(YATOM("Channel")));
 	    if (!c)
 		return false;
 	    Message *m = c->message("call.execute",false,true);
@@ -534,7 +531,7 @@ bool PBXAssist::msgDisconnect(Message& msg, const String& reason)
     }
 
     if (!m_guest && (state() != "new")) {
-	Channel* c = static_cast<Channel*>(msg.userObject("Channel"));
+	Channel* c = static_cast<Channel*>(msg.userObject(YATOM("Channel")));
 	if (!c)
 	    return false;
 	Message *m = c->message("call.execute",false,true);
@@ -874,7 +871,7 @@ bool PBXAssist::operConference(Message& msg)
 	    return errorBeep("conference failed");
     }
     else {
-	Channel* c = static_cast<Channel*>(msg.userObject("Channel"));
+	Channel* c = static_cast<Channel*>(msg.userObject(YATOM("Channel")));
 	if (!c)
 	    return errorBeep("no channel");
 	if (!room)
@@ -1006,7 +1003,7 @@ bool PBXAssist::operReturnConf(Message& msg)
 {
     if ((state() == "conference") || (state() == "new") || m_room.null())
 	return errorBeep("cannot return to conference");
-    Channel* c = static_cast<Channel*>(msg.userObject("Channel"));
+    Channel* c = static_cast<Channel*>(msg.userObject(YATOM("Channel")));
     if (!c)
 	return errorBeep("no channel");
     bool owner = msg.getBoolValue("confowner",m_keep.getBoolValue("pbxconfowner",s_confOwner));
