@@ -5,21 +5,18 @@
  * Call distribution and queues with settings from a database.
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include <yatephone.h>
@@ -222,8 +219,8 @@ CallsQueue* CallsQueue::create(const char* name, const NamedList& params)
 	Debug(&__plugin,DebugWarn,"Query on '%s' failed: '%s'",s_account.c_str(),query.c_str());
 	return 0;
     }
-    Array* res = static_cast<Array*>(m.userObject("Array"));
-    if (!res || (m.getIntValue("rows") != 1)) {
+    Array* res = static_cast<Array*>(m.userObject(YATOM("Array")));
+    if (!res || (m.getIntValue(YSTRING("rows")) != 1)) {
 	Debug(&__plugin,DebugWarn,"Missing queue '%s'",name);
 	return 0;
     }
@@ -442,14 +439,14 @@ void CallsQueue::startACD()
 	Debug(&__plugin,DebugWarn,"Query on '%s' failed: '%s'",s_account.c_str(),query.c_str());
 	return;
     }
-    Array* res = static_cast<Array*>(msg.userObject("Array"));
-    if (!res || (msg.getIntValue("rows") < 1))
+    Array* res = static_cast<Array*>(msg.userObject(YATOM("Array")));
+    if (!res || (msg.getIntValue(YSTRING("rows")) < 1))
 	return;
     for (int i = 1; i < res->getRows(); i++) {
 	NamedList params("");
 	copyArrayParams(params,res,i);
-	const char* callto = params.getValue("location");
-	const char* user = params.getValue("username");
+	const char* callto = params.getValue(YSTRING("location"));
+	const char* user = params.getValue(YSTRING("username"));
 	if (!(callto && user))
 	    continue;
 	QueuedCall* call = markCall(user);
