@@ -5,21 +5,18 @@
  * Registration, authentication, authorization and accounting from a database.
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include <yatephone.h>
@@ -86,6 +83,7 @@ protected:
     String m_name;
     String m_queryInitialize;
     String m_queryUpdate;
+    String m_queryCombined;
     bool m_critical;
 };
 
@@ -292,7 +290,7 @@ static bool copyParams(Message &msg, Array *a, const String& resultName)
 		m->retValue() = *s;
 	    else
 		m->setParam(*name,*s);
-	}	
+	}
 	if (j>1) {
 	    if (m->retValue().null()) {
 		Debug(&module,DebugWarn,"Skipping void route #%d",j);
@@ -363,9 +361,9 @@ void AAAHandler::indirectQuery(String& query)
     prepareQuery(m,m_account,query,true);
     query.clear();
     // query must return exactly one row, one column
-    if (!Engine::dispatch(m) || (m.getIntValue("rows") != 1) || (m.getIntValue("columns") != 1))
+    if (!Engine::dispatch(m) || (m.getIntValue(YSTRING("rows")) != 1) || (m.getIntValue(YSTRING("columns")) != 1))
 	return;
-    Array* a = static_cast<Array*>(m.userObject("Array"));
+    Array* a = static_cast<Array*>(m.userObject(YATOM("Array")));
     if (!a)
 	return;
     query = YOBJECT(String,a->get(0,1));
@@ -401,7 +399,7 @@ void AAAHandler::initQuery()
 void AAAHandler::chkConfig()
 {
     if (m_query && m_account.null())
-	Debug(&module,DebugMild,"Missing database account for '%s'",name().c_str());
+	Alarm(&module,"config",DebugMild,"Missing database account for '%s'",name().c_str());
 }
 
 // little helper function to make code cleaner
@@ -448,7 +446,7 @@ bool AAAHandler::received(Message& msg)
 	    if (Engine::dispatch(m))
 		if (m.getIntValue("rows") >=1)
 		{
-		    Array* a = static_cast<Array*>(m.userObject("Array"));
+		    Array* a = static_cast<Array*>(m.userObject(YATOM("Array")));
 		    if (!copyParams(msg,a,m_result)) {
 			Debug(&module,DebugWarn,"Misconfigured result column for '%s'",name().c_str());
 			msg.setParam("error","failure");
@@ -470,7 +468,7 @@ bool AAAHandler::received(Message& msg)
 	    if (Engine::dispatch(m))
 		if (m.getIntValue("rows") >=1)
 		{
-		    Array* a = static_cast<Array*>(m.userObject("Array"));
+		    Array* a = static_cast<Array*>(m.userObject(YATOM("Array")));
 		    copyParams(msg,a,m_result);
 		}
 	    return false;
@@ -487,7 +485,7 @@ bool AAAHandler::received(Message& msg)
 	    if (Engine::dispatch(m))
 		if (m.getIntValue("rows") >=1)
 		{
-		    Array* a = static_cast<Array*>(m.userObject("Array"));
+		    Array* a = static_cast<Array*>(m.userObject(YATOM("Array")));
 		    copyParams(msg,a,m_result);
 		    if (msg.retValue().null())
 		    {
@@ -552,13 +550,15 @@ bool CDRHandler::loadQuery()
 {
     m_queryInitialize = s_cfg.getValue(name(),"cdr_initialize");
     m_queryUpdate = s_cfg.getValue(name(),"cdr_update");
+    m_queryCombined = s_cfg.getValue(name(),"cdr_combined");
     m_query = s_cfg.getValue(name(),"cdr_finalize");
     if (m_query.null())
 	m_query = s_cfg.getValue(name(),"query");
     indirectQuery(m_queryInitialize);
     indirectQuery(m_queryUpdate);
+    indirectQuery(m_queryCombined);
     indirectQuery(m_query);
-    return m_queryInitialize || m_queryUpdate || m_query;
+    return m_queryInitialize || m_queryUpdate || m_queryCombined || m_query;
 }
 
 bool CDRHandler::received(Message& msg)
@@ -571,11 +571,13 @@ bool CDRHandler::received(Message& msg)
     if (!msg.getBoolValue("cdrwrite",true))
 	return false;
     String query(msg.getValue("operation"));
-    if (query == "initialize")
+    if (query == YSTRING("initialize"))
 	query = m_queryInitialize;
-    else if (query == "update")
+    else if (query == YSTRING("update"))
 	query = m_queryUpdate;
-    else if (query == "finalize")
+    else if (query == YSTRING("combined"))
+	query = m_queryCombined;
+    else if (query == YSTRING("finalize"))
 	query = m_query;
     else
 	return false;
@@ -640,7 +642,7 @@ Array* EventNotify::queryDatabase(Message& msg, const String& notifier,
     if (!Engine::dispatch(msg))
 	return 0;
     rows = msg.getIntValue("rows",0);
-    Array* subscriptions = static_cast<Array*>(msg.userObject("Array"));
+    Array* subscriptions = static_cast<Array*>(msg.userObject(YATOM("Array")));
     if (!(subscriptions && rows))
 	return 0;
     DDebug(&module,DebugAll,
@@ -822,7 +824,7 @@ bool SubscribeHandler::received(Message& msg)
     }
 
     Message* notify = new Message("resource.notify");
-    Array* a = static_cast<Array*>(m.userObject("Array"));
+    Array* a = static_cast<Array*>(m.userObject(YATOM("Array")));
     if (subscribe) {
 	copyParams2(*notify,a,1);
 	notify->addParam("subscriptionstate","active");
@@ -879,7 +881,7 @@ bool SubscribeTimerHandler::received(Message& msg)
 	m_nextTime = t + m_expireTime;
     else
 	return false;
-	
+
     if(!m_queryExpire)
 	return false;
 
@@ -893,7 +895,7 @@ bool SubscribeTimerHandler::received(Message& msg)
 	return false;
 
     int rows = m.getIntValue("rows",0);
-    Array* a = static_cast<Array*>(m.userObject("Array"));
+    Array* a = static_cast<Array*>(m.userObject(YATOM("Array")));
     if(!a || rows < 1)
 	return false;
     for(int i = 1; i <= rows; i++) {
@@ -1163,7 +1165,7 @@ bool AccountsModule::received(Message &msg, int id)
 	    if (rows>0) {
 		for (int i=1 ; i<=rows ; i++) {
 		    Message *m1= new Message("user.login");
-		    Array* a = static_cast<Array*>(m.userObject("Array"));
+		    Array* a = static_cast<Array*>(m.userObject(YATOM("Array")));
 		    copyParams2(*m1,a, i);
 		    Engine::enqueue(m1);
 		} 
