@@ -5,21 +5,18 @@
  * SIGTRAN transports provider, supports SCTP, TCP, UDP, UNIX sockets
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2009-2010 Null Team
+ * Copyright (C) 2009-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include <stdlib.h>
@@ -32,6 +29,8 @@
 
 #define CONN_RETRY_MIN   250000
 #define CONN_RETRY_MAX 60000000
+#define DECREASE_INTERVAL 1000000
+#define DECREASE_AMOUNT    250000
 
 using namespace TelEngine;
 namespace { // anonymous
@@ -140,7 +139,7 @@ public:
     inline TReader()
 	: Mutex(true,"TReader"),
 	  m_sending(true,"TReader::sending"), m_canSend(true), m_reconnect(false),
-	  m_tryAgain(0), m_interval(CONN_RETRY_MIN), m_downTime(0)
+	  m_tryAgain(0), m_interval(CONN_RETRY_MIN), m_downTime(0), m_decrease(0)
 	{ }
     virtual ~TReader();
     virtual void listen(int maxConn) = 0;
@@ -157,6 +156,7 @@ protected:
     u_int64_t m_tryAgain;
     u_int32_t m_interval;
     u_int64_t m_downTime;
+    u_int64_t m_decrease;
 };
 
 class Transport : public SIGTransport
@@ -203,7 +203,7 @@ public:
     bool addSocket(Socket* socket,SocketAddr& adress);
     virtual bool reliable() const
 	{ return m_type == Sctp || m_type == Tcp; }
-    virtual bool control(const NamedList &param);
+    virtual bool control(NamedList &param);
     virtual bool connected(int id) const
 	{ return m_state == Up;}
     virtual void attached(bool ual)
@@ -662,6 +662,7 @@ Transport::~Transport()
 
 void Transport::destroyed()
 {
+    SignallingComponent::destroyed();
     m_readerMutex.lock();
     TReader* tmp = m_reader;
     m_reader = 0;
@@ -683,22 +684,29 @@ void Transport::resetReader(TReader* caller)
 	TelEngine::destruct(caller);
 }
 
-bool Transport::control(const NamedList &param)
+bool Transport::control(NamedList& param)
 {
+    String cmp = param.getValue(YSTRING("component"));
+    if (!cmp)
+	return false;
+    if (cmp.startsWith(YSTRING("Transport:")) && cmp != toString())
+	return false;
+    if (toString() != String("Transport:" + cmp))
+	return false;
     String oper = param.getValue(YSTRING("operation"),YSTRING("init"));
     if (oper == YSTRING("init"))
-	return initialize(&param);
+	return TelEngine::controlReturn(&param,initialize(&param));
     else if (oper == YSTRING("add_addr")) {
 	if (!m_listener) {
 	    Debug(this,DebugWarn,"Unable to listen on another address, listener is missing");
-	    return false;
+	    return TelEngine::controlReturn(&param,false);
 	}
-	return m_listener->addAddress(param);
+	return TelEngine::controlReturn(&param,m_listener->addAddress(param));
     } else if (oper == YSTRING("reconnect")) {
 	reconnect(true);
-	return true;
+	return TelEngine::controlReturn(&param,true);
     }
-    return false;
+    return TelEngine::controlReturn(&param,false);
 }
 
 void Transport::reconnect(bool force)
@@ -1004,6 +1012,7 @@ bool Transport::connectSocket()
 	return false;
     }
     m_reader->setSocket(sock);
+    mylock.drop();
     setStatus(Up);
     return true;
 }
@@ -1247,7 +1256,7 @@ bool StreamReader::sendBuffer(int streamId)
 bool StreamReader::connectSocket()
 {
     Time t;
-    if (t < m_tryAgain && !m_reconnect) {
+    if (t.usec() < m_tryAgain && !m_reconnect) {
 	Thread::yield(true);
 	return false;
     }
@@ -1256,16 +1265,14 @@ bool StreamReader::connectSocket()
 	m_interval = CONN_RETRY_MIN;
     }
     m_tryAgain = t + m_interval;
-    if (m_transport->connectSocket()) {
-	m_interval = CONN_RETRY_MIN;
-	m_tryAgain = 0;
-	return true;
-    }
-    m_tryAgain = Time::now() + m_interval;
     // exponential backoff
     m_interval *= 2;
     if (m_interval > CONN_RETRY_MAX)
 	m_interval = CONN_RETRY_MAX;
+    if (m_transport->connectSocket()) {
+	m_decrease = t.usec() + DECREASE_INTERVAL;
+	return true;
+    }
     return false;
 }
 
@@ -1280,6 +1287,16 @@ bool StreamReader::readData()
 	stopThread();
 	return false;
     }
+    if (m_interval > CONN_RETRY_MIN) {
+	Time t;
+	if (t.usec() > m_decrease) {
+	    if (((int64_t)m_interval - DECREASE_AMOUNT) > CONN_RETRY_MIN)
+		m_interval -= DECREASE_AMOUNT;
+	    else
+		m_interval = CONN_RETRY_MIN;
+	    m_decrease = t.usec() + DECREASE_INTERVAL;
+	}
+    }
     sendBuffer();
     if (!m_socket)
 	return false;
