@@ -5,21 +5,18 @@
  * Subscription handler and presence notifier
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 // TODO:
@@ -500,8 +497,8 @@ public:
     bool handleUserRosterDelete(const String& user, const String& contact, Message& msg);
     // Handle 'user.update' messages with operation 'delete'
     void handleUserUpdateDelete(const String& user, Message& msg);
-    // Handle 'msg.route' messages
-    bool imRoute(Message& msg);
+    // Handle 'call.route' messages
+    bool imRoute(Message& msg, const String& type);
     void expireSubscriptions();
     // Build a database message from account and query.
     // Replace query params. Return Message pointer on success
@@ -521,6 +518,7 @@ public:
     String m_contactSetFullQuery;
     String m_contactDeleteQuery;
     String m_genericUserLoadQuery;
+    String m_routeCallto;
     UserList m_users;
     Mutex m_eventsMutex;
     ObjList m_events;
@@ -1383,7 +1381,7 @@ PresenceUser* UserList::askDatabase(const String& name)
     u_int64_t start = Time::now();
 #endif
     PresenceUser* u = 0;
-    Array* a = static_cast<Array*>(m->userObject("Array"));
+    Array* a = static_cast<Array*>(m->userObject(YATOM("Array")));
     int rows = 0;
     int cols = 0;
     ObjList** columns = 0;
@@ -1473,7 +1471,7 @@ void GenericUserList::load()
     clear();
     if (!m)
 	return;
-    Array* a = static_cast<Array*>(m->userObject("Array"));
+    Array* a = static_cast<Array*>(m->userObject(YATOM("Array")));
     if (!a) {
 	TelEngine::destruct(m);
 	return;
@@ -1651,7 +1649,7 @@ bool SubMessageHandler::received(Message& msg)
 		u_int64_t start = Time::now();
 		unsigned int n = 0;
 		unsigned int nc = 0;
-		Array* a = static_cast<Array*>(m->userObject("Array"));
+		Array* a = static_cast<Array*>(m->userObject(YATOM("Array")));
 		ObjList** columns = 0;
 		String** titles = 0;
 		int rows = 0;
@@ -1690,7 +1688,7 @@ bool SubMessageHandler::received(Message& msg)
 		    n,nc,ellapsedMs(start));
 	    }
 	    else
-		Debug(&__plugin,DebugMild,"Failed to load users");
+		Alarm(&__plugin,"database",DebugMild,"Failed to load users");
 	}
 	__plugin.m_genericUsers.load();
 	return false;
@@ -1732,13 +1730,12 @@ SubscriptionModule::~SubscriptionModule()
 void SubscriptionModule::initialize()
 {
     Output("Initializing module Subscriptions");
-
+    Configuration cfg(Engine::configFile("subscription"));
     if (m_handlers.skipNull()) {
 	// Reload generic users (wait engine.start for the first load)
 	m_genericUsers.load();
     }
     else {
-	Configuration cfg(Engine::configFile("subscription"));
 	m_account = cfg.getValue("general","account");
 	m_userLoadQuery = cfg.getValue("general","user_roster_load");
 	m_userEventQuery = cfg.getValue("general","user_event_auth");
@@ -1756,7 +1753,7 @@ void SubscriptionModule::initialize()
 	// Install relays
 	setup();
 	installRelay(Halt);
-	installRelay(ImRoute);
+	installRelay(Route,cfg.getIntValue("priorities","call.route",100));
 	// Install handlers
 	for (const TokenDict* d = s_msgHandler; d->token; d++) {
 	    if (d->value == SubMessageHandler::CallCdr && !m_userEventQuery)
@@ -1766,6 +1763,10 @@ void SubscriptionModule::initialize()
 	    m_handlers.append(h);
 	}
     }
+    Lock lck(this);
+    m_routeCallto = cfg.getValue("general","route_callto","jabber/${called}");
+    if (!m_routeCallto)
+	Debug(this,DebugConf,"Empty 'route_callto' in config");
 }
 
 // Enqueue a resource.notify for a given instance
@@ -1845,7 +1846,7 @@ Array* SubscriptionModule::notifyRosterUpdate(const char* username, const char*
     m = queryDb(m);
     Array* data = 0;
     if (m && m->getIntValue("rows") >= 1) {
-	data = static_cast<Array*>(m->userObject("Array"));
+	data = static_cast<Array*>(m->userObject(YATOM("Array")));
 	if (data && data->ref())
 	    m->userData(0);
 	else
@@ -2557,8 +2558,8 @@ bool SubscriptionModule::handleUserRosterQuery(const String& user, const String*
     m = queryDb(m);
     if (!m)
 	return false;
-    bool hierarchical = msg.getBoolValue("hierarchical");
-    Array* a = static_cast<Array*>(m->userObject("Array"));
+    bool hierarchical = msg.getBoolValue(YSTRING("hierarchical"));
+    Array* a = static_cast<Array*>(m->userObject(YATOM("Array")));
     int rows = 0;
     int cols = 0;
     unsigned int n = 0;
@@ -2791,8 +2792,8 @@ void SubscriptionModule::handleUserUpdateDelete(const String& user, Message& msg
     TelEngine::destruct(queryDb(m));
 }
 
-// Handle 'msg.route' messages
-bool SubscriptionModule::imRoute(Message& msg)
+// Handle 'call.route' messages
+bool SubscriptionModule::imRoute(Message& msg, const String& type)
 {
     String* caller = msg.getParam("caller");
     String* called = msg.getParam("called");
@@ -2802,16 +2803,30 @@ bool SubscriptionModule::imRoute(Message& msg)
 	msg.c_str(),caller->c_str(),called->c_str());
     PresenceUser* u = m_users.getUser(*called);
     if (!u) {
-	Debug(this,DebugStub,"%s caller=%s called=%s destination is an unknown user",
-	    msg.c_str(),caller->c_str(),called->c_str());
+	XDebug(this,DebugAll,
+	    "%s '%s' caller=%s called=%s destination is an unknown user",
+	    msg.c_str(),type.c_str(),caller->c_str(),called->c_str());
 	return false;
     }
     bool auth = msg.getBoolValue("auth");
     bool ok = true;
     unsigned int n = 0;
-    u->lock();
     String* tmp = msg.getParam("called_instance");
-    if (TelEngine::null(tmp)) {
+    bool haveInst = !TelEngine::null(tmp);
+    u->lock();
+    // An instance was given
+    if (haveInst) {
+	if (!auth || u->findContact(*caller) || *caller == *called) {
+	    Instance* inst = u->instances().findInstance(*tmp);
+	    if (inst)
+		inst->addListParam(msg,++n);
+	}
+	else
+	    ok = false;
+    }
+    // No instance given or requested to fallback to online instances
+    if (ok && !n &&
+	(!haveInst || msg.getBoolValue(YSTRING("fallback_online_instances")))) {
 	String* skip = 0;
 	if (*caller == *called)
 	    skip = msg.getParam("caller_instance");
@@ -2820,18 +2835,22 @@ bool SubscriptionModule::imRoute(Message& msg)
 	if (ok)
 	    n = u->instances().addListParam(msg,skip);
     }
-    else if (!auth || u->findContact(*caller) || *caller == *called) {
-	Instance* inst = u->instances().findInstance(*tmp);
-	if (inst)
-	    inst->addListParam(msg,++n);
-    }
-    else
-	ok = false;
     u->unlock();
     TelEngine::destruct(u);
-    if (ok)
-	msg.addParam("instance.count",String(n));
-    return ok && n != 0;
+    if (!ok)
+	return false;
+    msg.addParam("instance.count",String(n));
+    if (n) {
+	lock();
+	msg.retValue() = m_routeCallto;
+	unlock();
+	msg.replaceParams(msg.retValue());
+	if (!msg.retValue())
+	    return false;
+	Debug(this,DebugAll,"Routing '%s' caller=%s called=%s to '%s' instances=%u",
+	    type.c_str(),caller->c_str(),called->c_str(),msg.retValue().c_str(),n);
+    }
+    return n != 0;
 }
 
 void SubscriptionModule::expireSubscriptions()
@@ -2919,8 +2938,13 @@ bool SubscriptionModule::received(Message& msg, int id)
 	case Timer:
 	    s_check = true;
 	    break;
-	case ImRoute:
-	    return imRoute(msg);
+	case Route:
+	    {
+		if (!m_routeCallto)
+		    return false;
+		String* t = msg.getParam(YSTRING("route_type"));
+		return t && (*t == YSTRING("msg")) && imRoute(msg,*t);
+	    }
 	case Halt:
 	    Lock lock(this);
 	    if (m_expire)
