@@ -5,21 +5,18 @@
  * Wanpipe PRI cards signalling and data driver
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include <yatephone.h>
@@ -1035,13 +1032,13 @@ bool WpInterface::control(Operation oper, NamedList* params)
 	case EnableTx:
 	case DisableTx:
 	    if (m_readOnly == (oper == DisableTx))
-		return true;
+		return TelEngine::controlReturn(params,true);
 	    m_readOnly = (oper == DisableTx);
 	    m_sendReadOnly = false;
 	    Debug(this,DebugInfo,"Tx is %sabled [%p]",m_readOnly?"dis":"en",this);
-	    return true;
+	    return TelEngine::controlReturn(params,true);
 	case Query:
-	    return m_socket.valid() && m_thread && m_thread->running();
+	    return TelEngine::controlReturn(params,m_socket.valid() && m_thread && m_thread->running());
 	default:
 	    return SignallingInterface::control(oper,params);
     }
@@ -1063,7 +1060,7 @@ bool WpInterface::control(Operation oper, NamedList* params)
 	    Debug(this,DebugWarn,"Enable failed [%p]",this);
 	    control(Disable,0);
 	}
-	return ok;
+	return TelEngine::controlReturn(params,ok);
     }
     // oper is Disable
     m_timerRxUnder.stop();
@@ -1074,7 +1071,7 @@ bool WpInterface::control(Operation oper, NamedList* params)
     }
     m_socket.close();
     DDebug(this,DebugAll,"Disabled [%p]",this);
-    return true;
+    return TelEngine::controlReturn(params,true);
 }
 
 // Update link status. Notify the receiver if state changed
