@@ -5,21 +5,18 @@
  * Tones generator
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include <yatephone.h>
@@ -1010,11 +1007,11 @@ DataBlock* getRawData(Message& msg)
     NamedString* data = msg.getParam("rawdata");
     if (!data)
 	return 0;
-    NamedPointer* p = static_cast<NamedPointer*>(data->getObject("NamedPointer"));
+    NamedPointer* p = static_cast<NamedPointer*>(data->getObject(YATOM("NamedPointer")));
     if (!p)
 	return 0;
     GenObject* gen = p->userData();
-    if (!(gen && gen->getObject("DataBlock")))
+    if (!(gen && gen->getObject(YATOM("DataBlock"))))
 	return 0;
     return static_cast<DataBlock*>(p->takeData());
 }
@@ -1033,9 +1030,9 @@ bool AttachHandler::received(Message& msg)
     if (src.null() && ovr.null() && repl.null())
 	return false;
 
-    RefPointer<DataEndpoint> de = static_cast<DataEndpoint*>(msg.userObject("DataEndpoint"));
+    RefPointer<DataEndpoint> de = static_cast<DataEndpoint*>(msg.userObject(YATOM("DataEndpoint")));
     if (!de) {
-	CallEndpoint* ch = static_cast<CallEndpoint*>(msg.userObject("CallEndpoint"));
+	CallEndpoint* ch = static_cast<CallEndpoint*>(msg.userObject(YATOM("CallEndpoint")));
 	if (ch) {
 	    DataEndpoint::commonMutex().lock();
 	    de = ch->setEndpoint();
