@@ -5,22 +5,19 @@
  * STUN support module
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  * Author: Marian Podgoreanu
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include <yatephone.h>
@@ -1195,7 +1192,7 @@ void YStunSocketFilter::processBindResult(YStunMessage* msg)
  */
 bool StunHandler::received(Message& msg)
 {
-    Socket* socket = static_cast<Socket*>(msg.userObject("Socket"));
+    Socket* socket = static_cast<Socket*>(msg.userObject(YATOM("Socket")));
     if (!socket) {
 	Debug(&iplugin,DebugGoOn,"StunHandler: No socket to install filter for.");
 	return true;
