@@ -5,21 +5,18 @@
  * Engine, plugins and messages related classes
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2006 Null Team
+ * Copyright (C) 2004-2013 Null Team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #ifndef __YATENGINE_H
@@ -69,6 +66,13 @@ public:
     inline unsigned int sections() const
 	{ return m_sections.length(); }
 
+    /**
+     * Get the number of non null sections
+     * @return Count of sections
+     */
+    inline unsigned int count() const
+	{ return m_sections.count(); }
+
     /**
      * Retrieve an entire section
      * @param index Index of the section
@@ -653,7 +657,7 @@ public:
  * No new methods are provided - we only need the multiple inheritance.
  * @short Post-dispatching message hook that can be added to a list
  */
-class YATE_API MessagePostHook : public GenObject, public MessageNotifier
+class YATE_API MessagePostHook : public RefObject, public MessageNotifier
 {
 };
 
@@ -681,6 +685,7 @@ public:
 
     /**
      * Retrieve the tracker parameter name
+     * @return Name of the parameter used to track message dispatching
      */
     inline const String& trackParam() const
 	{ return m_trackParam; }
@@ -744,7 +749,7 @@ public:
      * Clear all the message handlers and post-dispatch hooks
      */
     inline void clear()
-	{ m_handlers.clear(); m_hooks.clear(); }
+	{ m_handlers.clear(); m_hookAppend = &m_hooks; m_hooks.clear(); }
 
     /**
      * Get the number of messages waiting in the queue
@@ -758,6 +763,12 @@ public:
      */
     unsigned int handlerCount();
 
+    /**
+     * Get the number of post-handling hooks in this dispatcher
+     * @return Count of hooks
+     */
+    unsigned int postHookCount();
+
     /**
      * Install or remove a hook to catch messages after being dispatched
      * @param hook Pointer to a post-dispatching message hook
@@ -777,11 +788,140 @@ private:
     ObjList m_handlers;
     ObjList m_messages;
     ObjList m_hooks;
+    Mutex m_hookMutex;
+    ObjList* m_msgAppend;
+    ObjList* m_hookAppend;
     String m_trackParam;
     unsigned int m_changes;
     u_int64_t m_warnTime;
+    int m_hookCount;
+    bool m_hookHole;
 };
 
+/**
+ * Abstract class for message hook
+ * @short Abstract message hook
+ */
+class YATE_API MessageHook : public RefObject
+{
+public:
+    /**
+     * Try to enqueue a message to this hook's queue
+     * @param msg The message to enqueue
+     * @return True if the message was enqueued.
+     */
+    virtual bool enqueue(Message* msg) = 0;
+
+    /**
+     * Clear this hook data
+     */
+    virtual void clear() = 0;
+
+    /**
+     * Check if the given message can be inserted in this queue
+     * @param msg The message to check
+     * @return True if the message can be inserted in this queue
+     */
+    virtual bool matchesFilter(const Message& msg) = 0;
+};
+
+
+/**
+ * MessageQueue class allows to create a private queue for a message who matches
+ * the specified filters.
+ * @short A message queue
+ */
+class YATE_API MessageQueue : public MessageHook, public Mutex
+{
+    friend class Engine;
+public:
+    /**
+     * Creates a new message queue.
+     * @param hookName Name of the message served by this queue
+     * @param numWorkers The number of workers who serve this queue
+     */
+    MessageQueue(const char* hookName, int numWorkers = 0);
+
+    /**
+     * Destroys the message queue
+     */
+    ~MessageQueue();
+
+    /**
+     * Append a message in the queue
+     * @param msg The message to enqueue, will be destroyed after the processing is done
+     * @return True if successfully queued, false otherwise
+     */
+    virtual bool enqueue(Message* msg);
+
+    /**
+     * Process a message from the waiting queue
+     * @return False if the message queue is empty
+     */
+    bool dequeue();
+
+    /**
+     * Add a new filter to this queue
+     * @param name The filter name
+     * @param value The filter value
+     */
+    void addFilter(const char* name, const char* value);
+
+    /**
+     * Remove a filter form this queue
+     * @param name The filter name
+     */
+    void removeFilter(const String& name);
+
+    /**
+     * Clear private data
+     */
+    virtual void clear();
+
+    /**
+     * Remove a thread from workers list
+     * @param thread The thread to remove
+     */
+    void removeThread(Thread* thread);
+
+    /**
+     * Helper method to obtain the number of unprocessed messages in the queue
+     * @return The number of queued messages.
+     */
+    inline unsigned int count() const
+	{ return m_count; }
+
+    /**
+     * Obtain the filter list for this queue
+     * @return The filter list
+     */
+    inline const NamedList& getFilters() const
+	{ return m_filters; }
+
+    /**
+     * Check if the given message can be inserted in this queue
+     * @param msg The message to check
+     * @return True if the message can be inserted in this queue
+     */
+    virtual bool matchesFilter(const Message& msg);
+protected:
+
+    /**
+     * Callback method for message processing
+     * Default calls Engine::dispatch
+     * @param msg The message to process
+     */
+    virtual void received(Message& msg);
+
+private:
+    NamedList m_filters;
+    ObjList m_messages;
+    ObjList m_workers;
+    ObjList* m_append;
+    unsigned int m_count;
+};
+
+
 /**
  * Initialization and information about plugins.
  * Plugins are located in @em shared libraries that are loaded at runtime.
@@ -1190,9 +1330,10 @@ public:
     /**
      * Enqueue a message in the message queue for asynchronous dispatching
      * @param msg The message to enqueue, will be destroyed after dispatching
+     * @param skipHooks True to append the message directly into the main queue
      * @return True if enqueued, false on error (already queued)
      */
-    static bool enqueue(Message* msg);
+    static bool enqueue(Message* msg, bool skipHooks = false);
 
     /**
      * Convenience function.
@@ -1235,6 +1376,26 @@ public:
     inline void setHook(MessagePostHook* hook, bool remove = false)
 	{ m_dispatcher.setHook(hook,remove); }
 
+    /**
+     * Retrieve the tracker parameter name
+     * @return Name of the parameter used to track message dispatching
+     */
+    inline static const String& trackParam()
+	{ return s_self ? s_self->m_dispatcher.trackParam() : String::empty(); }
+
+    /**
+     * Appends a new message hook to the hooks list.
+     * @param hook The message hook to append.
+     * @return True if the message hook was successfully appended to the hooks list
+     */
+    static bool installHook(MessageHook* hook);
+
+    /**
+     * Remove a message hook from the hooks list.
+     * @param hook The hook to remove.
+     */
+    static void uninstallHook(MessageHook* hook);
+
     /**
      * Get a count of plugins that are actively in use
      * @return Count of plugins in use
@@ -1255,6 +1416,13 @@ public:
     inline unsigned int handlerCount()
 	{ return m_dispatcher.handlerCount(); }
 
+    /**
+     * Get the number of post-handling hooks in the dispatcher
+     * @return Count of hooks
+     */
+    inline unsigned int postHookCount()
+	{ return m_dispatcher.postHookCount(); }
+
     /**
      * Loads the plugins from an extra plugins directory or just an extra plugin
      * @param relPath Path to the extra directory, relative to the main modules
@@ -1316,6 +1484,7 @@ protected:
 private:
     Engine();
     void internalStatisticsStart();
+    void tryPluginFile(const String& name, const String& path, bool defload);
     ObjList m_libs;
     MessageDispatcher m_dispatcher;
     static Engine* s_self;
